Package: rsimsum
Title: Analysis of Simulation Studies Including Monte Carlo Error
Version: 0.13.1
Authors@R: c(
    person(
        given = "Alessandro", 
        family = "Gasparini", 
        email = "alessandro@ellessenne.xyz", 
        comment = c(ORCID = "0000-0002-8319-7624"),
        role = c("cre", "aut")),
    person(
        given = "Ian R.", 
        family = "White", 
        role = "aut"))
Description: Summarise results from simulation studies and compute Monte
    Carlo standard errors of commonly used summary statistics. This
    package is modelled on the 'simsum' user-written command in 'Stata'
    (White I.R., 2010
    <https://www.stata-journal.com/article.html?article=st0200>), further
    extending it with additional performance measures and functionality.
License: GPL (>= 3)
URL: https://ellessenne.github.io/rsimsum/,
        https://github.com/ellessenne/rsimsum
BugReports: https://github.com/ellessenne/rsimsum/issues
Depends: R (>= 2.10)
Imports: checkmate, generics, ggplot2, ggridges, knitr, lifecycle,
        rlang (>= 1.2.0), scales, stats
Suggests: covr, devtools, dplyr, eha, rmarkdown, rstpm2, survival,
        testthat, usethis
VignetteBuilder: knitr
ByteCompile: true
Encoding: UTF-8
Language: en-GB
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-21 17:39:21 UTC; ellessenne
Author: Alessandro Gasparini [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-8319-7624>),
  Ian R. White [aut]
Maintainer: Alessandro Gasparini <alessandro@ellessenne.xyz>
Repository: CRAN
Date/Publication: 2026-04-21 18:12:23 UTC
Built: R 4.6.0; ; 2026-04-26 14:55:28 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(autoplot,multisimsum)
S3method(autoplot,simsum)
S3method(autoplot,summary.multisimsum)
S3method(autoplot,summary.simsum)
S3method(kable,multisimsum)
S3method(kable,simsum)
S3method(kable,summary.multisimsum)
S3method(kable,summary.simsum)
S3method(print,multisimsum)
S3method(print,simsum)
S3method(print,summary.multisimsum)
S3method(print,summary.simsum)
S3method(summary,multisimsum)
S3method(summary,simsum)
S3method(tidy,multisimsum)
S3method(tidy,simsum)
S3method(tidy,summary.multisimsum)
S3method(tidy,summary.simsum)
export(dropbig)
export(get_data)
export(is.multisimsum)
export(is.simsum)
export(is.summary.multisimsum)
export(is.summary.simsum)
export(kable)
export(multisimsum)
export(nsim)
export(simsum)
export(tidy)
import(generics)
import(ggplot2)
import(knitr)
import(rlang)
import(scales)
import(stats)
importFrom(checkmate,assert_character)
importFrom(checkmate,assert_int)
importFrom(checkmate,assert_list)
importFrom(checkmate,assert_logical)
importFrom(checkmate,assert_number)
importFrom(checkmate,assert_string)
importFrom(checkmate,assert_subset)
importFrom(checkmate,assert_true)
importFrom(checkmate,makeAssertCollection)
importFrom(checkmate,reportAssertions)
importFrom(generics,tidy)
importFrom(ggridges,geom_density_ridges)
importFrom(lifecycle,deprecate_warn)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# {rsimsum}: Analysis of Simulation Studies Including Monte Carlo Error <img src="man/figures/hex.png" width = "200" align="right" alt="Hex logo of the {rsimsum} R package."/>

<!-- badges: start -->

[![R-CMD-check](https://github.com/ellessenne/rsimsum/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/ellessenne/rsimsum/actions/workflows/R-CMD-check.yaml)
[![Codecov Test
Coverage](https://codecov.io/gh/ellessenne/rsimsum/branch/master/graph/badge.svg)](https://app.codecov.io/gh/ellessenne/rsimsum?branch=master)
[![CRAN
Status](https://www.r-pkg.org/badges/version/rsimsum)](https://CRAN.R-project.org/package=rsimsum)
[![CRAN Logs Monthly
Downloads](https://cranlogs.r-pkg.org/badges/rsimsum)](https://CRAN.R-project.org/package=rsimsum)
[![CRAN Logs Total
Downloads](https://cranlogs.r-pkg.org/badges/grand-total/rsimsum)](https://CRAN.R-project.org/package=rsimsum)
[![JOSS
DOI](https://joss.theoj.org/papers/10.21105/joss.00739/status.svg)](https://doi.org/10.21105/joss.00739)
[![Lifecycle:
Stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
<!-- badges: end -->

`rsimsum` is an R package that can compute summary statistics from
simulation studies. `rsimsum` is modelled upon a similar package
available in Stata, the user-written command `simsum` (White I.R.,
2010).

The aim of `rsimsum` is to help to report simulation studies, including
understanding the role of chance in results of simulation studies: Monte
Carlo standard errors and confidence intervals based on them are
computed and presented to the user by default. `rsimsum` can compute a
wide variety of summary statistics: bias, empirical and model-based
standard errors, relative precision, relative error in model standard
error, mean squared error, coverage, bias. Further details on each
summary statistic are presented elsewhere (White I.R., 2010; Morris *et
al*, 2019).

The main function of `rsimsum` is called `simsum` and can handle
simulation studies with a single estimand of interest at a time. Missing
values are excluded by default, and it is possible to define boundary
values to drop estimated values or standard errors exceeding such
limits. It is possible to define a variable representing methods
compared with the simulation study, and it is possible to define *by*
factors, that is, factors that vary between the different simulated
scenarios (data-generating mechanisms, DGMs). However, methods and DGMs
are not strictly required: in that case, a simulation study with a
single scenario and a single method is assumed. Finally, `rsimsum`
provides a function named `multisimsum` that allows summarising
simulation studies with multiple estimands as well.

An important step of reporting a simulation study consists in
visualising the results; therefore, `rsimsum` exploits the R package
[`ggplot2`](https://CRAN.R-project.org/package=ggplot2) to produce a
portfolio of opinionated data visualisations for quick exploration of
results, inferring colours and facetting by data-generating mechanisms.
`rsimsum` includes methods to produce (1) plots of summary statistics
with confidence intervals based on Monte Carlo standard errors (forest
plots, lolly plots), (2) zipper plots to graphically visualise coverage
by directly plotting confidence intervals, (3) plots for method-wise
comparisons of estimates and standard errors (scatter plots,
Bland-Altman plots, ridgeline plots), and (4) heat plots. The latter is
a visualisation type that has not been traditionally used to present
results of simulation studies, and consists in a mosaic plot where the
factor on the x-axis is the methods compared with the current simulation
study and the factor on the y-axis is the data-generating factors. Each
tile of the mosaic plot is coloured according to the value of the
summary statistic of interest, with a red colour representing values
above the target value and a blue colour representing values below the
target.

## Installation

You can install `rsimsum` from CRAN:

``` r
install.packages("rsimsum")
```

Alternatively, it is possible to install the development version from
GitHub using the `remotes` package:

``` r
# install.packages("remotes")
remotes::install_github("ellessenne/rsimsum")
```

## Example

This is a basic example using data from a simulation study on missing
data (type `help("MIsim", package = "rsimsum")` in the R console for
more information):

``` r
library(rsimsum)
data("MIsim", package = "rsimsum")
s <- simsum(data = MIsim, estvarname = "b", true = 0.5, se = "se", methodvar = "method", x = TRUE)
#> 'ref' method was not specified, CC set as the reference
s
#> Summary of a simulation study with a single estimand.
#> True value of the estimand: 0.5 
#> 
#> Method variable: method 
#>  Unique methods: CC, MI_LOGT, MI_T 
#>  Reference method: CC 
#> 
#> By factors: none
#> 
#> Monte Carlo standard errors were computed.
```

We set `x = TRUE` as it will be required for some plot types.

Summarising the results:

``` r
summary(s)
#> Values are:
#>  Point Estimate (Monte Carlo Standard Error)
#> 
#> Non-missing point estimates/standard errors:
#>    CC MI_LOGT MI_T
#>  1000    1000 1000
#> 
#> Average point estimate:
#>      CC MI_LOGT   MI_T
#>  0.5168  0.5009 0.4988
#> 
#> Median point estimate:
#>      CC MI_LOGT   MI_T
#>  0.5070  0.4969 0.4939
#> 
#> Average variance:
#>      CC MI_LOGT   MI_T
#>  0.0216  0.0182 0.0179
#> 
#> Median variance:
#>      CC MI_LOGT   MI_T
#>  0.0211  0.0172 0.0169
#> 
#> Bias in point estimate:
#>               CC         MI_LOGT             MI_T
#>  0.0168 (0.0048) 0.0009 (0.0042) -0.0012 (0.0043)
#> 
#> Relative bias in point estimate:
#>               CC         MI_LOGT             MI_T
#>  0.0335 (0.0096) 0.0018 (0.0083) -0.0024 (0.0085)
#> 
#> Empirical standard error:
#>               CC         MI_LOGT            MI_T
#>  0.1511 (0.0034) 0.1320 (0.0030) 0.1344 (0.0030)
#> 
#> % gain in precision relative to method CC:
#>               CC          MI_LOGT             MI_T
#>  0.0000 (0.0000) 31.0463 (3.9375) 26.3682 (3.8424)
#> 
#> Mean squared error:
#>               CC         MI_LOGT            MI_T
#>  0.0231 (0.0011) 0.0174 (0.0009) 0.0181 (0.0009)
#> 
#> Model-based standard error:
#>               CC         MI_LOGT            MI_T
#>  0.1471 (0.0005) 0.1349 (0.0006) 0.1338 (0.0006)
#> 
#> Relative % error in standard error:
#>                CC         MI_LOGT             MI_T
#>  -2.6594 (2.2055) 2.2233 (2.3323) -0.4412 (2.2695)
#> 
#> Coverage of nominal 95% confidence interval:
#>               CC         MI_LOGT            MI_T
#>  0.9430 (0.0073) 0.9490 (0.0070) 0.9430 (0.0073)
#> 
#> Bias-eliminated coverage of nominal 95% confidence interval:
#>               CC         MI_LOGT            MI_T
#>  0.9400 (0.0075) 0.9490 (0.0070) 0.9430 (0.0073)
#> 
#> Power of 5% level test:
#>               CC         MI_LOGT            MI_T
#>  0.9460 (0.0071) 0.9690 (0.0055) 0.9630 (0.0060)
```

## Vignettes

`rsimsum` comes with 5 vignettes. In particular, check out the
introductory one:

``` r
vignette(topic = "A-introduction", package = "rsimsum")
```

The list of vignettes could be obtained by typing the following in the R
console:

``` r
vignette(package = "rsimsum")
```

## Visualising results

As of version `0.2.0`, `rsimsum` can produce a variety of plots: among
others, lolly plots, forest plots, zipper plots, etc.:

``` r
library(ggplot2)
autoplot(s, type = "lolly", stats = "bias")
```

<img src="man/figures/README-lolly-1.png" alt="" width="85%" style="display: block; margin: auto;" />

``` r
autoplot(s, type = "zip")
```

<img src="man/figures/README-zipper-1.png" alt="" width="85%" style="display: block; margin: auto;" />

With `rsimsum` `0.5.0` the plotting functionality has been completely
rewritten, and new plot types have been implemented:

- Scatter plots for method-wise comparisons, including Bland-Altman type
  plots;

``` r
autoplot(s, type = "est_ba")
#> `geom_smooth()` using formula = 'y ~ x'
```

<img src="man/figures/README-ba-1.png" alt="" width="85%" style="display: block; margin: auto;" />

- Ridgeline plots.

``` r
autoplot(s, type = "est_ridge")
#> Picking joint bandwidth of 0.0295
```

<img src="man/figures/README-ridgeline-1.png" alt="" width="85%" style="display: block; margin: auto;" />

Nested loop plots have been implemented in `rsimsum` `0.6.0`:

``` r
data("nlp", package = "rsimsum")
s.nlp <- rsimsum::simsum(
  data = nlp, estvarname = "b", true = 0, se = "se",
  methodvar = "model", by = c("baseline", "ss", "esigma")
)
#> 'ref' method was not specified, 1 set as the reference
autoplot(s.nlp, stats = "bias", type = "nlp")
```

<img src="man/figures/README-nlp-1.png" alt="" width="85%" style="display: block; margin: auto;" />

Finally, as of `rsimsum` `0.7.1` contour plots and hexbin plots have
been implemented as well:

``` r
autoplot(s, type = "est_density")
#> `geom_smooth()` using formula = 'y ~ x'
```

<img src="man/figures/README-density-1.png" alt="" width="85%" style="display: block; margin: auto;" />

``` r
autoplot(s, type = "est_hex")
#> `geom_smooth()` using formula = 'y ~ x'
```

<img src="man/figures/README-hex-1.png" alt="" width="85%" style="display: block; margin: auto;" />

They provide a useful alternative when there are several data points
with large overlap (e.g. in a scatterplot).

The plotting functionality now extend the S3 generic `autoplot`: see
`?ggplot2::autoplot` and `?rsimsum::autoplot.simsum` for further
details.

More details and information can be found in the vignettes dedicated to
plotting:

``` r
vignette(topic = "C-plotting", package = "rsimsum")
vignette(topic = "D-nlp", package = "rsimsum")
```

# Citation

If you find `rsimsum` useful, please cite it in your publications:

``` r
citation("rsimsum")
#> To cite package 'rsimsum' in publications use:
#> 
#>   Gasparini, (2018). rsimsum: Summarise results from Monte Carlo simulation studies.
#>   Journal of Open Source Software, 3(26), 739, https://doi.org/10.21105/joss.00739
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Article{,
#>     author = {Alessandro Gasparini},
#>     title = {rsimsum: Summarise results from Monte Carlo simulation studies},
#>     journal = {Journal of Open Source Software},
#>     year = {2018},
#>     volume = {3},
#>     issue = {26},
#>     pages = {739},
#>     doi = {10.21105/joss.00739},
#>     url = {https://doi.org/10.21105/joss.00739},
#>   }
```

# References

- White, I.R. 2010. *simsum: Analyses of simulation studies including
  Monte Carlo error*. The Stata Journal, 10(3): 369-385
- Morris, T.P., White, I.R. and Crowther, M.J. 2019. *Using simulation
  studies to evaluate statistical methods*. Statistics in Medicine, 38:
  2074-2102
- Gasparini, A. 2018. *rsimsum: Summarise results from Monte Carlo
  simulation studies*. Journal of Open Source Software, 3(26):739

## ----setup, include = FALSE-------------------------------------------------------------------------------------------------------------------------
options(width = 150)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.align = "center", fig.height = 6, fig.width = 6,
  out.width = "75%"
)

## ----baseline-hazards-------------------------------------------------------------------------------------------------------------------------------
exp_basehaz <- function(t, lambda = 0.5) lambda * 1 * t^0
exp_weibull <- function(t, lambda = 0.5, gamma = 1.5) lambda * gamma * t^(gamma - 1)
curve(exp_basehaz, from = 0, to = 5, lty = 1, ylim = c(0, 2), ylab = expression(h[0](t)), xlab = "Follow-up time t")
curve(exp_weibull, from = 0, to = 5, lty = 2, add = TRUE)
legend(x = "topleft", lty = 1:2, legend = c("Exponential baseline hazard", "Weibull baseline hazard"), bty = "n")

## ----dgfun------------------------------------------------------------------------------------------------------------------------------------------
simulate_data <- function(dataset, n, baseline, params = list(), coveff = -0.50) {
  # Simulate treatment indicator variable
  x <- rbinom(n = n, size = 1, prob = 0.5)
  # Draw from a U(0,1) random variable
  u <- runif(n)
  # Simulate survival times depending on the baseline hazard
  if (baseline == "Exponential") {
    t <- -log(u) / (params$lambda * exp(x * coveff))
  } else {
    t <- (-log(u) / (params$lambda * exp(x * coveff)))^(1 / params$gamma)
  }
  # Winsorising tiny values for t (smaller than one day on a yearly-scale, e.g. 1 / 365.242), and adding a tiny amount of white noise not to have too many concurrent values
  t <- ifelse(t < 1 / 365.242, 1 / 365.242, t)
  t[t == 1 / 365.242] <- t[t == 1 / 365.242] + rnorm(length(t[t == 1 / 365.242]), mean = 0, sd = 1e-4)
  # ...and make sure that the resulting value is positive
  t <- abs(t)

  # Make event indicator variable applying administrative censoring at t = 5
  d <- as.numeric(t < 5)
  t <- pmin(t, 5)
  # Return a data.frame
  data.frame(dataset = dataset, x = x, t = t, d = d, n = n, baseline = baseline, stringsAsFactors = FALSE)
}

## ----set-seed---------------------------------------------------------------------------------------------------------------------------------------
set.seed(755353002)

## ----generate-data----------------------------------------------------------------------------------------------------------------------------------
reps <- 1:100
data <- list()
data[["n = 50, baseline = Exp"]] <- lapply(
  X = reps,
  FUN = simulate_data,
  n = 50,
  baseline = "Exponential",
  params = list(lambda = 0.5)
)
data[["n = 250, baseline = Exp"]] <- lapply(
  X = reps,
  FUN = simulate_data,
  n = 250,
  baseline = "Exponential",
  params = list(lambda = 0.5)
)
data[["n = 50, baseline = Wei"]] <- lapply(
  X = reps,
  FUN = simulate_data,
  n = 50,
  baseline = "Weibull",
  params = list(lambda = 0.5, gamma = 1.5)
)
data[["n = 250, baseline = Wei"]] <- lapply(
  X = reps,
  FUN = simulate_data,
  n = 250,
  baseline = "Weibull",
  params = list(lambda = 0.5, gamma = 1.5)
)

## ----fitting-function-------------------------------------------------------------------------------------------------------------------------------
library(survival)
library(rstpm2)
library(eha)

fit_models <- function(data, model) {
  # Fit model
  if (model == "Cox") {
    fit <- survival::coxph(Surv(t, d) ~ x, data = data)
  } else if (model == "RP(2)") {
    fit <- rstpm2::stpm2(Surv(t, d) ~ x, data = data, df = 2)
  } else {
    fit <- eha::phreg(Surv(t, d) ~ x, data = data, dist = "weibull", shape = 1)
  }
  # Return relevant coefficients
  data.frame(
    dataset = unique(data$dataset),
    n = unique(data$n),
    baseline = unique(data$baseline),
    theta = coef(fit)["x"],
    se = sqrt(ifelse(model == "Exp", fit$var["x", "x"], vcov(fit)["x", "x"])),
    model = model,
    stringsAsFactors = FALSE,
    row.names = NULL
  )
}

## ----run-models-------------------------------------------------------------------------------------------------------------------------------------
results <- list()
results[["n = 50, baseline = Exp, model = Cox"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 50, baseline = Exp"]],
    FUN = fit_models,
    model = "Cox"
  )
)
results[["n = 250, baseline = Exp, model = Cox"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 250, baseline = Exp"]],
    FUN = fit_models,
    model = "Cox"
  )
)
results[["n = 50, baseline = Wei, model = Cox"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 50, baseline = Wei"]],
    FUN = fit_models,
    model = "Cox"
  )
)
results[["n = 250, baseline = Wei, model = Cox"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 250, baseline = Wei"]],
    FUN = fit_models,
    model = "Cox"
  )
)

results[["n = 50, baseline = Exp, model = Exp"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 50, baseline = Exp"]],
    FUN = fit_models,
    model = "Exp"
  )
)
results[["n = 250, baseline = Exp, model = Exp"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 250, baseline = Exp"]],
    FUN = fit_models,
    model = "Exp"
  )
)
results[["n = 50, baseline = Wei, model = Exp"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 50, baseline = Wei"]],
    FUN = fit_models,
    model = "Exp"
  )
)
results[["n = 250, baseline = Wei, model = Exp"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 250, baseline = Wei"]],
    FUN = fit_models,
    model = "Exp"
  )
)

results[["n = 50, baseline = Exp, model = RP(2)"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 50, baseline = Exp"]],
    FUN = fit_models,
    model = "RP(2)"
  )
)
results[["n = 250, baseline = Exp, model = RP(2)"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 250, baseline = Exp"]],
    FUN = fit_models,
    model = "RP(2)"
  )
)
results[["n = 50, baseline = Wei, model = RP(2)"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 50, baseline = Wei"]],
    FUN = fit_models,
    model = "RP(2)"
  )
)
results[["n = 250, baseline = Wei, model = RP(2)"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 250, baseline = Wei"]],
    FUN = fit_models,
    model = "RP(2)"
  )
)

## ----aggregate-results------------------------------------------------------------------------------------------------------------------------------
relhaz <- do.call(
  rbind.data.frame,
  results
)
row.names(relhaz) <- NULL

## ----saving, eval = FALSE---------------------------------------------------------------------------------------------------------------------------
# library(usethis)
# usethis::use_data(relhaz, overwrite = TRUE)

## ----compute-summaries------------------------------------------------------------------------------------------------------------------------------
library(rsimsum)
s <- rsimsum::simsum(data = relhaz, estvarname = "theta", se = "se", true = -0.50, methodvar = "model", ref = "Cox", by = c("n", "baseline"))
s

## ----print-summaries--------------------------------------------------------------------------------------------------------------------------------
summary(s)

#' @title print.summary.simsum
#' @description Print method for `summary.simsum` objects
#' @param x An object of class `summary.simsum`.
#' @param digits Number of significant digits used for printing.
#'     Defaults to 4.
#' @param mcse Should Monte Carlo standard errors be reported?
#'     If `mcse = FALSE`, confidence intervals based on Monte Carlo standard errors will be reported instead, see [summary.simsum()].
#'     If a `NULL` value is passed, only point estimates are printed regardless of whether Monte Carlo standard errors were computed or not.
#'     Defaults to `TRUE`.
#' @param ... Ignored.
#' @export
#'
#' @examples
#' data("MIsim")
#' x <- simsum(
#'   data = MIsim, estvarname = "b", true = 0.5, se = "se",
#'   methodvar = "method"
#' )
#' xs <- summary(x)
#' xs
#'
#' # Printing less significant digits:
#' print(xs, digits = 2)
#'
#' # Printing confidence intervals:
#' print(xs, mcse = FALSE)
#'
#' # Printing values only:
#' print(xs, mcse = NULL)
print.summary.simsum <- function(x, digits = 4, mcse = TRUE, ...) {
  ### Check arguments
  arg_checks <- checkmate::makeAssertCollection()
  # `digits` must be an integer value greater than or equal to zero
  checkmate::assert_int(x = digits, lower = 0, upper = Inf, add = arg_checks)
  checkmate::assert_logical(x = mcse, len = 1, null.ok = TRUE, add = arg_checks)
  ### Report if there are any errors
  if (!arg_checks$isEmpty()) {
    checkmate::reportAssertions(arg_checks)
  }

  ### Make sure users are not asking for the moon
  if (!x$control$mcse) {
    mcse <- NULL
    message(
      "Monte Carlo Standard Errors were not computed!\nDisplaying point estimates only."
    )
  }
  if (is.null(mcse)) {
    cat("Values are:\n\tPoint Estimate\n")
  } else if (mcse) {
    cat("Values are:\n\tPoint Estimate (Monte Carlo Standard Error)\n")
  } else {
    cat(paste0(
      "Values are:\n\tPoint Estimate (",
      sprintf("%.0f%%", 100 * (x$ci_level)),
      " Confidence Interval based on Monte Carlo Standard Errors)\n"
    ))
  }

  ### Format summary table
  x <- .format(x = x, digits = digits, mcse = mcse)

  ### Make names of the summary table
  names(x$summ)[names(x$summ) == "description"] <- "Performance Measure"
  names(x$summ)[names(x$summ) == "est"] <- "Estimate"

  ### Order data.frame with results
  x$summ <- .order(
    data = x$summ,
    by = c("Performance Measure", x$methodvar, x$by)
  )

  ### If length(methodvar) > 1 then process multiple columns into one...
  if (length(x$methodvar) > 1) {
    x$summ <- .compact_method_columns(
      data = x$summ,
      methodvar = x$methodvar
    )$data
    x$methodvar <- ":methodvar"
  }

  ### If methodvar, put them side by side
  if (!is.null(x$methodvar)) {
    x$summ <- .bind_methods(data = x$summ, by = x$by, methodvar = x$methodvar)
  }

  ### Split by summary statistics for printing
  x$summ[["Performance Measure"]] <- droplevels(x$summ[["Performance Measure"]])
  output <- .split_by(data = x$summ, by = "Performance Measure")

  ### Print
  for (i in seq_along(output)) {
    cat(paste0("\n", names(output)[i], ":\n"))
    output[[i]][["Performance Measure"]] <- NULL
    print(output[[i]], row.names = FALSE)
  }

  ### Return output invisibly
  output <- lapply(X = output, FUN = function(x) {
    row.names(x) <- NULL
    return(x)
  })
  return(invisible(output))
}

#' @title get_data
#' @description Extract data slots from an object of class `simsum`, `summary.simsum`, `multisimsum`, or `summary.multisimsum`.
#' @param x An object of class `simsum`.
#' @param stats Summary statistics to include; can be a scalar value or a vector. Possible choices are:
#' * `nsim`, the number of replications with non-missing point estimates and standard error.
#' * `thetamean`, average point estimate.
#' * `thetamedian`, median point estimate.
#' * `se2mean`, average standard error.
#' * `se2median`, median standard error.
#' * `bias`, bias in point estimate.
#' * `empse`, empirical standard error.
#' * `mse`, mean squared error.
#' * `relprec`, percentage gain in precision relative to the reference method.
#' * `modelse`, model-based standard error.
#' * `relerror`, relative percentage error in standard error.
#' * `cover`, coverage of a nominal `level`\% confidence interval.
#' * `becover`, bias-eliminated coverage of a nominal `level`\% confidence interval.
#' * `power`, power of a (1 - `level`)\% level test.
#' Defaults to `NULL`, in which case all summary statistics are returned.
#' @param ... Ignored.
#' @return A `data.frame` containing summary statistics from a simulation study.
#' @export
#'
#' @importFrom lifecycle deprecate_warn
#'
#' @examples
#' data(MIsim)
#' x <- simsum(
#'   data = MIsim, estvarname = "b", true = 0.5, se = "se",
#'   methodvar = "method"
#' )
#' get_data(x)
#'
#' # Extracting only bias and coverage:
#' get_data(x, stats = c("bias", "cover"))
#'
#' xs <- summary(x)
#' get_data(xs)
get_data <- function(x, stats = NULL, ...) {
  ### Deprecated
  lifecycle::deprecate_warn(
    when = "0.10.0",
    what = "get_data()",
    with = "tidy()"
  )

  ### Check arguments
  arg_checks <- checkmate::makeAssertCollection()

  # 'x' must be an object of class 'simsum', 'summary.simsum', 'multisimsum', 'summary.multisimsum' (any)
  checkmate::assert_true(
    x = inherits(
      x = x,
      what = c("simsum", "summary.simsum", "multisimsum", "summary.multisimsum")
    ),
    add = arg_checks
  )

  # 'stats' must be one of the possible choices
  checkmate::assert_character(x = stats, null.ok = TRUE, add = arg_checks)
  checkmate::assert_subset(
    x = stats,
    choices = c(
      "nsim",
      "thetamean",
      "thetamedian",
      "se2mean",
      "se2median",
      "bias",
      "empse",
      "mse",
      "relprec",
      "modelse",
      "relerror",
      "cover",
      "becover",
      "power"
    ),
    add = arg_checks
  )

  ### Report if there are any errors
  if (!arg_checks$isEmpty()) {
    checkmate::reportAssertions(arg_checks)
  }

  ### Select only summary statistics on interest
  if (!is.null(stats)) {
    x$summ <- x$summ[x$summ$stat %in% stats, ]
  }

  ### Remove row.names
  row.names(x$summ) <- NULL

  ### Return data
  return(x$summ)
}

testthat::context("get_data")

testthat::test_that("get_data returns a data.frame", {
  suppressWarnings({
    data("MIsim", package = "rsimsum")
    x <- simsum(
      data = MIsim,
      estvarname = "b",
      true = 0.5,
      se = "se",
      methodvar = "method"
    )
    testthat::expect_s3_class(object = get_data(x), class = "data.frame")
    xs <- summary(x)
    testthat::expect_s3_class(object = get_data(xs), class = "data.frame")
    data("frailty", package = "rsimsum")
    ms <- multisimsum(
      data = frailty,
      par = "par",
      true = c(trt = -0.50, fv = 0.75),
      estvarname = "b",
      se = "se",
      methodvar = "model",
      by = "fv_dist"
    )
    testthat::expect_s3_class(object = get_data(ms), class = "data.frame")
    sms <- summary(ms)
    testthat::expect_s3_class(object = get_data(sms), class = "data.frame")
  })
})

testthat::test_that("get_data with 'stats' results only 'stats'", {
  suppressWarnings({
    data("MIsim", package = "rsimsum")
    x <- simsum(
      data = MIsim,
      estvarname = "b",
      true = 0.5,
      se = "se",
      methodvar = "method"
    )
    out <- rsimsum::get_data(x, stats = "bias")
    testthat::expect_true(object = all(out$stat == "bias"))
    out <- rsimsum::get_data(x, stats = c("bias", "cover"))
    testthat::expect_true(object = all(out$stat %in% c("bias", "cover")))
  })
})

testthat::test_that("get_data with wrong 'stats' throws an error", {
  suppressWarnings({
    data("MIsim", package = "rsimsum")
    x <- simsum(
      data = MIsim,
      estvarname = "b",
      true = 0.5,
      se = "se",
      methodvar = "method"
    )
    testthat::expect_error(object = rsimsum::get_data(x, stats = "42"))
    testthat::expect_error(object = rsimsum::get_data(x, stats = TRUE))
    testthat::expect_error(object = rsimsum::get_data(x, stats = 42))
  })
})

#' @title Compute number of simulations required
#' @description The function `nsim` computes the number of simulations \eqn{B}{B} to perform based on the accuracy of an estimate of interest, using the following equation:
#' \deqn{B = \left( \frac{(Z_{1 - \alpha / 2} + Z_{1 - theta}) \sigma}{\delta} \right) ^ 2,}{B = [((Z(1 - \alpha / 2) + Z(1 - \theta)) \sigma) / \delta] ^ 2 }
#' where \eqn{\delta} is the specified level of accuracy of the estimate of interest you are willing to accept (i.e. the permissible difference from the true value \eqn{\beta}{\beta}), \eqn{Z_{1 - \alpha / 2}}{Z(1 - \alpha / 2)} is the \eqn{(1 - \alpha / 2)}{(1 - \alpha / 2)} quantile of the standard normal distribution, \eqn{Z_{1 - \theta}}{Z(1 - \theta)} is the \eqn{(1 - \theta)}{(1 - \theta)} quantile of the standard normal distribution with \eqn{(1 - \theta)}{1 - \theta} being the power to detect a specific difference from the true value as significant, and \eqn{\sigma ^ 2}{\sigma ^ 2]} is the variance of the parameter of interest.
#' @param alpha Significance level. Must be a value between 0 and 1.
#' @param sigma Variance for the parameter of interest. Must be greater than 0.
#' @param delta Specified level of accuracy of the estimate of interest you are willing to accept. Must be greater than 0.
#' @param power Power to detect a specific difference from the true value as significant. Must be a value between 0 and 1. Defaults to 0.5, e.g. a power of 50%.
#' @return A scalar value \eqn{B}{B} representing the number of simulations to perform based on the accuracy required.
#' @export
#' @references Burton, A., Douglas G. Altman, P. Royston. et al. 2006. The design of simulation studies in medical statistics. Statistics in Medicine 25: 4279-4292 \doi{10.1002/sim.2673}
#'
#' @examples
#' # Number of simulations required to produce an estimate to within 5%
#' # accuracy of the true coefficient of 0.349 with a 5% significance level,
#' # assuming the variance of the estimate is 0.0166 and 50% power:
#' nsim(alpha = 0.05, sigma = sqrt(0.0166), delta = 0.349 * 5 / 100, power = 0.5)
#'
#' # Number of simulations required to produce an estimate to within 1%
#' # accuracy of the true coefficient of 0.349 with a 5% significance level,
#' # assuming the variance of the estimate is 0.0166 and 50% power:
#' nsim(alpha = 0.05, sigma = sqrt(0.0166), delta = 0.349 * 1 / 100, power = 0.5)
nsim <- function(alpha, sigma, delta, power = 0.5) {
  ### Check arguments
  arg_checks <- checkmate::makeAssertCollection()

  # 'alpha' and 'power' must be a numeric value between 0 and 1
  checkmate::assert_number(alpha, lower = 0, upper = 1, add = arg_checks)
  checkmate::assert_number(power, lower = 0, upper = 1, add = arg_checks)

  # 'sigma' and 'delta' must be a numeric value
  checkmate::assert_number(sigma, lower = 0, add = arg_checks)
  checkmate::assert_number(delta, lower = 0, add = arg_checks)

  ### Report if there are any errors
  if (!arg_checks$isEmpty()) {
    checkmate::reportAssertions(arg_checks)
  }

  ### Compute B
  B <- (((stats::qnorm(1 - alpha / 2) + stats::qnorm(power)) * sigma) / delta)^2
  return(B)
}

testthat::context("nsim")

testthat::test_that("nsim returns a single, double value", {
  testthat::expect_length(
    nsim(alpha = 0.05, sigma = 1, delta = 1, power = 0.5),
    n = 1
  )
  testthat::expect_type(
    nsim(alpha = 0.05, sigma = 1, delta = 1, power = 0.5),
    type = "double"
  )
})

testthat::test_that("alpha is in the appropriate range", {
  testthat::expect_error(
    nsim(alpha = 1.5, sigma = 1, delta = 1, power = 0.5),
    "Variable 'alpha': .+ <= 1",
    fixed = FALSE
  )
  testthat::expect_error(
    nsim(alpha = -0.1, sigma = 1, delta = 1, power = 0.5),
    "Variable 'alpha': .+ >= 0",
    fixed = FALSE
  )
})

testthat::test_that("sigma is greater than zero", {
  testthat::expect_error(
    nsim(alpha = 0.05, sigma = -0.1, delta = 1, power = -0.1),
    "Variable 'sigma': .+ >= 0",
    fixed = FALSE
  )
})

testthat::test_that("delta is greater than zero", {
  testthat::expect_error(
    nsim(alpha = 0.05, sigma = 1, delta = -0.1, power = -0.1),
    "Variable 'delta': .+ >= 0",
    fixed = FALSE
  )
})

testthat::test_that("power is in the appropriate range", {
  testthat::expect_error(
    nsim(alpha = 0.05, sigma = 1, delta = 1, power = 1.5),
    "Variable 'power': .+ <= 1",
    fixed = FALSE
  )
  testthat::expect_error(
    nsim(alpha = 0.05, sigma = 1, delta = 1, power = -0.1),
    "Variable 'power': .+ >= 0",
    fixed = FALSE
  )
})

#' @title is.simsum
#' @description Reports whether x is a simsum object
#' @param x An object to test.
#' @export
is.simsum <- function(x) {
  inherits(x, "simsum")
}

#' @title is.summary.simsum
#' @description Reports whether x is a summary.simsum object
#' @param x An object to test.
#' @export
is.summary.simsum <- function(x) {
  inherits(x, "summary.simsum")
}

#' @title is.multisimsum
#' @description Reports whether x is a multisimsum object
#' @param x An object to test.
#' @export
is.multisimsum <- function(x) {
  inherits(x, "multisimsum")
}

#' @title is.summary.multisimsum
#' @description Reports whether x is a summary.multisimsum object
#' @param x An object to test.
#' @export
is.summary.multisimsum <- function(x) {
  inherits(x, "summary.multisimsum")
}

testthat::context("is.simsum")

testthat::test_that("is.simsum works as expected", {
  data("MIsim", package = "rsimsum")
  x <- rsimsum::simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  testthat::expect_true(object = is.simsum(x))
  testthat::expect_false(object = is.summary.simsum(x))
  testthat::expect_false(object = is.simsum(MIsim))
  testthat::expect_false(object = is.multisimsum(x))
})
testthat::test_that("is.summary.simsum works as expected", {
  data("MIsim", package = "rsimsum")
  x <- rsimsum::simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  sx <- summary(x)
  testthat::expect_true(object = is.summary.simsum(sx))
  testthat::expect_false(object = is.simsum(sx))
  testthat::expect_false(object = is.multisimsum(sx))
  testthat::expect_false(object = is.summary.multisimsum(sx))
})

testthat::context("is.multisimsum")
testthat::test_that("is.multisimsum works as expected", {
  data("frailty", package = "rsimsum")
  x <- rsimsum::multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist"
  )
  testthat::expect_true(object = is.multisimsum(x))
  testthat::expect_false(object = is.multisimsum(frailty))
  testthat::expect_false(object = is.simsum(x))
})
testthat::test_that("is.summary.multisimsum works as expected", {
  data("frailty", package = "rsimsum")
  x <- rsimsum::multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist"
  )
  sx <- summary(x)
  testthat::expect_true(object = is.summary.multisimsum(sx))
  testthat::expect_false(object = is.simsum(sx))
  testthat::expect_false(object = is.multisimsum(sx))
  testthat::expect_false(object = is.summary.simsum(sx))
})

#' @title print.summary.multisimsum
#' @description Print method for `summary.multisimsum` objects
#' @param x An object of class `summary.multisimsum`.
#' @param digits Number of significant digits used for printing.
#'     Defaults to 4.
#' @param mcse Should Monte Carlo standard errors be reported?
#'     If `mcse = FALSE`, confidence intervals based on Monte Carlo standard errors will be reported instead, see [summary.multisimsum()].
#'     If a `NULL` value is passed, only point estimates are printed regardless of whether Monte Carlo standard errors were computed or not.
#'     Defaults to `TRUE`.
#' @param ... Ignored.
#' @export
#'
#' @examples
#' data(frailty)
#' ms <- multisimsum(
#'   data = frailty, par = "par", true = c(
#'     trt = -0.50,
#'     fv = 0.75
#'   ), estvarname = "b", se = "se", methodvar = "model",
#'   by = "fv_dist"
#' )
#' sms <- summary(ms, stats = c("bias", "cover", "mse"))
#' sms
#'
#' # Printing less significant digits:
#' print(sms, digits = 3)
#'
#' # Printing confidence intervals:
#' print(sms, digits = 3, mcse = FALSE)
#'
#' # Printing values only:
#' print(sms, mcse = NULL)
print.summary.multisimsum <- function(x, digits = 4, mcse = TRUE, ...) {
  ### Check arguments
  arg_checks <- checkmate::makeAssertCollection()
  # `digits` must be an integer value greater than or equal to zero
  checkmate::assert_int(x = digits, lower = 0, upper = Inf, add = arg_checks)
  checkmate::assert_logical(x = mcse, len = 1, null.ok = TRUE, add = arg_checks)
  ### Report if there are any errors
  if (!arg_checks$isEmpty()) {
    checkmate::reportAssertions(arg_checks)
  }

  ### Make sure users are not asking for the moon
  if (!x$control$mcse) {
    mcse <- NULL
    message(
      "Monte Carlo Standard Errors were not computed!\nDisplaying point estimates only."
    )
  }
  if (is.null(mcse)) {
    cat("Values are:\n\tPoint Estimate\n")
  } else if (mcse) {
    cat("Values are:\n\tPoint Estimate (Monte Carlo Standard Error)\n")
  } else {
    cat(paste0(
      "Values are:\n\tPoint Estimate (",
      sprintf("%.0f%%", 100 * (x$ci_level)),
      " Confidence Interval based on Monte Carlo Standard Errors)\n"
    ))
  }

  ### Format summary table
  x <- .format(x = x, digits = digits, mcse = mcse)

  ### Make names of the summary table
  names(x$summ)[names(x$summ) == "description"] <- "Performance Measure"
  names(x$summ)[names(x$summ) == "est"] <- "Estimate"

  ### Order data.frame with results
  x$summ <- .order(
    data = x$summ,
    by = c("Performance Measure", x$par, x$methodvar, x$by)
  )

  ### If length(methodvar) > 1 then process multiple columns into one...
  if (length(x$methodvar) > 1) {
    x$summ <- .compact_method_columns(
      data = x$summ,
      methodvar = x$methodvar
    )$data
    x$methodvar <- ":methodvar"
  }

  ### Split summary table by parameter
  par_split <- .split_by(data = x$summ, by = x$par)

  ### Create a list of printed stuff to invisibly return
  full_output <- vector(mode = "list", length = length(par_split))
  names(full_output) <- names(par_split)

  ### Loop printing by parameter
  for (i in seq_along(par_split)) {
    cat(paste(
      "\n",
      ifelse(
        i > 1,
        paste(rep("-", times = options()$width), collapse = ""),
        ""
      ),
      ifelse(i > 1, "\n", ""),
      "\nParameter:",
      names(par_split)[i],
      "\n"
    ))
    par_split[[i]][[x$par]] <- NULL

    ### If methodvar, put them side by side
    if (!is.null(x$methodvar)) {
      par_split[[i]] <- .bind_methods(
        data = par_split[[i]],
        by = x$by,
        methodvar = x$methodvar
      )
    }

    ### Split by summary statistics for printing
    par_split[[i]][["Performance Measure"]] <- droplevels(par_split[[i]][[
      "Performance Measure"
    ]])
    output <- .split_by(data = par_split[[i]], by = "Performance Measure")

    for (ii in seq_along(output)) {
      cat(paste0("\n", names(output)[ii], ":\n"))
      output[[ii]][["Performance Measure"]] <- NULL
      print(output[[ii]], row.names = FALSE)
    }

    ### Add to output to return invisibly
    full_output[[i]] <- lapply(X = output, FUN = function(x) {
      row.names(x) <- NULL
      return(x)
    })
  }
  return(invisible(full_output))
}

#' @keywords internal
.compact_method_columns <- function(data, methodvar) {
  reftable <- do.call(
    expand.grid,
    lapply(methodvar, FUN = function(x) unique(data[[x]]))
  )
  names(reftable) <- methodvar
  reftable[[":methodvar"]] <- do.call(paste, c(as.list(reftable), sep = ":"))
  data[[".nr"]] <- seq(nrow(data))
  data <- merge(data, reftable, by = methodvar)
  data <- data[order(data[[".nr"]]), ]
  data[, methodvar] <- NULL
  data[, ".nr"] <- NULL
  return(list(data = data, reftable = reftable))
}

#' @title Create 'kable's
#'
#' @description Create tables in LaTeX, HTML, Markdown, or reStructuredText from objects of class `simsum`, `summary.simsum`, `multisimsum`, `summary.multisimsum`.
#'
#' @param x An object of class `simsum`, `summary.simsum`, `multisimsum`, `summary.multisimsum`;
#' @param stats Summary statistics to include. See [tidy()] for more details;
#' @param digits Maximum number of digits for numeric columns;
#' @param ... Further arguments passed to [knitr::kable()].
#' @seealso [knitr::kable()]
#' @export
kable.simsum <- function(
  x,
  stats = NULL,
  digits = max(3, getOption("digits") - 3),
  ...
) {
  # Get dataset
  out <- tidy(x, stats = stats)
  # Pass dataset to knitr::kable
  knitr::kable(x = out, digits = digits, ...)
}

#' @rdname kable.simsum
#' @export
kable.summary.simsum <- function(
  x,
  stats = NULL,
  digits = max(3, getOption("digits") - 3),
  ...
) {
  kable.simsum(x = x, stats = stats, digits = digits, ...)
}

#' @rdname kable.simsum
#' @export
kable.multisimsum <- function(
  x,
  stats = NULL,
  digits = max(3, getOption("digits") - 3),
  ...
) {
  kable.simsum(x = x, stats = stats, digits = digits, ...)
}

#' @rdname kable.simsum
#' @export
kable.summary.multisimsum <- function(
  x,
  stats = NULL,
  digits = max(3, getOption("digits") - 3),
  ...
) {
  kable.simsum(x = x, stats = stats, digits = digits, ...)
}

#' @rdname kable.simsum
#' @export
kable <- function(x, ...) {
  UseMethod("kable", x)
}

testthat::context("kable")

testthat::test_that("Output is of class 'knitr_kable' for simsum", {
  s <- simsum(data = tt, estvarname = "diff", se = "se", true = -1)
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )

  s <- simsum(
    data = tt,
    estvarname = "diff",
    methodvar = "method",
    se = "se",
    true = -1
  )
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )

  s <- simsum(data = tt, estvarname = "diff", by = "dgm", se = "se", true = -1)
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )

  s <- simsum(
    data = tt,
    estvarname = "diff",
    methodvar = "method",
    by = "dgm",
    se = "se",
    true = -1
  )
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )
})

testthat::test_that("Output is of class 'knitr_kable' for summary.simsum", {
  s <- simsum(data = tt, estvarname = "diff", se = "se", true = -1)
  s <- summary(s)
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )

  s <- simsum(
    data = tt,
    estvarname = "diff",
    methodvar = "method",
    se = "se",
    true = -1
  )
  s <- summary(s)
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )

  s <- simsum(data = tt, estvarname = "diff", by = "dgm", se = "se", true = -1)
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )

  s <- simsum(
    data = tt,
    estvarname = "diff",
    methodvar = "method",
    by = "dgm",
    se = "se",
    true = -1
  )
  s <- summary(s)
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )
})

testthat::test_that("Output is of class 'knitr_kable' for multisimsum", {
  data("frailty", package = "rsimsum")
  s <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se"
  )
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )

  data("frailty", package = "rsimsum")
  s <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model"
  )
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )

  data("frailty", package = "rsimsum")
  s <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    by = "fv_dist"
  )
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )

  data("frailty", package = "rsimsum")
  s <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist"
  )
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )
})

testthat::test_that("Output is of class 'knitr_kable' for summary.multisimsum", {
  data("frailty", package = "rsimsum")
  s <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se"
  )
  s <- summary(s)
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )

  data("frailty", package = "rsimsum")
  s <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model"
  )
  s <- summary(s)
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )

  data("frailty", package = "rsimsum")
  s <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    by = "fv_dist"
  )
  s <- summary(s)
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )

  data("frailty", package = "rsimsum")
  s <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist"
  )
  s <- summary(s)
  testthat::expect_s3_class(object = kable(s), class = "knitr_kable")
  testthat::expect_s3_class(
    object = kable(s, digits = 2),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, format = "latex"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = "bias"),
    class = "knitr_kable"
  )
  testthat::expect_s3_class(
    object = kable(s, stats = c("bias", "cover")),
    class = "knitr_kable"
  )
})

#' @title autoplot method for summary.multisimsum objects
#'
#' @param object An object of class `summary.multisimsum`.
#' @inheritParams autoplot.multisimsum
#'
#' @return A `ggplot` object.
#' @export
#'
#' @examples
#' data("frailty", package = "rsimsum")
#' ms <- multisimsum(
#'   data = frailty,
#'   par = "par", true = c(trt = -0.50, fv = 0.75),
#'   estvarname = "b", se = "se", methodvar = "model",
#'   by = "fv_dist", x = TRUE
#' )
#' sms <- summary(ms)
#'
#' library(ggplot2)
#' autoplot(sms, par = "trt")
autoplot.summary.multisimsum <- function(
  object,
  par,
  type = "forest",
  stats = "nsim",
  target = NULL,
  fitted = TRUE,
  scales = "fixed",
  top = TRUE,
  density.legend = TRUE,
  zoom = 1,
  zip_ci_colours = "yellow",
  ...
) {
  autoplot.multisimsum(
    object = object,
    par = par,
    type = type,
    stats = stats,
    target = target,
    fitted = fitted,
    scales = scales,
    top = top,
    density.legend = density.legend,
    zoom = zoom,
    zip_ci_colours = zip_ci_colours
  )
}

#' @import generics ggplot2 knitr rlang scales stats
#' @importFrom checkmate assert_character
#' @importFrom checkmate assert_int
#' @importFrom checkmate assert_list
#' @importFrom checkmate assert_logical
#' @importFrom checkmate assert_number
#' @importFrom checkmate assert_string
#' @importFrom checkmate assert_subset
#' @importFrom checkmate assert_true
#' @importFrom checkmate makeAssertCollection
#' @importFrom checkmate reportAssertions

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

# Quiets concerns of R CMD check re: variable names used internally
if (getRversion() >= "2.15.1") {
  utils::globalVariables(c(
    ".scenario",
    ".dgm",
    "X",
    "Y",
    "contrast",
    "cover_lower",
    "cover_upper",
    "covering",
    "est",
    "lower",
    "upper",
    "level"
  ))
}

### Split 'data' by what is in 'by'
#' @keywords internal
.split_by <- function(data, by) {
  if (!is.null(by)) {
    out <- split(
      x = data,
      f = lapply(X = by, FUN = function(f) data[[f]]),
      sep = "~"
    )
  } else {
    out <- list(data)
  }
  return(out)
}

### Bind rows of data.frame objects contained in 'x', with 'x' generally a list
#' @keywords internal
.br <- function(x) do.call(rbind.data.frame, x)

### Factorise all 'cols' in 'data' (if not already a factor)
#' @keywords internal
.factorise <- function(data, cols) {
  for (c in cols) {
    if (!is.factor(data[[c]])) data[[c]] <- factor(data[[c]])
  }
  return(data)
}

### Validate levels of 'cols' in 'data', cannot contain the character 'char'
#' @keywords internal
.validate_levels <- function(data, cols, char) {
  for (c in cols) {
    if (any(grepl(char, levels(data[[c]])))) {
      stop(paste0(
        "Levels of the column '",
        c,
        "' contain the forbidden character '",
        char,
        "'"
      ))
    }
  }
}

### Set to NA if standardised values of 'estvarname' or 'se' > 'max' (grouped by 'methodvar', 'by') using either normal ((x - mean) / stdev) or robust ((x - median) / IQR) standardisation
#' @keywords internal
.dropbig <- function(
  data,
  estvarname,
  se = NULL,
  methodvar,
  by,
  max,
  semax,
  robust,
  internal = TRUE
) {
  splt_c <- c(methodvar, by)
  if (length(splt_c) > 0) {
    idata <- .split_by(data, splt_c)
  } else {
    idata <- list(data)
  }
  odata <- lapply(seq_along(idata), function(i) {
    tmp <- idata[[i]]
    if (robust) {
      tmp[[paste0(".", estvarname, ".std")]] <- (tmp[[estvarname]] -
        median(tmp[[estvarname]], na.rm = TRUE)) /
        (fivenum(tmp[[estvarname]], na.rm = TRUE)[4] -
          fivenum(tmp[[estvarname]], na.rm = TRUE)[2])
      if (!is.null(se)) {
        tmp[[paste0(".", se, ".std")]] <- (tmp[[se]] -
          median(tmp[[se]], na.rm = TRUE)) /
          (fivenum(tmp[[se]], na.rm = TRUE)[4] -
            fivenum(tmp[[se]], na.rm = TRUE)[2])
      }
    } else {
      tmp[[paste0(".", estvarname, ".std")]] <- (tmp[[estvarname]] -
        mean(tmp[[estvarname]], na.rm = TRUE)) /
        sqrt(var(tmp[[estvarname]], na.rm = TRUE))
      if (!is.null(se)) {
        tmp[[paste0(".", se, ".std")]] <- (tmp[[se]] -
          mean(tmp[[se]], na.rm = TRUE)) /
          sqrt(var(tmp[[se]], na.rm = TRUE))
      }
    }
    tmp
  })
  odata <- .br(odata)
  if (internal) {
    odata[[estvarname]] <- ifelse(
      abs(odata[[paste0(".", estvarname, ".std")]]) > max,
      NA,
      odata[[estvarname]]
    )
    odata[[paste0(".", estvarname, ".std")]] <- NULL
    if (!is.null(se)) {
      odata[[se]] <- ifelse(
        abs(odata[[paste0(".", se, ".std")]]) > semax,
        NA,
        odata[[se]]
      )
      odata[[paste0(".", se, ".std")]] <- NULL
    }
  } else {
    if (!is.null(se)) {
      odata[[".dropbig"]] <- ifelse(
        abs(odata[[paste0(".", estvarname, ".std")]]) > max |
          abs(odata[[paste0(".", se, ".std")]]) > semax,
        TRUE,
        FALSE
      )
      odata[[paste0(".", se, ".std")]] <- NULL
    } else {
      odata[[".dropbig"]] <- ifelse(
        abs(odata[[paste0(".", estvarname, ".std")]]) > max,
        TRUE,
        FALSE
      )
    }
    odata[[paste0(".", estvarname, ".std")]] <- NULL
  }
  return(odata)
}

### Set both est, se to NA if any of the two is NA
#' @keywords internal
.na_pair <- function(data, estvarname, se = NULL) {
  if (!is.null(se)) {
    toNA <- (is.na(data[[estvarname]]) | is.na(data[[se]]))
    data[[se]][toNA] <- NA
  } else {
    toNA <- (is.na(data[[estvarname]]))
  }
  data[[estvarname]][toNA] <- NA
  data
}

### Merge description of summary statistics
#' @keywords internal
.describe <- function(x, ref, level) {
  description_df <- data.frame(
    stat = c(
      "nsim",
      "thetamean",
      "thetamedian",
      "se2mean",
      "se2median",
      "bias",
      "rbias",
      "empse",
      "relprec",
      "mse",
      "modelse",
      "relerror",
      "cover",
      "becover",
      "power"
    ),
    description = c(
      "Non-missing point estimates/standard errors",
      "Average point estimate",
      "Median point estimate",
      "Average variance",
      "Median variance",
      "Bias in point estimate",
      "Relative bias in point estimate",
      "Empirical standard error",
      paste("% gain in precision relative to method", ref),
      "Mean squared error",
      "Model-based standard error",
      "Relative % error in standard error",
      paste(
        "Coverage of nominal",
        sprintf("%.0f%%", 100 * (level)),
        "confidence interval"
      ),
      paste(
        "Bias-eliminated coverage of nominal",
        sprintf("%.0f%%", 100 * (level)),
        "confidence interval"
      ),
      paste("Power of", sprintf("%.0f%%", 100 * (1 - level)), "level test")
    ),
    stringsAsFactors = FALSE
  )
  x <- merge(x, description_df, by = "stat")
  x <- x[, names(x)[names(x) != "stat"]]
  x$description <- factor(
    x$description,
    levels = c(
      "Non-missing point estimates/standard errors",
      "Average point estimate",
      "Median point estimate",
      "Average variance",
      "Median variance",
      "Bias in point estimate",
      "Relative bias in point estimate",
      "Empirical standard error",
      paste("% gain in precision relative to method", ref),
      "Mean squared error",
      "Model-based standard error",
      "Relative % error in standard error",
      paste(
        "Coverage of nominal",
        sprintf("%.0f%%", 100 * (level)),
        "confidence interval"
      ),
      paste(
        "Bias-eliminated coverage of nominal",
        sprintf("%.0f%%", 100 * (level)),
        "confidence interval"
      ),
      paste("Power of", sprintf("%.0f%%", 100 * (1 - level)), "level test")
    )
  )
  x <- x[order(x$description), ]
  x <- x[, c(
    "description",
    "est",
    names(x)[!(names(x) %in% c("description", "est"))]
  )]
  return(x)
}

### Format table of results for pretty printing
#' @keywords internal
.format <- function(x, digits, mcse) {
  x$summ$est <- ifelse(
    x$summ$stat == "nsim",
    sprintf("%.0f", x$summ$est),
    sprintf(paste0("%.", digits, "f"), x$summ$est)
  )
  if (x$control$mcse) {
    x$summ$mcse <- sprintf(paste0("%.", digits, "f"), x$summ$mcse)
    x$summ$lower <- sprintf(paste0("%.", digits, "f"), x$summ$lower)
    x$summ$upper <- sprintf(paste0("%.", digits, "f"), x$summ$upper)
  }
  if (is.null(mcse)) {
    # Do nothing, will return est only
  } else if (mcse) {
    x$summ$est <- ifelse(
      x$summ$stat %in%
        c("nsim", "thetamean", "thetamedian", "se2mean", "se2median"),
      x$summ$est,
      paste0(x$summ$est, " (", x$summ$mcse, ")")
    )
  } else {
    x$summ$est <- ifelse(
      x$summ$stat %in%
        c("nsim", "thetamean", "thetamedian", "se2mean", "se2median"),
      x$summ$est,
      paste0(x$summ$est, " (", x$summ$lower, ", ", x$summ$upper, ")")
    )
  }
  x$summ[["mcse"]] <- NULL
  x$summ[["lower"]] <- NULL
  x$summ[["upper"]] <- NULL
  x$summ <- .describe(x$summ, ref = x$ref, level = x$control$level)
  return(x)
}

### Order a data.frame by a vector of columns
.order <- function(data, by) {
  data <- data[do.call(order, lapply(X = by, FUN = function(f) data[[f]])), ]
  data
}

### Bind methods side by side for results table
#' @keywords internal
.bind_methods <- function(data, by, methodvar) {
  data <- .split_by(data = data, by = methodvar)
  lhs <- lapply(X = c("Performance Measure", by), FUN = function(f) {
    data[[1]][[f]]
  })
  names(lhs) <- c("Performance Measure", by)
  data <- lapply(X = data, FUN = function(x) {
    tmp <- x
    row.names(tmp) <- NULL
    tmp
  })
  data <- lapply(X = seq_along(data), FUN = function(i) {
    tmp <- data[[i]]
    tmp[[methodvar]] <- NULL
    tmp[["Performance Measure"]] <- NULL
    if (!is.null(by)) {
      for (b in by) {
        tmp[[b]] <- NULL
      }
    }
    names(tmp) <- names(data)[i]
    tmp
  })
  data <- do.call(cbind, c(lhs, data))
  row.names(data) <- NULL
  data
}

### Make internal_df for 'vs' plots
#' @keywords internal
.make_internal_df <- function(data, b, methodvar, by) {
  ### Identify combinations of methodvar
  cs <- t(utils::combn(x = unique(data[[methodvar]]), m = 2))
  colnames(cs) <- c("X", "Y")
  ### Split data by 'by' factors
  data_split <- .split_by(data = data, by = by)
  ### Restructure data
  internal_df <- list()
  for (i in seq_along(data_split)) {
    tmp <- list()
    for (j in seq(nrow(cs))) {
      tmp[[j]] <- data.frame(
        X = data_split[[i]][[b]][data_split[[i]][[methodvar]] == cs[j, "X"]],
        Y = data_split[[i]][[b]][data_split[[i]][[methodvar]] == cs[j, "Y"]],
        contrast = paste0("X: ", cs[j, "X"], " vs Y: ", cs[j, "Y"]),
        row.names = NULL
      )
    }
    tmp <- .br(tmp)
    if (!is.null(by)) {
      for (byval in by) {
        tmp[[byval]] <- unique(data_split[[i]][[byval]])
      }
    }
    internal_df[[i]] <- tmp
  }
  internal_df <- .br(internal_df)
  ### Return
  return(internal_df)
}

### Drop split data.frame in 'data' with zero rows
.drop_empty_splits <- function(data) {
  data <- lapply(data, FUN = function(x) {
    idx <- vapply(
      X = x,
      FUN = function(x) {
        nrow(x) > 0
      },
      FUN.VALUE = logical(1)
    )
    x[idx]
  })
  nrs <- vapply(X = data, FUN = length, FUN.VALUE = numeric(1))
  data <- data[nrs > 0]
  return(data)
}

### Check private names
.check_private <- function(var, label, private_names) {
  if (!is.null(var)) {
    if (any(var %in% private_names)) {
      this <- which(var %in% private_names)
      stop(
        paste0(
          "'",
          var[this],
          "' is not an allowed name for '",
          label,
          "'; see help('simsum') for more details."
        ),
        call. = FALSE
      )
    }
  }
}

testthat::context("utils")
set.seed(20181220)
n <- 30

testthat::test_that(".validate_levels works as expected", {
  data("iris", package = "datasets")
  for (char in c("-", ".", "~", "#", "@")) {
    iris$Test <- paste0(
      iris$Species,
      ifelse(iris$Sepal.Length > mean(iris$Sepal.Length), char, "")
    )
    iris$Test <- factor(iris$Test)
    testthat::expect_error(
      object = .validate_levels(data = iris, cols = "Test", char = char)
    )
  }
})

testthat::test_that(".dropbig detects values above 'max'", {
  df <- data.frame(
    theta = rnorm(n),
    se = rnorm(n)
  )
  df$theta[1] <- rnorm(1, mean = 100)
  out <- .dropbig(
    data = df,
    estvarname = "theta",
    se = "se",
    methodvar = NULL,
    by = NULL,
    max = 5,
    semax = 10,
    robust = FALSE
  )
  testthat::expect_equal(object = is.na(out$theta[1]), expected = TRUE)
  out <- .dropbig(
    data = df,
    estvarname = "theta",
    se = "se",
    methodvar = NULL,
    by = NULL,
    max = 5,
    semax = 10,
    robust = TRUE
  )
  testthat::expect_equal(object = is.na(out$theta[1]), expected = TRUE)
})

testthat::test_that(".na_pair works as expected", {
  df <- data.frame(
    theta = rnorm(n),
    se = rnorm(n)
  )
  df$theta[1:10] <- NA
  df.processed <- .na_pair(data = df, estvarname = "theta", se = "se")
  testthat::expect_equal(
    object = sum(is.na(df.processed$se)),
    expected = sum(is.na(df$theta))
  )
})

testthat::test_that(".order actually orders a dataset", {
  data("iris", package = "datasets")
  iris2 <- iris[
    order(
      iris$Sepal.Length,
      iris$Sepal.Width,
      iris$Petal.Length,
      iris$Petal.Width
    ),
  ]
  messyIris <- iris[
    sample(x = seq(nrow(iris)), size = nrow(iris), replace = FALSE),
  ]
  fixedIris <- .order(
    data = messyIris,
    by = c("Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width")
  )
  testthat::expect_equivalent(object = fixedIris, expected = iris2)
})

testthat::test_that(".dropbig works if splitting by 'by' or 'methodvar'", {
  df <- data.frame(
    theta = rnorm(n),
    se = rnorm(n),
    group = rep(x = 1:3, each = n / 3)
  )
  df$theta[1] <- rnorm(1, mean = 1000)
  ### Check standardised values:
  # library(tidyverse)
  # df %>%
  # 	group_by(group) %>%
  # 	mutate(std = (theta - mean(theta)) / sqrt(var(theta)),
  # 				 std.r = (theta - median(theta)) / (fivenum(theta)[4] - fivenum(theta)[2]))
  out <- .dropbig(
    data = df,
    estvarname = "theta",
    se = "se",
    methodvar = NULL,
    by = "group",
    max = 2,
    semax = 10,
    robust = FALSE
  )
  testthat::expect_equal(object = is.na(out$theta[1]), expected = TRUE)
  out <- .dropbig(
    data = df,
    estvarname = "theta",
    se = "se",
    methodvar = NULL,
    by = "group",
    max = 10,
    semax = 10,
    robust = TRUE
  )
  testthat::expect_equal(object = is.na(out$theta[1]), expected = TRUE)
})

testthat::context("simsum")

testthat::test_that("simsum prints ok", {
  data("MIsim", package = "rsimsum")
  data("relhaz", package = "rsimsum")
  testthat::expect_output(print(rsimsum::simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )))
  testthat::expect_output(print(rsimsum::simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se"
  )))
  testthat::expect_output(print(rsimsum::simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    control = list(mcse = FALSE)
  )))
  testthat::expect_output(print(rsimsum::simsum(
    data = MIsim,
    estvarname = "b",
    se = "se"
  )))
  testthat::expect_output(print(rsimsum::simsum(
    data = relhaz,
    estvarname = "theta",
    true = -0.5,
    se = "se",
    methodvar = "model",
    by = c("n", "baseline")
  )))
  testthat::expect_output(print(rsimsum::simsum(
    data = relhaz,
    estvarname = "theta",
    true = -0.5,
    se = "se",
    by = c("n", "baseline")
  )))
  testthat::expect_output(print(rsimsum::simsum(
    data = relhaz,
    estvarname = "theta",
    se = "se"
  )))
})

testthat::test_that("simsum returns an object of class simsum", {
  data("MIsim", package = "rsimsum")
  s <- rsimsum::simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    ref = "CC"
  )
  testthat::expect_s3_class(s, "simsum")
})

testthat::test_that("summ slot of a simsum object is a data.frame", {
  data("MIsim", package = "rsimsum")
  s <- rsimsum::simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    ref = "CC"
  )
  testthat::expect_s3_class(s$summ, "data.frame")
})

testthat::test_that("not passing estvarname throws an error", {
  testthat::expect_error(
    {
      data("MIsim", package = "rsimsum")
      s <- rsimsum::simsum(
        data = MIsim,
        true = 0.5,
        se = "se",
        methodvar = "method",
        ref = "CC"
      )
    },
    'argument "estvarname" is missing, with no default'
  )
})

testthat::test_that("specifying ref and not methodvar throws a warning", {
  testthat::expect_warning(
    {
      data("MIsim", package = "rsimsum")
      s <- rsimsum::simsum(
        data = MIsim,
        estvarname = "b",
        true = 0.5,
        se = "se",
        ref = "CC"
      )
    },
    "'ref' method is specified while 'methodvar' is not: 'ref' will be ignored"
  )
})

testthat::test_that("specifying methodvar and not ref shows a message", {
  testthat::expect_message(
    {
      data("MIsim", package = "rsimsum")
      s <- simsum(
        data = MIsim,
        estvarname = "b",
        true = 0.5,
        se = "se",
        methodvar = "method"
      )
    },
    "'ref' method was not specified, CC set as the reference"
  )
})

testthat::test_that("running simsum on MIsim return summaries of the correct dimension", {
  data("MIsim", package = "rsimsum")
  s <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  testthat::expect_equal(
    dim(s$summ),
    expected = c(15 * length(unique(MIsim$method)), 4)
  )
})

testthat::test_that("simsum with mcse option returns mcse", {
  data("MIsim", package = "rsimsum")
  s <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  testthat::expect_true("mcse" %in% names(s$summ))
})

testthat::test_that("simsum without mcse option does not returns mcse", {
  data("MIsim", package = "rsimsum")
  s <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    control = list(mcse = FALSE)
  )
  testthat::expect_false("mcse" %in% names(s$summ))
})

testthat::test_that("simsum with by factors returns error when 'by' name is not a variable in data", {
  testthat::expect_error({
    data("relhaz", package = "rsimsum")
    simsum(
      data = relhaz,
      estvarname = "theta",
      true = -0.5,
      se = "se",
      methodvar = "model",
      by = "by"
    )
  })
})

testthat::test_that("simsum with by factors works fine", {
  data("relhaz", package = "rsimsum")
  testthat::expect_output(
    object = print(simsum(
      data = relhaz,
      estvarname = "theta",
      true = -0.5,
      se = "se",
      methodvar = "model",
      by = "n"
    ))
  )
  testthat::expect_output(
    object = print(simsum(
      data = relhaz,
      estvarname = "theta",
      true = -0.5,
      se = "se",
      methodvar = "model",
      by = "baseline"
    ))
  )
  testthat::expect_output(
    object = print(simsum(
      data = relhaz,
      estvarname = "theta",
      true = -0.5,
      se = "se",
      methodvar = "model",
      by = c("n", "baseline")
    ))
  )
})

testthat::test_that("simsum with by factors returns a data.frame with results", {
  data("relhaz", package = "rsimsum")
  s <- simsum(
    data = relhaz,
    estvarname = "theta",
    true = -0.5,
    se = "se",
    methodvar = "model",
    by = c("n", "baseline")
  )
  testthat::expect_s3_class(object = s$summ, class = "data.frame")
})

testthat::test_that("simsum works with missing data and default arguments", {
  data("MIsim")
  x <- MIsim
  set.seed(180123)
  x[which(rnorm(nrow(x)) > 2), "b"] <- NA
  x[which(rnorm(nrow(x)) > 2), "se"] <- NA
  testthat::expect_output(
    object = print(simsum(
      data = x,
      estvarname = "b",
      true = 0.5,
      se = "se",
      methodvar = "method"
    ))
  )
})

testthat::test_that("simsum with x = FALSE does not return data", {
  data("MIsim", package = "rsimsum")
  s <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    x = FALSE
  )
  testthat::expect_null(object = s$x)
})

testthat::test_that("simsum with custom ci.limits works as expected", {
  data("MIsim", package = "rsimsum")
  s <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    ci.limits = c(-Inf, Inf)
  )
  testthat::expect_true(object = all(s$summ$est[s$summ$stat == "cover"] == 1))
  s <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    ci.limits = c(Inf, -Inf)
  )
  testthat::expect_true(object = all(s$summ$est[s$summ$stat == "cover"] == 0))
})

testthat::test_that("simsum with x = TRUE does return data", {
  data("MIsim", package = "rsimsum")
  s <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    x = TRUE
  )
  testthat::expect_s3_class(object = s$x, class = "data.frame")
  testthat::expect_equal(object = nrow(s$x), expected = nrow(MIsim))
})

testthat::test_that("simsum with dropbig = TRUE does drop all the big stuff", {
  data("MIsim", package = "rsimsum")
  s <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    x = TRUE,
    dropbig = TRUE,
    control = list(dropbig.max = 3, dropbig.semax = 10, dropbig.robust = FALSE)
  )
  expected <- .dropbig(
    data = MIsim,
    estvarname = "b",
    se = "se",
    methodvar = "method",
    by = NULL,
    robust = FALSE,
    max = 3,
    semax = 10
  )
  expected <- .na_pair(data = expected, estvarname = "b", se = "se")
  expected$method <- factor(expected$method)
  testthat::expect_equivalent(object = s$x, expected = expected)
  s <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    methodvar = "method",
    x = TRUE,
    dropbig = TRUE,
    control = list(dropbig.max = 3, dropbig.robust = FALSE)
  )
  expected <- .dropbig(
    data = MIsim,
    estvarname = "b",
    methodvar = "method",
    by = NULL,
    robust = FALSE,
    max = 3
  )
  expected <- .na_pair(data = expected, estvarname = "b")
  expected$method <- factor(expected$method)
  testthat::expect_equivalent(object = s$x, expected = expected)
})

testthat::test_that("simsum without 'true' does not compute bias, rbias, cover, mse", {
  data("MIsim", package = "rsimsum")
  s <- simsum(data = MIsim, estvarname = "b", se = "se", methodvar = "method")
  testthat::expect_false(
    object = any(c("bias", "rbias", "cover", "mse") %in% s$summ$stat)
  )
})

testthat::test_that("simsum without 'se' does not compute se2mean, se2median, modelse, relerror, cover, becover, power", {
  data("MIsim", package = "rsimsum")
  s <- simsum(data = MIsim, estvarname = "b", true = 0.5, methodvar = "method")
  testthat::expect_false(
    object = any(
      c(
        "se2mean",
        "se2median",
        "modelse",
        "relerror",
        "cover",
        "becover",
        "power"
      ) %in%
        s$summ$stat
    )
  )
})

testthat::test_that("simsum without 'se' nor 'true' does not compute se2mean, se2median, modelse, relerror, cover, becover, power, bias, rbias mse", {
  data("MIsim", package = "rsimsum")
  s <- simsum(data = MIsim, estvarname = "b", methodvar = "method")
  testthat::expect_false(
    object = any(
      c(
        "se2mean",
        "se2median",
        "modelse",
        "relerror",
        "cover",
        "becover",
        "power",
        "bias",
        "rbias",
        "mse"
      ) %in%
        s$summ$stat
    )
  )
})

testthat::test_that("simsum with both ci.limits and df throws an error", {
  data("tt", package = "rsimsum")
  testthat::expect_error(
    object = simsum(
      data = tt,
      estvarname = "diff",
      se = "se",
      methodvar = "method",
      ci.limits = c("lower", "upper"),
      df = "df"
    )
  )
})

#' @title print.simsum
#' @description Print method for simsum objects
#' @param x An object of class `simsum`.
#' @param ... Ignored.
#' @export
#'
#' @examples
#' data("MIsim")
#' x <- simsum(
#'   data = MIsim, estvarname = "b", true = 0.5, se = "se",
#'   methodvar = "method"
#' )
#' x
#'
#' MIsim$true <- 0.5
#' x <- simsum(data = MIsim, estvarname = "b", true = "true", se = "se")
#' x
print.simsum <- function(x, ...) {
  cat("Summary of a simulation study with a single estimand.\n")
  if (!is.null(x$true)) {
    if (is.character(x$true)) {
      cat(
        "True value of the estimand from column",
        paste0("'", x$true, "'"),
        "\n"
      )
    } else {
      cat("True value of the estimand:", x$true, "\n")
    }
  } else {
    cat(
      "True value of the estimand not defined: bias, relative bias, coverage, and mean squared error were not computed.\n"
    )
  }

  ### Print `methodvar` (if any), possible methods, and reference method
  if (!is.null(x$methodvar)) {
    if (length(x$methodvar) > 1) {
      cat(
        "\nColumns identifying methods:",
        paste(x$methodvar, collapse = ", "),
        "\n"
      )
      reftable <- .compact_method_columns(
        data = tidy.simsum(x),
        methodvar = x$methodvar
      )$data[[":methodvar"]]
      cat("\tUnique methods:", paste(unique(reftable), collapse = ", "), "\n")
    } else {
      cat("\nMethod variable:", x$methodvar, "\n")
      methods <- unique(x$summ[[x$methodvar]])
      cat("\tUnique methods:", paste(methods, collapse = ", "), "\n")
      cat("\tReference method:", x$ref, "\n")
    }
  } else {
    cat("\nMethod variable: none\n")
  }

  ### Print `by` factors (if any)
  if (!is.null(x$by)) {
    cat("\nBy factors:", paste(x$by, collapse = ", "), "\n")
  } else {
    cat("\nBy factors: none\n")
  }

  ### Print whether Monte Carlo SEs were computed or not
  if (x$control$mcse) {
    cat("\nMonte Carlo standard errors were computed.\n")
  } else {
    cat("\nMonte Carlo standard errors were not computed.\n")
  }
}

### Forest plot
#' @keywords internal
.forest_plot <- function(data, methodvar, by, stats, ci, target, scales) {
  ### Build basic plot
  if (!is.null(methodvar)) {
    methodvar <- rlang::sym(methodvar)
    gg <- ggplot2::ggplot(
      data = data,
      ggplot2::aes(x = {{ methodvar }}, y = est)
    )
  } else {
    gg <- ggplot2::ggplot(
      data = data,
      ggplot2::aes(x = "Single Method", y = est)
    ) +
      ggplot2::labs(x = "")
  }
  gg <- gg +
    ggplot2::geom_hline(yintercept = target, linetype = "dotted") +
    ggplot2::geom_point() +
    ggplot2::labs(y = stats)

  ### Wrap by 'by' factors if defined
  if (!is.null(by)) {
    by <- rlang::syms(by)
    gg <- gg +
      ggplot2::facet_wrap(
        facets = ggplot2::vars(!!!{{ by }}),
        labeller = ggplot2::label_both,
        scales = scales
      )
  }

  ### Add confidence intervals if we are calling autoplot on a summary object
  if (ci) {
    gg <- gg +
      ggplot2::geom_errorbar(
        ggplot2::aes(ymin = lower, ymax = upper),
        width = 1 / 3
      )
  }

  ### Return plot
  return(gg)
}

### Lolly plot
#' @keywords internal
.lolly_plot <- function(data, methodvar, by, stats, ci, target, scales) {
  ### Build basic plot
  if (!is.null(methodvar)) {
    methodvar <- rlang::sym(methodvar)
    gg <- ggplot2::ggplot(
      data = data,
      ggplot2::aes(x = est, y = {{ methodvar }})
    ) +
      ggplot2::geom_vline(xintercept = target, linetype = "dotted") +
      ggplot2::geom_segment(aes(xend = target, yend = {{ methodvar }}))
  } else {
    gg <- ggplot2::ggplot(
      data = data,
      ggplot2::aes(x = est, y = "Single Method")
    ) +
      ggplot2::geom_vline(xintercept = target, linetype = "dotted") +
      ggplot2::geom_segment(aes(xend = target, yend = "Single Method")) +
      ggplot2::labs(y = "")
  }
  gg <- gg +
    ggplot2::geom_point() +
    ggplot2::labs(x = stats)

  ### Wrap by 'by' factors if defined
  if (!is.null(by)) {
    by <- rlang::syms(by)
    gg <- gg +
      ggplot2::facet_wrap(
        facets = ggplot2::vars(!!!{{ by }}),
        labeller = ggplot2::label_both,
        scales = scales
      )
  }

  ### Add confidence intervals if we are calling autoplot on a summary object
  if (ci) {
    if (!is.null(methodvar)) {
      gg <- gg +
        ggplot2::geom_point(
          ggplot2::aes(x = lower, y = {{ methodvar }}),
          shape = 40
        ) +
        ggplot2::geom_point(
          ggplot2::aes(x = upper, y = {{ methodvar }}),
          shape = 41
        )
    } else {
      gg <- gg +
        ggplot2::geom_point(
          ggplot2::aes(x = lower, y = "Single Method"),
          shape = 40
        ) +
        ggplot2::geom_point(
          ggplot2::aes(x = upper, y = "Single Method"),
          shape = 41
        )
    }
  }

  ### Return plot
  return(gg)
}

### Zip plot
#' @keywords internal
.zip_plot <- function(
  data,
  estvarname,
  se,
  true,
  methodvar,
  by,
  ci.limits,
  df,
  control,
  summ,
  zoom,
  zip_ci_colours
) {
  ### Extract overall coverage
  summ <- summ[summ$stat == "cover", ]
  summ$cover <- summ$est
  if (all(c("lower", "upper") %in% names(summ))) {
    summ$cover_lower <- summ$lower
    summ$cover_upper <- summ$upper
    summ$lower <- NULL
    summ$upper <- NULL
  } else {
    summ$cover_lower <- summ$cover -
      stats::qnorm(1 - (1 - 0.95) / 2) * summ$mcse
    summ$cover_upper <- summ$cover +
      stats::qnorm(1 - (1 - 0.95) / 2) * summ$mcse
  }
  summ$est <- NULL
  summ$stat <- NULL
  summ$mcse <- NULL

  ### Define critical value utilised to compute coverage probabilities
  if (is.null(df)) {
    data[["crit"]] <- stats::qnorm(1 - (1 - control$level) / 2)
  } else {
    data[["crit"]] <- stats::qt(1 - (1 - control$level) / 2, df = data[[df]])
  }

  ### Compute coverage
  if (is.null(ci.limits)) {
    data[["lower"]] <- data[[estvarname]] - data[["crit"]] * data[[se]]
    data[["upper"]] <- data[[estvarname]] + data[["crit"]] * data[[se]]
  } else if (is.character(ci.limits)) {
    data[["lower"]] <- data[[ci.limits[1]]]
    data[["upper"]] <- data[[ci.limits[2]]]
  } else if (is.numeric(ci.limits)) {
    data[["lower"]] <- ci.limits[1]
    data[["upper"]] <- ci.limits[2]
  }
  data[["covering"]] <- (true >= data[["lower"]] & true <= data[["upper"]])
  data[["covering"]] <- factor(
    data[["covering"]],
    levels = c(FALSE, TRUE),
    labels = c("Non-coverers", "Coverers")
  )

  ### Compute z value
  data[["z"]] <- abs((data[[estvarname]] - true) / data[[se]])

  ### Split data by 'methodvar', 'by'
  data <- .split_by(data = data, by = by)
  data <- lapply(data, function(x) .split_by(data = x, by = methodvar))

  ### Compute ranking for each data split
  for (i in seq_along(data)) {
    for (j in seq_along(data[[i]])) {
      A <- rank(data[[i]][[j]][["z"]])
      B <- max(A)
      data[[i]][[j]][["rank"]] <- A / B
    }
    data[[i]] <- .br(data[[i]])
  }
  data <- .br(data)

  ### Merge back summary statistics
  data <- merge(x = data, y = summ)

  ### Label of the y-axis
  ylab <- ifelse(
    is.null(df),
    "Fractional centile of |z-score|",
    "Fractional centile of |t-score|"
  )

  ### Define CI lines colors
  if (length(zip_ci_colours) == 2) {
    data$line_color_lower <- ifelse(
      data$cover_lower <= control$level & control$level <= data$cover_upper,
      zip_ci_colours[1],
      zip_ci_colours[2]
    )
    data$line_color_upper <- ifelse(
      data$cover_lower <= control$level & control$level <= data$cover_upper,
      zip_ci_colours[1],
      zip_ci_colours[2]
    )
  } else if (length(zip_ci_colours) == 3) {
    data$line_color_lower <- ifelse(
      data$cover_lower > control$level & data$cover_upper > control$level,
      zip_ci_colours[3],
      ifelse(
        data$cover_lower < control$level & data$cover_upper < control$level,
        zip_ci_colours[2],
        ifelse(
          data$cover_lower <= control$level & control$level <= data$cover_upper,
          zip_ci_colours[1],
          NA
        )
      )
    )

    data$line_color_upper <- ifelse(
      data$cover_lower > control$level & data$cover_upper > control$level,
      zip_ci_colours[3],
      ifelse(
        data$cover_lower < control$level & data$cover_upper < control$level,
        zip_ci_colours[2],
        ifelse(
          data$cover_lower <= control$level & control$level <= data$cover_upper,
          zip_ci_colours[1],
          NA
        )
      )
    )
  } else {
    data$line_color_lower <- zip_ci_colours
    data$line_color_upper <- zip_ci_colours
  }

  ### Build plot
  gg <- ggplot2::ggplot(
    data,
    ggplot2::aes(y = rank, x = lower, color = covering)
  ) +
    ggplot2::geom_segment(ggplot2::aes(yend = rank, xend = upper)) +
    ggplot2::geom_vline(
      xintercept = true,
      color = "black",
      linetype = "dashed"
    ) +
    ggplot2::geom_hline(
      ggplot2::aes(yintercept = cover_lower),
      color = data$line_color_lower,
      linetype = "dashed",
      linewidth = 1
    ) +
    ggplot2::geom_hline(
      ggplot2::aes(yintercept = 0.95),
      color = "black",
      linetype = "dashed"
    ) +
    ggplot2::geom_hline(
      ggplot2::aes(yintercept = cover_upper),
      color = data$line_color_upper,
      linetype = "dashed",
      linewidth = 1
    ) +
    ggplot2::labs(
      y = ylab,
      x = paste0(100 * control$level, "% confidence intervals"),
      color = ""
    ) +
    theme(legend.position = "bottom")

  ### If 'by', use facet_grid; facet_wrap otherwise
  if (!is.null(by) & !is.null(methodvar)) {
    by <- rlang::syms(by)
    methodvar <- rlang::sym(methodvar)
    gg <- gg +
      ggplot2::facet_grid(
        cols = ggplot2::vars(!!!{{ by }}),
        rows = ggplot2::vars({{ methodvar }}),
        labeller = ggplot2::labeller(
          .rows = ggplot2::label_value,
          .cols = ggplot2::label_both
        )
      )
  } else if (is.null(by) & !is.null(methodvar)) {
    methodvar <- rlang::sym(methodvar)
    gg <- gg +
      ggplot2::facet_wrap(facets = ggplot2::vars({{ methodvar }}))
  } else if (!is.null(by) & is.null(methodvar)) {
    by <- rlang::syms(by)
    gg <- gg +
      ggplot2::facet_wrap(facets = ggplot2::vars(!!!{{ by }}))
  }

  ### Zoom (or not)
  gg <- gg +
    ggplot2::coord_cartesian(ylim = c(1 - zoom, 1))

  ### Return plot
  return(gg)
}

### Method vs method; supports Bland-Altman type plots
#' @keywords internal
.vs_plot <- function(data, b, methodvar, by, fitted, scales, ba) {
  ### Compute internal df
  internal_df <- .make_internal_df(
    data = data,
    b = b,
    methodvar = methodvar,
    by = by
  )

  ### if Bland-Altman type plot, replace X and Y for mean and diff
  if (ba) {
    internal_df[["mean"]] <- (internal_df[["X"]] + internal_df[["Y"]]) / 2
    internal_df[["diff"]] <- internal_df[["X"]] - internal_df[["Y"]]
    internal_df[["X"]] <- internal_df[["mean"]]
    internal_df[["Y"]] <- internal_df[["diff"]]
    internal_df[["mean"]] <- NULL
    internal_df[["diff"]] <- NULL
  }

  ### Build plot
  caption <- paste0("Comparison of variable '", b, "'")
  if (ba) {
    caption <- paste0(caption, "; Bland-Altman type plot")
  }
  gg <- ggplot2::ggplot(data = internal_df, ggplot2::aes(x = X, y = Y)) +
    ggplot2::geom_point() +
    ggplot2::labs(caption = caption)

  ### Add reference line, depending on the type of plot
  if (ba) {
    gg <- gg +
      ggplot2::geom_hline(yintercept = 0, linetype = "dashed")
  } else {
    gg <- gg +
      ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed")
  }

  ### if Bland-Altman type plot, fix labels
  if (ba) {
    gg <- gg +
      ggplot2::labs(x = "Mean", y = "Difference")
  }

  ### If 'by', use facet_grid; facet_wrap otherwise
  if (!is.null(by)) {
    by <- rlang::syms(by)
    gg <- gg +
      ggplot2::facet_grid(
        cols = ggplot2::vars(!!!{{ by }}),
        rows = ggplot2::vars(contrast),
        scales = scales,
        labeller = ggplot2::labeller(
          .rows = ggplot2::label_value,
          .cols = ggplot2::label_both
        )
      )
  } else {
    gg <- gg +
      ggplot2::facet_wrap(~contrast, scales = scales)
  }

  ### If 'fitted' add regression line
  if (fitted) {
    gg <- gg +
      ggplot2::geom_smooth(method = "lm")
  }

  ### Return plot
  return(gg)
}

### Ridgeline plot
#' @keywords internal
#' @importFrom ggridges geom_density_ridges
.ridge_plot <- function(data, b, methodvar, by) {
  ### Create a .dgm column
  if (!is.null(by)) {
    tmp <- lapply(by, function(x) data[[x]])
    data[[".dgm"]] <- do.call(paste, c(tmp, sep = ", "))
  } else {
    data[[".dgm"]] <- "Single DGM"
  }

  ### Build plot
  b <- rlang::sym(b)
  if (!is.null(methodvar)) {
    methodvar <- rlang::sym(methodvar)
    gg <- ggplot2::ggplot(
      data = data,
      ggplot2::aes(
        x = !!{{ b }},
        y = .dgm,
        color = {{ methodvar }},
        fill = {{ methodvar }}
      )
    )
  } else {
    gg <- ggplot2::ggplot(data = data, ggplot2::aes(x = !!{{ b }}, y = .dgm))
  }
  gg <- gg +
    ggridges::geom_density_ridges(alpha = 0.25) +
    ggplot2::labs(y = "")

  ### Return plot
  return(gg)
}

### Heat plot
#' @keywords internal
.heat_plot <- function(data, methodvar, by, stats) {
  ### Create a .dgm column
  if (!is.null(by)) {
    tmp <- lapply(by, function(x) data[[x]])
    data[[".dgm"]] <- do.call(paste, c(tmp, sep = ", "))
  } else {
    data[[".dgm"]] <- "Single DGM"
  }

  ### Build basic plot
  if (!is.null(methodvar)) {
    methodvar <- rlang::sym(methodvar)
    gg <- ggplot2::ggplot(
      data = data,
      ggplot2::aes(x = {{ methodvar }}, y = .dgm, fill = est)
    )
  } else {
    gg <- ggplot2::ggplot(
      data = data,
      ggplot2::aes(x = "Single Method", y = .dgm, fill = est)
    ) +
      ggplot2::labs(x = "")
  }
  gg <- gg +
    ggplot2::geom_tile() +
    ggplot2::labs(y = "", fill = stats)

  ### Return plot
  return(gg)
}

### Nested loop plot
#' @keywords internal
.nlp <- function(data, methodvar, by, stats, target, top) {
  ### Compute internal data
  opts <- lapply(X = by, FUN = function(x) levels(data[[x]]))
  names(opts) <- by
  dgms <- do.call(expand.grid, opts)
  dgms[[".scenario"]] <- seq(nrow(dgms))
  opts_methods <- lapply(X = methodvar, FUN = function(x) levels(data[[x]]))
  names(opts_methods) <- methodvar
  opts_methods[[".scenario"]] <- unique(dgms[[".scenario"]])
  opts_methods <- do.call(expand.grid, opts_methods)
  dgms <- merge(dgms, opts_methods)
  data <- merge(x = data, y = dgms)
  data <- data[order(data[[".scenario"]]), ]

  ### Compute limits and placement of nested loop plot labels
  limits <- range(data[["est"]], na.rm = TRUE)
  delta <- diff(range(data[["est"]])) / 10
  placement <- vector(mode = "list", length = length(by))
  for (i in seq_along(placement)) {
    if (i == 1) {
      if (top) {
        placement[[i]] <- c(
          round(limits[2], digits = 2) + delta,
          round(limits[2], digits = 2) + 2 * delta
        )
      } else {
        placement[[i]] <- c(
          round(limits[1], digits = 2) - 2 * delta,
          round(limits[1], digits = 2) - delta
        )
      }
    } else {
      if (top) {
        placement[[i]] <- c(
          placement[[i - 1]][2] + delta,
          placement[[i - 1]][2] + 2 * delta
        )
      } else {
        placement[[i]] <- c(
          placement[[i - 1]][1] - 2 * delta,
          placement[[i - 1]][1] - delta
        )
      }
    }
  }

  ### Identify if simulation design is fully factorial
  ff <- max(data[[".scenario"]]) == length(unique(data[[".scenario"]]))
  num_ff <- length(unique(data[[".scenario"]]))

  ### Rescale variables included in the nested loop plot
  for (i in seq_along(by)) {
    data[[paste0(".", by[i])]] <- scales::rescale(
      x = as.numeric(data[[by[i]]]),
      to = placement[[i]]
    )
  }

  ### Add back scenarios, to obtain equivalent fully-factorial NLP
  data <- merge(x = data, y = dgms, all = TRUE)
  data[[".notmissing"]] <- !is.na(data[["est"]])

  ### Build basic plot
  if (!is.null(methodvar)) {
    methodvar <- rlang::sym(methodvar)
    gg <- ggplot2::ggplot(
      data = data,
      mapping = ggplot2::aes(x = .scenario, y = est)
    ) +
      ggplot2::geom_hline(yintercept = target, linetype = "dotted") +
      ggplot2::geom_step(mapping = ggplot2::aes(color = !!methodvar))
  } else {
    gg <- ggplot2::ggplot(
      data = data,
      mapping = ggplot2::aes(x = .scenario, y = est)
    ) +
      ggplot2::geom_hline(yintercept = target, linetype = "dotted") +
      ggplot2::geom_step()
  }
  if (ff) {
    gg <- gg +
      ggplot2::labs(
        x = paste0(
          paste(
            vapply(
              X = by,
              FUN = function(x) length(levels(data[[x]])),
              FUN.VALUE = numeric(1)
            ),
            collapse = " x "
          ),
          " = ",
          length(unique(data[[".scenario"]])),
          " ordered scenarios"
        ),
        y = stats
      )
  } else {
    gg <- gg +
      ggplot2::labs(x = paste0(num_ff, " ordered scenarios"), y = stats)
  }

  ### Build and add legends of nested loop plot
  for (i in seq_along(by)) {
    .tmp <- rlang::sym(paste0(".", by[i]))
    gg <- gg +
      ggplot2::geom_step(mapping = ggplot2::aes(y = !!.tmp)) +
      ggplot2::annotate(
        geom = "text",
        x = 1,
        y = placement[[i]][2] + delta / 2,
        label = paste0(
          by[i],
          ": ",
          paste(levels(data[[by[i]]]), collapse = ", ")
        ),
        hjust = 0,
        vjust = 0.5
      )
  }

  ### Return plot
  return(gg)
}

#' @keywords internal
.density_plot <- function(
  data,
  b,
  methodvar,
  by,
  fitted,
  scales,
  hex,
  density.legend
) {
  ### Compute internal df
  internal_df <- .make_internal_df(
    data = data,
    b = b,
    methodvar = methodvar,
    by = by
  )

  ### Build plot
  caption <- paste0("Comparison of variable '", b, "'")
  gg <- ggplot2::ggplot(data = internal_df, ggplot2::aes(x = X, y = Y)) +
    ggplot2::labs(caption = caption, fill = "Count")

  ### Add layer with contour or hexbin plot
  if (hex) {
    gg <- gg +
      ggplot2::geom_hex(show.legend = density.legend)
  } else {
    gg <- gg +
      ggplot2::stat_density_2d(
        mapping = ggplot2::aes(fill = after_stat(level)),
        geom = "polygon",
        show.legend = density.legend
      )
  }

  ### Add reference line
  gg <- gg +
    ggplot2::geom_abline(slope = 1, intercept = 0, linetype = "dashed")

  ### If 'by', use facet_grid; facet_wrap otherwise
  if (!is.null(by)) {
    by <- rlang::syms(by)
    gg <- gg +
      ggplot2::facet_grid(
        cols = ggplot2::vars(!!!{{ by }}),
        rows = ggplot2::vars(contrast),
        scales = scales,
        labeller = ggplot2::labeller(
          .rows = ggplot2::label_value,
          .cols = ggplot2::label_both
        )
      )
  } else {
    gg <- gg +
      ggplot2::facet_wrap(~contrast)
  }

  ### If 'fitted' add regression line
  if (fitted) {
    gg <- gg +
      ggplot2::geom_smooth(method = "lm")
  }

  ### Return plot
  return(gg)
}

#' @title print.multisimsum
#' @description Print method for multisimsum objects
#' @param x An object of class `multisimsum`.
#' @param ... Ignored.
#' @export
#'
#' @examples
#' data(frailty)
#' ms <- multisimsum(
#'   data = frailty, par = "par", true = c(
#'     trt = -0.50,
#'     fv = 0.75
#'   ), estvarname = "b", se = "se", methodvar = "model",
#'   by = "fv_dist"
#' )
#' ms
#'
#' data("frailty", package = "rsimsum")
#' frailty$true <- ifelse(frailty$par == "trt", -0.50, 0.75)
#' ms <- multisimsum(data = frailty, par = "par", estvarname = "b", true = "true")
#' ms
print.multisimsum <- function(x, ...) {
  ### Print `par`, possible estimands
  cat("\nEstimands variable:", x$par, "\n")
  estimands <- unique(x$summ[[x$par]])
  cat("\tUnique estimands:", paste(estimands, collapse = ", "), "\n")
  if (!is.null(x$true)) {
    if (rlang::is_named(x$true)) {
      cat(
        "\tTrue values:",
        paste(estimands, "=", x$true[estimands], collapse = ", "),
        "\n"
      )
    } else {
      if (is.character(x$true)) {
        cat("\tTrue values from column", paste0("'", x$true, "'"), "\n")
      } else {
        cat("\tTrue values fixed at value", x$true, "\n")
      }
    }
  } else {
    cat(
      "\tTrue value of the estimands not defined: bias, relative bias, coverage, and mean squared error were not computed.\n"
    )
  }

  ### Print `methodvar` (if any), possible methods, and reference method
  if (!is.null(x$methodvar)) {
    if (length(x$methodvar) > 1) {
      cat(
        "\nColumns identifying methods:",
        paste(x$methodvar, collapse = ", "),
        "\n"
      )
      reftable <- .compact_method_columns(
        data = tidy.multisimsum(x),
        methodvar = x$methodvar
      )$data[[":methodvar"]]
      cat("\tUnique methods:", paste(unique(reftable), collapse = ", "), "\n")
    } else {
      cat("\nMethod variable:", x$methodvar, "\n")
      methods <- unique(x$summ[[x$methodvar]])
      cat("\tUnique methods:", paste(methods, collapse = ", "), "\n")
      cat("\tReference method:", x$ref, "\n")
    }
  } else {
    cat("\nMethod variable: none\n")
  }

  ### Print `by` factors (if any)
  if (!is.null(x$by)) {
    cat("\nBy factors:", paste(x$by, collapse = ", "), "\n")
  } else {
    cat("\nBy factors: none\n")
  }

  ### Print whether Monte Carlo SEs were computed or not
  if (x$control$mcse) {
    cat("\nMonte Carlo standard errors were computed.\n")
  } else {
    cat("\nMonte Carlo standard errors were not computed.\n")
  }
}

#' @title Analyses of simulation studies with multiple estimands at once, including Monte Carlo error
#' @description `multisimsum` is an extension of [simsum()] that can handle multiple estimated parameters at once.
#' `multisimsum` calls [simsum()] internally, each estimands at once.
#' There is only one new argument that must be set when calling `multisimsum`: `par`, a string representing the column of `data` that identifies the different estimands.
#' Additionally, with `multisimsum` the argument `true` can be a named vector, where names correspond to each estimand (see examples).
#' Otherwise, constant values (or values identified by a column in `data`) will be utilised.
#' See `vignette("E-custom-inputs", package = "rsimsum")` for more details.
#' @param par The name of the variable containing the methods to compare.
#' Can be `NULL`.
#' @inheritParams simsum
#' @return An object of class `multisimsum`.
#' @export
#' @details
#' The following names are not allowed for `estvarname`, `se`, `methodvar`, `by`, `par`: `stat`, `est`, `mcse`, `lower`, `upper`, `:methodvar`.
#' @examples
#' data("frailty", package = "rsimsum")
#' ms <- multisimsum(
#'   data = frailty,
#'   par = "par", true = c(trt = -0.50, fv = 0.75),
#'   estvarname = "b", se = "se", methodvar = "model",
#'   by = "fv_dist"
#' )
#' ms
multisimsum <- function(
  data,
  par,
  estvarname,
  se = NULL,
  true = NULL,
  methodvar = NULL,
  ref = NULL,
  by = NULL,
  ci.limits = NULL,
  df = NULL,
  dropbig = FALSE,
  x = FALSE,
  control = list()
) {
  ### Check arguments
  arg_checks <- checkmate::makeAssertCollection()
  # 'methodvar', 'ref', 'par' must be a single string value
  checkmate::assert_string(x = par, add = arg_checks)
  # 'methodvar', 'par' must be in 'data'
  checkmate::assert_subset(
    x = methodvar,
    choices = names(data),
    add = arg_checks
  )
  checkmate::assert_subset(x = par, choices = names(data), add = arg_checks)
  # Report
  if (!arg_checks$isEmpty()) {
    checkmate::reportAssertions(arg_checks)
  }
  ### <- The above assumes that the other checks will be run by repeated calls to simsum() below

  ### Set control parameters
  control.default <- list(
    mcse = TRUE,
    level = 0.95,
    power_df = NULL,
    na.rm = TRUE,
    char.sep = "~",
    dropbig.max = 10,
    dropbig.semax = 100,
    dropbig.robust = TRUE
  )
  control.tmp <- unlist(
    list(
      control[names(control) %in% names(control.default)],
      control.default[!(names(control.default) %in% names(control))]
    ),
    recursive = FALSE
  )
  control <- control.tmp

  ### Factorise 'par', 'methodvar'
  data <- .factorise(data = data, cols = c(par, methodvar))

  ### Check that levels of factors are ok
  .validate_levels(
    data = data,
    cols = c(par, methodvar),
    char = ifelse(!is.null(control$char.sep), control$char.sep, "~")
  )

  ### Set reference method if 'ref' is not specified
  if (!is.null(methodvar)) {
    if (is.null(ref)) {
      if (length(methodvar) > 1) {
        ref <- .compact_method_columns(
          data = data,
          methodvar = methodvar
        )$reftable[[":methodvar"]][1]
      } else {
        ref <- levels(data[[methodvar]])[1]
        data[[methodvar]] <- relevel(data[[methodvar]], ref = ref)
      }
      message(paste(
        "'ref' method was not specified,",
        ref,
        "set as the reference"
      ))
    }
  }

  ### Throw a warning if `ref` is specified and `methodvar` is not
  if (is.null(methodvar) & !is.null(ref)) {
    warning(
      "'ref' method is specified while 'methodvar' is not: 'ref' will be ignored"
    )
    ref <- NULL
  }

  ### Split data by 'par'
  par_split <- .split_by(data = data, by = par)

  ### Call 'simsum' on each element of 'par_split'; save data if 'x = TRUE'
  par_simsum <- vector(mode = "list", length = length(par_split))
  if (x) {
    par_data <- vector(mode = "list", length = length(par_split))
  }
  for (i in seq_along(par_split)) {
    if (rlang::is_named(true) | is.null(true)) {
      run <- simsum(
        data = par_split[[i]],
        estvarname = estvarname,
        true = true[names(par_split)[i]],
        se = se,
        methodvar = methodvar,
        ref = ref,
        by = by,
        ci.limits = ci.limits,
        df = df,
        dropbig = dropbig,
        x = x,
        control = control
      )
    } else {
      run <- simsum(
        data = par_split[[i]],
        estvarname = estvarname,
        true = true,
        se = se,
        methodvar = methodvar,
        ref = ref,
        by = by,
        ci.limits = ci.limits,
        dropbig = dropbig,
        x = x,
        control = control
      )
    }
    par_simsum[[i]] <- run[["summ"]]
    if (x) par_data[[i]] <- run[["x"]]
  }
  names(par_simsum) <- names(par_split)

  ### Add a column with the parameter to each slot, and turn it into factor
  for (i in seq_along(par_simsum)) {
    par_simsum[[i]][[par]] <- names(par_simsum)[i]
  }

  ### Bind summ slots
  summ <- .br(x = par_simsum)
  summ <- .factorise(data = summ, cols = par)
  row.names(summ) <- NULL

  ### Include stuff into object to return
  obj <- list()
  obj$summ <- summ
  obj$par <- par
  obj$estvarname <- estvarname
  obj$true <- true
  obj$se <- se
  obj$methodvar <- methodvar
  obj$ref <- ref
  obj$dropbig <- dropbig
  obj$ci.limits <- ci.limits
  obj$df <- df
  obj$by <- by
  obj$control <- control
  if (x) {
    obj$x <- .br(par_data)
    rownames(obj$x) <- NULL
  }

  ### Return object of class simsum
  class(obj) <- c("multisimsum", "list")
  return(obj)
}

### New tests for issue #23
testthat::context("#28")

testthat::test_that("regex check new output options of multisimsum", {
  data("frailty", package = "rsimsum")
  frailty$true <- ifelse(frailty$par == "trt", -0.50, 0.75)
  ms0 <- multisimsum(
    data = frailty,
    par = "par",
    estvarname = "b",
    true = c(trt = -0.50, fv = 0.75)
  )
  ms1 <- multisimsum(
    data = frailty,
    par = "par",
    estvarname = "b",
    true = "true"
  )
  ms2 <- multisimsum(data = frailty, par = "par", estvarname = "b", true = 1)

  testthat::expect_output(
    object = print(ms0),
    regexp = "True values: fv = -0.5, trt = 0.75"
  )
  testthat::expect_output(
    object = print(ms1),
    regexp = "True values from column 'true'"
  )
  testthat::expect_output(
    object = print(ms2),
    regexp = "True values fixed at value 1 "
  )
})

testthat::test_that("regex check new output options of simsum", {
  data("MIsim")
  MIsim$true <- 0.5
  x0 <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  x1 <- simsum(
    data = MIsim,
    estvarname = "b",
    true = "true",
    se = "se",
    methodvar = "method"
  )

  testthat::expect_output(
    object = print(x0),
    regexp = "True value of the estimand: 0.5"
  )
  testthat::expect_output(
    object = print(x1),
    regexp = "True value of the estimand from column 'true'"
  )
})

testthat::context("autoplot")

data("MIsim", package = "rsimsum")
data("relhaz", package = "rsimsum")
single <- rsimsum::simsum(
  data = MIsim,
  estvarname = "b",
  true = 0.5,
  se = "se",
  methodvar = "method",
  ref = "CC",
  x = T
)
multi <- rsimsum::simsum(
  data = relhaz,
  estvarname = "theta",
  true = -0.5,
  se = "se",
  methodvar = "model",
  by = c("n", "baseline"),
  x = TRUE
)
singlesum <- summary(single)
multisum <- summary(multi)
data("nlp", package = "rsimsum")
nlps <- rsimsum::simsum(
  data = nlp,
  estvarname = "b",
  true = 0,
  se = "se",
  methodvar = "model",
  by = c("baseline", "ss", "esigma")
)
nlpssum <- summary(nlps)
data("MIsim2", package = "rsimsum")
multiplemethodvar <- rsimsum::simsum(
  data = MIsim2,
  estvarname = "b",
  true = 0.5,
  se = "se",
  methodvar = c("m1", "m2"),
  x = T
)
multiplemethodvarsum <- summary(multiplemethodvar)

testthat::test_that("output from autoplot is of class gg, ggplot", {
  # simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(single),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, type = "forest"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, type = "lolly"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, type = "est"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, type = "se"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, type = "est_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, type = "se_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, type = "est_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, type = "se_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, type = "est_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, type = "se_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, type = "est_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, type = "se_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, type = "heat"),
    class = c("gg", "ggplot")
  )
  # simsum object, with 'by'
  testthat::expect_s3_class(object = autoplot(multi), class = c("gg", "ggplot"))
  testthat::expect_s3_class(
    object = autoplot(multi, type = "forest"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, type = "lolly"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, type = "est"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, type = "se"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, type = "est_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, type = "se_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, type = "est_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, type = "se_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, type = "est_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, type = "se_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, type = "est_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, type = "se_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, type = "heat"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(singlesum),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "forest"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "lolly"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "est"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "se"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "est_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "se_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "est_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "se_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "est_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "se_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "est_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "se_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "heat"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multisum),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "forest"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "lolly"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "est"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "se"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "est_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "se_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "est_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "se_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "est_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "se_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "est_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "se_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "heat"),
    class = c("gg", "ggplot")
  )
  # nested loop plot
  testthat::expect_s3_class(
    object = autoplot(nlps, type = "nlp"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(nlpssum, type = "nlp"),
    class = c("gg", "ggplot")
  )
  # multiple columns identifying methods
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar, type = "forest"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar, type = "lolly"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar, type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar, type = "est"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar, type = "se"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar, type = "est_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar, type = "se_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar, type = "est_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar, type = "se_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar, type = "est_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar, type = "se_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar, type = "est_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar, type = "se_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvar, type = "heat"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum, type = "forest"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum, type = "lolly"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum, type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum, type = "est"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum, type = "se"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum, type = "est_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum, type = "se_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum, type = "est_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum, type = "se_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum, type = "est_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum, type = "se_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum, type = "est_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum, type = "se_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multiplemethodvarsum, type = "heat"),
    class = c("gg", "ggplot")
  )
})

testthat::test_that("argument checks works throws errors when appropriate", {
  # simsum object, no 'by'
  testthat::expect_error(
    object = print(autoplot(single, type = "megacool_plot"))
  )
  testthat::expect_error(object = print(autoplot(single, type = 1)))
  testthat::expect_error(object = print(autoplot(single, type = TRUE)))
  testthat::expect_error(object = print(autoplot(single, stats = "summary")))
  testthat::expect_error(object = print(autoplot(single, stats = "BIAS")))
  testthat::expect_error(object = print(autoplot(single, stats = 1)))
  testthat::expect_error(object = print(autoplot(single, stats = TRUE)))
  testthat::expect_error(object = print(autoplot(single, target = "top")))
  testthat::expect_error(object = print(autoplot(single, target = TRUE)))
  testthat::expect_error(object = print(autoplot(single, fitted = NULL)))
  testthat::expect_error(object = print(autoplot(single, fitted = 1)))
  testthat::expect_error(object = print(autoplot(single, fitted = "yes")))
  testthat::expect_error(object = print(autoplot(single, scales = "both")))
  testthat::expect_error(object = print(autoplot(single, scales = "none")))
  testthat::expect_error(object = print(autoplot(single, scales = "either")))
  testthat::expect_error(object = print(autoplot(single, top = 0)))
  testthat::expect_error(object = print(autoplot(single, top = "Yes!")))
  testthat::expect_error(object = print(autoplot(single, density.legend = 0)))
  testthat::expect_error(
    object = print(autoplot(single, density.legend = "Bamboozled"))
  )
  testthat::expect_error(object = print(autoplot(single, zoom = TRUE)))
  testthat::expect_error(object = print(autoplot(single, zoom = "Why not?")))
  # simsum object, with 'by'
  testthat::expect_error(
    object = print(autoplot(multi, type = "megacool_plot"))
  )
  testthat::expect_error(object = print(autoplot(multi, type = 1)))
  testthat::expect_error(object = print(autoplot(multi, type = TRUE)))
  testthat::expect_error(object = print(autoplot(multi, stats = "summary")))
  testthat::expect_error(object = print(autoplot(multi, stats = "BIAS")))
  testthat::expect_error(object = print(autoplot(multi, stats = 1)))
  testthat::expect_error(object = print(autoplot(multi, stats = TRUE)))
  testthat::expect_error(object = print(autoplot(multi, target = "top")))
  testthat::expect_error(object = print(autoplot(multi, target = TRUE)))
  testthat::expect_error(object = print(autoplot(multi, fitted = NULL)))
  testthat::expect_error(object = print(autoplot(multi, fitted = 1)))
  testthat::expect_error(object = print(autoplot(multi, fitted = "yes")))
  testthat::expect_error(object = print(autoplot(multi, scales = "both")))
  testthat::expect_error(object = print(autoplot(multi, scales = "none")))
  testthat::expect_error(object = print(autoplot(multi, scales = "either")))
  testthat::expect_error(object = print(autoplot(multi, top = 0)))
  testthat::expect_error(object = print(autoplot(multi, top = "Yes!")))
  testthat::expect_error(object = print(autoplot(multi, density.legend = 0)))
  testthat::expect_error(
    object = print(autoplot(multi, density.legend = "Bamboozled"))
  )
  testthat::expect_error(object = print(autoplot(multi, zoom = TRUE)))
  testthat::expect_error(object = print(autoplot(multi, zoom = "Why not?")))
  # summary.simsum object, no 'by'
  testthat::expect_error(
    object = print(autoplot(singlesum, type = "megacool_plot"))
  )
  testthat::expect_error(object = print(autoplot(singlesum, type = 1)))
  testthat::expect_error(object = print(autoplot(singlesum, type = TRUE)))
  testthat::expect_error(object = print(autoplot(singlesum, stats = "summary")))
  testthat::expect_error(object = print(autoplot(singlesum, stats = "BIAS")))
  testthat::expect_error(object = print(autoplot(singlesum, stats = 1)))
  testthat::expect_error(object = print(autoplot(singlesum, stats = TRUE)))
  testthat::expect_error(object = print(autoplot(singlesum, target = "top")))
  testthat::expect_error(object = print(autoplot(singlesum, target = TRUE)))
  testthat::expect_error(object = print(autoplot(singlesum, fitted = NULL)))
  testthat::expect_error(object = print(autoplot(singlesum, fitted = 1)))
  testthat::expect_error(object = print(autoplot(singlesum, fitted = "yes")))
  testthat::expect_error(object = print(autoplot(singlesum, scales = "both")))
  testthat::expect_error(object = print(autoplot(singlesum, scales = "none")))
  testthat::expect_error(object = print(autoplot(singlesum, scales = "either")))
  testthat::expect_error(object = print(autoplot(singlesum, top = 0)))
  testthat::expect_error(object = print(autoplot(singlesum, top = "Yes!")))
  testthat::expect_error(
    object = print(autoplot(singlesum, density.legend = 0))
  )
  testthat::expect_error(
    object = print(autoplot(singlesum, density.legend = "Bamboozled"))
  )
  testthat::expect_error(object = print(autoplot(singlesum, zoom = TRUE)))
  testthat::expect_error(object = print(autoplot(singlesum, zoom = "Why not?")))
  # summary.simsum object, with 'by'
  testthat::expect_error(
    object = print(autoplot(multisum, type = "megacool_plot"))
  )
  testthat::expect_error(object = print(autoplot(multisum, type = 1)))
  testthat::expect_error(object = print(autoplot(multisum, type = TRUE)))
  testthat::expect_error(object = print(autoplot(multisum, stats = "summary")))
  testthat::expect_error(object = print(autoplot(multisum, stats = "BIAS")))
  testthat::expect_error(object = print(autoplot(multisum, stats = 1)))
  testthat::expect_error(object = print(autoplot(multisum, stats = TRUE)))
  testthat::expect_error(object = print(autoplot(multisum, target = "top")))
  testthat::expect_error(object = print(autoplot(multisum, target = TRUE)))
  testthat::expect_error(object = print(autoplot(multisum, fitted = NULL)))
  testthat::expect_error(object = print(autoplot(multisum, fitted = 1)))
  testthat::expect_error(object = print(autoplot(multisum, fitted = "yes")))
  testthat::expect_error(object = print(autoplot(multisum, scales = "both")))
  testthat::expect_error(object = print(autoplot(multisum, scales = "none")))
  testthat::expect_error(object = print(autoplot(multisum, scales = "either")))
  testthat::expect_error(object = print(autoplot(multisum, top = 0)))
  testthat::expect_error(object = print(autoplot(multisum, top = "Yes!")))
  testthat::expect_error(object = print(autoplot(multisum, density.legend = 0)))
  testthat::expect_error(
    object = print(autoplot(multisum, density.legend = "Bamboozled"))
  )
  testthat::expect_error(object = print(autoplot(multisum, zoom = TRUE)))
  testthat::expect_error(object = print(autoplot(multisum, zoom = "Why not?")))
  # multiple columns identifying methods
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, type = "megacool_plot"))
  )
  testthat::expect_error(object = print(autoplot(multiplemethodvar, type = 1)))
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, type = TRUE))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, stats = "summary"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, stats = "BIAS"))
  )
  testthat::expect_error(object = print(autoplot(multiplemethodvar, stats = 1)))
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, stats = TRUE))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, target = "top"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, target = TRUE))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, fitted = NULL))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, fitted = 1))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, fitted = "yes"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, scales = "both"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, scales = "none"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, scales = "either"))
  )
  testthat::expect_error(object = print(autoplot(multiplemethodvar, top = 0)))
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, top = "Yes!"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, density.legend = 0))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, density.legend = "Bamboozled"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, zoom = TRUE))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvar, zoom = "Why not?"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, type = "megacool_plot"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, type = 1))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, type = TRUE))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, stats = "summary"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, stats = "BIAS"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, stats = 1))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, stats = TRUE))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, target = "top"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, target = TRUE))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, fitted = NULL))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, fitted = 1))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, fitted = "yes"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, scales = "both"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, scales = "none"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, scales = "either"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, top = 0))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, top = "Yes!"))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, density.legend = 0))
  )
  testthat::expect_error(
    object = print(autoplot(
      multiplemethodvarsum,
      density.legend = "Bamboozled"
    ))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, zoom = TRUE))
  )
  testthat::expect_error(
    object = print(autoplot(multiplemethodvarsum, zoom = "Why not?"))
  )
})

testthat::test_that("autoplot with target", {
  # simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(single, target = 1),
    class = c("gg", "ggplot")
  )
  # simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multi, target = 1),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(singlesum, target = 1),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multisum, target = 1),
    class = c("gg", "ggplot")
  )
})

testthat::test_that("autoplot with stats", {
  # simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(single, stats = "cover"),
    class = c("gg", "ggplot")
  )
  # simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multi, stats = "cover"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(singlesum, stats = "cover"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multisum, stats = "cover"),
    class = c("gg", "ggplot")
  )
})

testthat::test_that("autoplot with target, stats", {
  # simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(single, target = 0.50, stats = "cover"),
    class = c("gg", "ggplot")
  )
  # simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multi, target = 0.50, stats = "cover"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(singlesum, target = 0.50, stats = "cover"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multisum, target = 0.50, stats = "cover"),
    class = c("gg", "ggplot")
  )
})

testthat::test_that("autoplot with fitted", {
  # simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(single, fitted = TRUE),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, fitted = FALSE),
    class = c("gg", "ggplot")
  )
  # simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multi, fitted = TRUE),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, fitted = FALSE),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(singlesum, fitted = TRUE),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, fitted = FALSE),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multisum, fitted = TRUE),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, fitted = FALSE),
    class = c("gg", "ggplot")
  )
})

testthat::test_that("autoplot with scales", {
  # simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(single, scales = "fixed"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, scales = "free"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, scales = "free_x"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(single, scales = "free_y"),
    class = c("gg", "ggplot")
  )
  # simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multi, scales = "fixed"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, scales = "free"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, scales = "free_x"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, scales = "free_y"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(singlesum, scales = "fixed"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, scales = "free"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, scales = "free_x"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, scales = "free_y"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multisum, scales = "fixed"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, scales = "free"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, scales = "free_x"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, scales = "free_y"),
    class = c("gg", "ggplot")
  )
})

testthat::test_that("autoplot with top", {
  # nested loop plot
  testthat::expect_s3_class(
    object = autoplot(nlps, type = "nlp", top = TRUE),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(nlps, type = "nlp", top = FALSE),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(nlpssum, type = "nlp", top = TRUE),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(nlpssum, type = "nlp", top = FALSE),
    class = c("gg", "ggplot")
  )
})

testthat::test_that("autoplot with density.legend", {
  # density plot
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "est_density", density.legend = TRUE),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "se_density", density.legend = TRUE),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "est_density", density.legend = FALSE),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "se_density", density.legend = FALSE),
    class = c("gg", "ggplot")
  )
  # hex plot
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "est_hex", density.legend = TRUE),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "se_hex", density.legend = TRUE),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "est_hex", density.legend = FALSE),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "se_hex", density.legend = FALSE),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multisum, scales = "fixed"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, scales = "free"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, scales = "free_x"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, scales = "free_y"),
    class = c("gg", "ggplot")
  )
})

testthat::test_that("autoplot with zoom", {
  # nested loop plot
  testthat::expect_s3_class(
    object = autoplot(single, type = "zip", zoom = 0.5),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "zip", zoom = 0.5),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, type = "zip", zoom = 0.5),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, type = "zip", zoom = 0.5),
    class = c("gg", "ggplot")
  )
})

testthat::test_that("inferring target", {
  # simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(single, stats = "bias"),
    class = c("gg", "ggplot")
  )
  # simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multi, stats = "bias"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(singlesum, stats = "bias"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multisum, stats = "bias"),
    class = c("gg", "ggplot")
  )
  # simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(single, stats = "rbias"),
    class = c("gg", "ggplot")
  )
  # simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multi, stats = "rbias"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(singlesum, stats = "rbias"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multisum, stats = "rbias"),
    class = c("gg", "ggplot")
  )
  # simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(single, stats = "cover"),
    class = c("gg", "ggplot")
  )
  # simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multi, stats = "cover"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(singlesum, stats = "cover"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multisum, stats = "cover"),
    class = c("gg", "ggplot")
  )
  # simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(single, stats = "becover"),
    class = c("gg", "ggplot")
  )
  # simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multi, stats = "becover"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(singlesum, stats = "becover"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multisum, stats = "becover"),
    class = c("gg", "ggplot")
  )
  # simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(single, stats = "power"),
    class = c("gg", "ggplot")
  )
  # simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multi, stats = "power"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(singlesum, stats = "power"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multisum, stats = "power"),
    class = c("gg", "ggplot")
  )
  # simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(single, stats = "thetamean"),
    class = c("gg", "ggplot")
  )
  # simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multi, stats = "thetamean"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(singlesum, stats = "thetamean"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multisum, stats = "thetamean"),
    class = c("gg", "ggplot")
  )
  # simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(single, stats = "thetamedian"),
    class = c("gg", "ggplot")
  )
  # simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multi, stats = "thetamedian"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(singlesum, stats = "thetamedian"),
    class = c("gg", "ggplot")
  )
  # summary.simsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(multisum, stats = "thetamedian"),
    class = c("gg", "ggplot")
  )
})

testthat::test_that("zip with t critical values", {
  data("tt", package = "rsimsum")
  tt1 <- tt2 <- tt
  tt1$yo <- "One"
  tt2$yo <- "Two"
  doublett <- do.call(rbind.data.frame, list(tt1, tt2))

  single <- rsimsum::simsum(
    data = tt,
    estvarname = "diff",
    true = -1,
    se = "se",
    methodvar = "method",
    x = TRUE,
    df = "df"
  )
  multi <- rsimsum::multisimsum(
    data = doublett,
    par = "yo",
    estvarname = "diff",
    true = c(One = -1, Two = -1),
    se = "se",
    methodvar = "method",
    x = TRUE,
    df = "df"
  )
  singlesum <- summary(single)
  multisum <- summary(multi)

  testthat::expect_s3_class(
    object = autoplot(single, type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, par = "One", type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multi, par = "Two", type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(singlesum, type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, par = "One", type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(multisum, par = "Two", type = "zip"),
    class = c("gg", "ggplot")
  )
})

data("frailty", package = "rsimsum")
ms <- rsimsum::multisimsum(
  data = frailty,
  par = "par",
  true = c(trt = -0.50, fv = 0.75),
  estvarname = "b",
  se = "se",
  methodvar = "model",
  x = TRUE
)
sms <- summary(ms)
ms2 <- rsimsum::multisimsum(
  data = frailty,
  par = "par",
  true = c(trt = -0.50, fv = 0.75),
  estvarname = "b",
  se = "se",
  methodvar = "model",
  by = "fv_dist",
  x = TRUE
)
sms2 <- summary(ms2)

testthat::test_that("output from autoplot is of class gg, ggplot", {
  # multisimsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt", type = "forest"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt", type = "lolly"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt", type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt", type = "est"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt", type = "se"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt", type = "est_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt", type = "se_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt", type = "est_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt", type = "se_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt", type = "est_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt", type = "se_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt", type = "est_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt", type = "se_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms, par = "trt", type = "heat"),
    class = c("gg", "ggplot")
  )
  # multisimsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "forest"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "lolly"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "est"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "se"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "est_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "se_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "est_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "se_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "est_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "se_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "est_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "se_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "heat"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(ms2, par = "trt", type = "nlp"),
    class = c("gg", "ggplot")
  )
  # summary.multisimsum object, no 'by'
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt", type = "forest"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt", type = "lolly"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt", type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt", type = "est"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt", type = "se"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt", type = "est_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt", type = "se_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt", type = "est_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt", type = "se_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt", type = "est_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt", type = "se_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt", type = "est_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt", type = "se_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms, par = "trt", type = "heat"),
    class = c("gg", "ggplot")
  )
  # summary.multisimsum object, with 'by'
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "forest"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "lolly"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "est"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "se"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "est_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "se_ba"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "est_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "se_density"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "est_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "se_hex"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "est_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "se_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "heat"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(sms2, par = "trt", type = "nlp"),
    class = c("gg", "ggplot")
  )
})

testthat::test_that("putting wrong 'par' (or no 'par' at all) throws an error", {
  # multisimsum object, no 'by'
  testthat::expect_error(object = autoplot(ms))
  testthat::expect_error(object = autoplot(ms, par = "42"))
  testthat::expect_error(object = autoplot(ms, par = 42))
  testthat::expect_error(object = autoplot(ms, par = TRUE))
  # multisimsum object, with 'by'
  testthat::expect_error(object = autoplot(ms2))
  testthat::expect_error(object = autoplot(ms2, par = "42"))
  testthat::expect_error(object = autoplot(ms2, par = 42))
  testthat::expect_error(object = autoplot(ms2, par = TRUE))
  # summary.multisimsum object, no 'by'
  testthat::expect_error(object = autoplot(sms))
  testthat::expect_error(object = autoplot(sms, par = "42"))
  testthat::expect_error(object = autoplot(sms, par = 42))
  testthat::expect_error(object = autoplot(sms, par = TRUE))
  # summary.multisimsum object, with 'by'
  testthat::expect_error(object = autoplot(sms2))
  testthat::expect_error(object = autoplot(sms2, par = "42"))
  testthat::expect_error(object = autoplot(sms2, par = 42))
  testthat::expect_error(object = autoplot(sms2, par = TRUE))
})


testthat::test_that("nlp with no 'by' factors throw an error", {
  testthat::expect_error(
    object = autoplot(ms, par = "trt", type = "nlp"),
    regexp = "Nested loop plot not meaningful"
  )
})


### Test better handling of edge cases...
testthat::test_that("Edge cases for rsimsum >= 0.8.0", {
  data("tt", package = "rsimsum")
  tt$true <- -1
  s1 <- simsum(data = tt, estvarname = "diff", x = TRUE)
  s2 <- simsum(data = tt, estvarname = "diff", se = "se", x = TRUE)
  s3 <- simsum(data = tt, estvarname = "diff", true = -1, x = TRUE)
  s4 <- simsum(data = tt, estvarname = "diff", true = "true", x = TRUE)
  s5 <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    true = "true",
    x = TRUE
  )

  testthat::expect_error(object = autoplot(s1, type = "zip"))
  testthat::expect_error(object = autoplot(s2, type = "zip"))
  testthat::expect_error(object = autoplot(s3, type = "zip"))
  testthat::expect_error(object = autoplot(s4, type = "zip"))
  testthat::expect_error(object = autoplot(s5, type = "zip"))

  testthat::expect_error(object = autoplot(s1, type = "se"))
  testthat::expect_error(object = autoplot(s1, type = "se_ba"))
  testthat::expect_error(object = autoplot(s1, type = "se_ridge"))
  testthat::expect_error(object = autoplot(s1, type = "se_density"))
  testthat::expect_error(object = autoplot(s1, type = "se_hex"))

  testthat::expect_error(
    object = autoplot(s1, type = "lolly", stats = "thetamean")
  )
  testthat::expect_error(
    object = autoplot(s1, type = "lolly", stats = "thetamedian")
  )
  testthat::expect_error(
    object = autoplot(s4, type = "lolly", stats = "thetamean")
  )
  testthat::expect_error(
    object = autoplot(s4, type = "lolly", stats = "thetamedian")
  )
})

### More tests for zip plots...
testthat::test_that("Testing more zip plot cases", {
  data("tt", package = "rsimsum")

  s <- simsum(data = tt, estvarname = "diff", se = "se", true = -1, x = TRUE)
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )
  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    df = "df",
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )
  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    ci.limits = c("conf.low", "conf.high"),
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )

  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    methodvar = "method",
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )
  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    methodvar = "method",
    df = "df",
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )
  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    methodvar = "method",
    ci.limits = c("conf.low", "conf.high"),
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )

  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    by = "dgm",
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )
  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    by = "dgm",
    df = "df",
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )
  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    by = "dgm",
    ci.limits = c("conf.low", "conf.high"),
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )

  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    methodvar = "method",
    by = "dgm",
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )
  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    methodvar = "method",
    by = "dgm",
    df = "df",
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )
  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    methodvar = "method",
    by = "dgm",
    ci.limits = c("conf.low", "conf.high"),
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )

  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )
  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    ci.limits = c(-2, 0),
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )
  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    methodvar = "method",
    ci.limits = c(-2, 0),
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )
  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    by = "dgm",
    ci.limits = c(-2, 0),
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )
  s <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    methodvar = "method",
    by = "dgm",
    ci.limits = c(-2, 0),
    true = -1,
    x = TRUE
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )
})

testthat::context("tidy")

testthat::test_that("tidy returns a data.frame", {
  data("MIsim", package = "rsimsum")
  x <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  testthat::expect_s3_class(object = tidy(x), class = "data.frame")
  xs <- summary(x)
  testthat::expect_s3_class(object = tidy(xs), class = "data.frame")
  data("frailty", package = "rsimsum")
  ms <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist"
  )
  testthat::expect_s3_class(object = tidy(ms), class = "data.frame")
  sms <- summary(ms)
  testthat::expect_s3_class(object = tidy(sms), class = "data.frame")
})

testthat::test_that("tidy with 'stats' results only 'stats'", {
  data("MIsim", package = "rsimsum")
  x <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  out <- rsimsum::tidy(x, stats = "bias")
  testthat::expect_true(object = all(out$stat == "bias"))
  out <- rsimsum::tidy(x, stats = c("bias", "cover"))
  testthat::expect_true(object = all(out$stat %in% c("bias", "cover")))
})

testthat::test_that("tidy with wrong 'stats' throws an error", {
  data("MIsim", package = "rsimsum")
  x <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  testthat::expect_error(object = rsimsum::tidy(x, stats = "42"))
  testthat::expect_error(object = rsimsum::tidy(x, stats = TRUE))
  testthat::expect_error(object = rsimsum::tidy(x, stats = 42))
})

### New tests for PR #31
testthat::context("#31")

testthat::test_that("Splitting and not-splitting 'method' in MIsim leads to the same results", {
  data(MIsim, package = "rsimsum")
  data(MIsim2, package = "rsimsum")
  s1 <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  s2 <- simsum(
    data = MIsim2,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = c("m1", "m2")
  )
  testthat::expect_identical(object = tidy(s1)$stat, expected = tidy(s2)$stat)
  testthat::expect_identical(object = tidy(s1)$est, expected = tidy(s2)$est)
  testthat::expect_identical(object = tidy(s1)$mcse, expected = tidy(s2)$mcse)
})

testthat::test_that("multisimsum with multiple columns as 'methodvar' yields same results", {
  data("frailty")
  s1 <- multisimsum(
    data = frailty,
    par = "par",
    estvarname = "b",
    methodvar = "model",
    by = "fv_dist"
  )
  data("frailty2")
  s2 <- multisimsum(
    data = frailty2,
    par = "par",
    estvarname = "b",
    methodvar = c("m_baseline", "m_frailty"),
    by = "fv_dist"
  )
  testthat::expect_equal(object = tidy(s2)$est, expected = tidy(s1)$est)
  testthat::expect_equal(object = tidy(s2)$mcse, expected = tidy(s1)$mcse)
})

testthat::test_that("New code prints ok", {
  data(MIsim2, package = "rsimsum")
  s1 <- simsum(
    data = MIsim2,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = c("m1", "m2")
  )
  testthat::expect_output(print(s1))
  testthat::expect_output(print(summary(s1)))

  data("frailty2")
  s2 <- multisimsum(
    data = frailty2,
    par = "par",
    estvarname = "b",
    methodvar = c("m_baseline", "m_frailty"),
    by = "fv_dist"
  )
  testthat::expect_output(print(s2))
  testthat::expect_output(print(summary(s2)))
})

### New tests for issue #15
testthat::context("#15")

testthat::test_that("simsum with 'true' as a column returns the same results as using a scalar value if each row has the same value", {
  data(tt, package = "rsimsum")
  tt$true <- -1

  s1 <- simsum(
    data = tt,
    estvarname = "diff",
    true = "true",
    se = "se",
    ci.limits = c("conf.low", "conf.high"),
    methodvar = "method",
    by = "dgm"
  )
  s2 <- simsum(
    data = tt,
    estvarname = "diff",
    true = -1,
    se = "se",
    ci.limits = c("conf.low", "conf.high"),
    methodvar = "method",
    by = "dgm"
  )
  testthat::expect_identical(object = tidy(s1), expected = tidy(s2))

  s1 <- simsum(
    data = tt,
    estvarname = "diff",
    true = "true",
    se = "se",
    methodvar = "method",
    by = "dgm"
  )
  s2 <- simsum(
    data = tt,
    estvarname = "diff",
    true = -1,
    se = "se",
    methodvar = "method",
    by = "dgm"
  )
  testthat::expect_identical(object = tidy(s1), expected = tidy(s2))

  s1 <- simsum(
    data = tt,
    estvarname = "diff",
    true = "true",
    se = "se",
    ci.limits = c(-1.5, -0.5),
    methodvar = "method",
    by = "dgm"
  )
  s2 <- simsum(
    data = tt,
    estvarname = "diff",
    true = -1,
    se = "se",
    ci.limits = c(-1.5, -0.5),
    methodvar = "method",
    by = "dgm"
  )
  testthat::expect_identical(object = tidy(s1), expected = tidy(s2))
})

testthat::test_that("simsum with 'true' as a column returns different results compared to using a scalar value if each row has a different value", {
  data(tt, package = "rsimsum")
  tt$true <- stats::rnorm(n = nrow(tt), mean = -1, sd = 1)

  s1 <- simsum(
    data = tt,
    estvarname = "diff",
    true = "true",
    se = "se",
    ci.limits = c("conf.low", "conf.high"),
    methodvar = "method",
    by = "dgm"
  )
  s2 <- simsum(
    data = tt,
    estvarname = "diff",
    true = -1,
    se = "se",
    ci.limits = c("conf.low", "conf.high"),
    methodvar = "method",
    by = "dgm"
  )
  testthat::expect_false(object = identical(tidy(s1), tidy(s2)))

  s1 <- simsum(
    data = tt,
    estvarname = "diff",
    true = "true",
    se = "se",
    methodvar = "method",
    by = "dgm"
  )
  s2 <- simsum(
    data = tt,
    estvarname = "diff",
    true = -1,
    se = "se",
    methodvar = "method",
    by = "dgm"
  )
  testthat::expect_false(object = identical(tidy(s1), tidy(s2)))

  s1 <- simsum(
    data = tt,
    estvarname = "diff",
    true = "true",
    se = "se",
    ci.limits = c(-1.5, -0.5),
    methodvar = "method",
    by = "dgm"
  )
  s2 <- simsum(
    data = tt,
    estvarname = "diff",
    true = -1,
    se = "se",
    ci.limits = c(-1.5, -0.5),
    methodvar = "method",
    by = "dgm"
  )
  testthat::expect_false(object = identical(tidy(s1), tidy(s2)))
})

testthat::test_that("multisimsum with 'true' as a column returns the same results as using a scalar value if each row has the same value", {
  data(tt, package = "rsimsum")
  tt$true <- -1
  tt <- dplyr::bind_rows(
    dplyr::mutate(tt, par = "diff1"),
    dplyr::mutate(tt, diff = diff + 1, true = true + 1, par = "diff2")
  )

  m1 <- multisimsum(
    data = tt,
    estvarname = "diff",
    par = "par",
    true = c(diff1 = -1, diff2 = 0),
    se = "se",
    ci.limits = c("conf.low", "conf.high"),
    methodvar = "method",
    by = "dgm"
  )
  m2 <- multisimsum(
    data = tt,
    estvarname = "diff",
    par = "par",
    true = "true",
    se = "se",
    ci.limits = c("conf.low", "conf.high"),
    methodvar = "method",
    by = "dgm"
  )
  testthat::expect_identical(object = tidy(m1), expected = tidy(m2))
})

testthat::test_that("multisimsum with 'true' as a column returns different results compared to using a scalar value if each row has a different value", {
  data(tt, package = "rsimsum")
  tt$true <- stats::rnorm(n = tt, mean = -1, sd = 1)
  tt <- dplyr::bind_rows(
    dplyr::mutate(tt, par = "diff1"),
    dplyr::mutate(tt, diff = diff + 1, true = true + 1, par = "diff2")
  )

  m1 <- multisimsum(
    data = tt,
    estvarname = "diff",
    par = "par",
    true = c(diff1 = -1, diff2 = 0),
    se = "se",
    ci.limits = c("conf.low", "conf.high"),
    methodvar = "method",
    by = "dgm"
  )
  m2 <- multisimsum(
    data = tt,
    estvarname = "diff",
    par = "par",
    true = "true",
    se = "se",
    ci.limits = c("conf.low", "conf.high"),
    methodvar = "method",
    by = "dgm"
  )
  testthat::expect_false(object = identical(tidy(m1), tidy(m2)))
})

test_that("Invisible returns of expected size for the single-parameter case", {
  data("relhaz", package = "rsimsum")
  s2 <- simsum(
    data = relhaz,
    estvarname = "theta",
    true = -0.50,
    se = "se",
    methodvar = "model",
    by = c("baseline", "n")
  )
  out <- print(summary(s2, stats = "bias"))
  testthat::expect_equal(object = length(out), expected = 1)
  out <- print(summary(s2, stats = c("bias", "cover")))
  testthat::expect_equal(object = length(out), expected = 2)
  out <- print(summary(s2, stats = c("bias", "cover", "mse")))
  testthat::expect_equal(object = length(out), expected = 3)
})

test_that("Invisible returns of expected size for the multiple-parameter case", {
  data("frailty", package = "rsimsum")
  ms <- multisimsum(
    data = frailty,
    par = "par",
    true = c(
      trt = -0.50,
      fv = 0.75
    ),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist"
  )
  out <- print(summary(ms, stats = "bias"))
  testthat::expect_equal(
    object = length(out),
    expected = length(unique(frailty$par))
  )
  for (i in seq_along(out)) {
    testthat::expect_equal(object = length(out[[i]]), expected = 1)
  }
  out <- print(summary(ms, stats = c("bias", "cover")))
  testthat::expect_equal(
    object = length(out),
    expected = length(unique(frailty$par))
  )
  for (i in seq_along(out)) {
    testthat::expect_equal(object = length(out[[i]]), expected = 2)
  }
  out <- print(summary(ms, stats = c("bias", "cover", "mse")))
  testthat::expect_equal(
    object = length(out),
    expected = length(unique(frailty$par))
  )
  for (i in seq_along(out)) {
    testthat::expect_equal(object = length(out[[i]]), expected = 3)
  }
})

testthat::context("multisimsum")

testthat::test_that("multisimsum prints ok", {
  data("frailty")
  testthat::expect_output(print(multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist"
  )))
  testthat::expect_output(print(multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    by = "fv_dist"
  )))
  testthat::expect_output(print(multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model"
  )))
  testthat::expect_output(print(multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se"
  )))
  testthat::expect_output(print(multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist",
    control = list(mcse = FALSE)
  )))
  testthat::expect_output(print(multisimsum(
    data = frailty,
    par = "par",
    estvarname = "b",
    se = "se"
  )))
})

testthat::test_that("multisimsum returns an object of class multisimsum", {
  data("frailty")
  ms <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist"
  )
  testthat::expect_s3_class(ms, "multisimsum")
})

testthat::test_that("summ slot of a multisimsum object is a data.frame", {
  data("frailty")
  ms <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist"
  )
  testthat::expect_s3_class(ms$summ, "data.frame")
})

testthat::test_that("not passing estvarname throws an error", {
  testthat::expect_error(
    {
      data("frailty")
      ms <- multisimsum(
        data = frailty,
        par = "par",
        true = c(trt = -0.50, fv = 0.75),
        se = "se",
        methodvar = "model",
        by = "fv_dist"
      )
    },
    'argument "estvarname" is missing, with no default'
  )
})

testthat::test_that("specifying ref and not methodvar throws a warning", {
  testthat::expect_warning(
    {
      data("frailty")
      ms <- multisimsum(
        data = frailty,
        par = "par",
        true = c(trt = -0.50, fv = 0.75),
        estvarname = "b",
        se = "se",
        ref = "Cox, Gamma",
        by = "fv_dist"
      )
    },
    "'ref' method is specified while 'methodvar' is not: 'ref' will be ignored"
  )
})

testthat::test_that("specifying methodvar and not ref shows a message", {
  testthat::expect_message(
    {
      data("frailty")
      ms <- multisimsum(
        data = frailty,
        par = "par",
        true = c(trt = -0.50, fv = 0.75),
        estvarname = "b",
        se = "se",
        methodvar = "model",
        by = "fv_dist"
      )
    },
    "'ref' method was not specified, Cox, Gamma set as the reference"
  )
})

testthat::test_that("running multisimsum on frailty return summaries of the correct dimension", {
  data("frailty")
  ms <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist"
  )
  testthat::expect_equal(
    dim(ms$summ),
    expected = c(
      15 *
        length(unique(frailty$par)) *
        length(unique(frailty$fv_dist)) *
        length(unique(frailty$model)),
      6
    )
  )
})

testthat::test_that("multisimsum with mcse option returns mcse", {
  data("frailty")
  ms <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist",
    control = list(mcse = TRUE)
  )
  testthat::expect_true("mcse" %in% names(ms$summ))
})

testthat::test_that("multisimsum without mcse option does not returns mcse", {
  data("frailty")
  ms <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist",
    control = list(mcse = FALSE)
  )
  testthat::expect_false("mcse" %in% names(ms$summ))
})

testthat::test_that("multisimsum with by factors returns error when 'by' name is not a variable in data", {
  testthat::expect_error({
    data("frailty")
    ms <- multisimsum(
      data = frailty,
      par = "par",
      true = c(trt = -0.50, fv = 0.75),
      estvarname = "b",
      se = "se",
      methodvar = "model",
      by = "not_in_data"
    )
  })
})

testthat::test_that("multisimsum with by factors returns a data.frame with results", {
  data("frailty")
  ms <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist"
  )
  testthat::expect_s3_class(object = ms$summ, class = "data.frame")
})

testthat::test_that("multisimsum with x = FALSE does not return data", {
  data("frailty")
  s <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist",
    x = FALSE
  )
  testthat::expect_null(object = s$x)
})

testthat::test_that("multisimsum with x = TRUE does return data", {
  data("frailty")
  s <- multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist",
    x = TRUE
  )
  testthat::expect_s3_class(object = s$x, class = "data.frame")
  testthat::expect_equal(object = nrow(s$x), expected = nrow(frailty))
})

testthat::test_that("multisimsum adds ci.limits if requested", {
  data("frailty")
  s <- suppressWarnings(multisimsum(
    data = frailty,
    par = "par",
    true = c(trt = -0.50, fv = 0.75),
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist",
    ci.limits = c(-0.55, -0.45)
  ))
  testthat::expect_equal(object = s$ci.limits, expected = c(-0.55, -0.45))
})

testthat::test_that("multisimsum argument checks", {
  data("frailty")
  testthat::expect_error(
    object = rsimsum::multisimsum(
      data = frailty,
      par = 1,
      true = c(trt = -0.50, fv = 0.75),
      estvarname = "b",
      se = "se",
      methodvar = "model",
      by = "fv_dist"
    )
  )
  testthat::expect_error(
    object = rsimsum::multisimsum(
      data = frailty,
      par = TRUE,
      true = c(trt = -0.50, fv = 0.75),
      estvarname = "b",
      se = "se",
      methodvar = "model",
      by = "fv_dist"
    )
  )
  testthat::expect_error(
    object = rsimsum::multisimsum(
      data = frailty,
      par = NULL,
      true = c(trt = -0.50, fv = 0.75),
      estvarname = "b",
      se = "se",
      methodvar = "model",
      by = "fv_dist"
    )
  )
  testthat::expect_error(
    object = rsimsum::multisimsum(
      data = frailty,
      par = "par",
      true = c(1, 1),
      estvarname = "b",
      se = "se",
      methodvar = "model",
      by = "fv_dist"
    )
  )
  testthat::expect_error(
    object = rsimsum::multisimsum(
      data = frailty,
      par = "par",
      true = TRUE,
      estvarname = "b",
      se = "se",
      methodvar = "model",
      by = "fv_dist"
    )
  )
})

testthat::test_that("multisimsum without 'true' does not compute bias, rbias, cover, mse", {
  data("frailty")
  s <- multisimsum(
    data = frailty,
    par = "par",
    estvarname = "b",
    se = "se",
    methodvar = "model",
    by = "fv_dist"
  )
  testthat::expect_false(
    object = any(c("bias", "rbias", "cover", "mse") %in% s$summ$stat)
  )
})

testthat::test_that("multisimsum without 'se' does not compute se2mean, se2median, modelse, relerror, cover, becover, power", {
  data("frailty")
  s <- multisimsum(
    data = frailty,
    par = "par",
    estvarname = "b",
    true = c(trt = -0.50, fv = 0.75),
    methodvar = "model",
    by = "fv_dist"
  )
  testthat::expect_false(
    object = any(
      c(
        "se2mean",
        "se2median",
        "modelse",
        "relerror",
        "cover",
        "becover",
        "power"
      ) %in%
        s$summ$stat
    )
  )
})

testthat::test_that("multisimsum without 'se' nor 'true' does not compute se2mean, se2median, modelse, relerror, cover, becover, power, bias, rbias, mse", {
  data("frailty")
  s <- multisimsum(
    data = frailty,
    par = "par",
    estvarname = "b",
    methodvar = "model",
    by = "fv_dist"
  )
  testthat::expect_false(
    object = any(
      c(
        "se2mean",
        "se2median",
        "modelse",
        "relerror",
        "cover",
        "becover",
        "power",
        "bias",
        "rbias",
        "mse"
      ) %in%
        s$summ$stat
    )
  )
})

#' @title autoplot method for multisimsum objects
#' @description `autoplot` can produce a series of plot to summarise results of simulation studies. See `vignette("C-plotting", package = "rsimsum")` for further details.
#' @param object An object of class `multisimsum`.
#' @param par The parameter results to plot.
#' @param type The type of the plot to be produced. Possible choices are: `forest`, `lolly`, `zip`, `est`, `se`, `est_ba`, `se_ba`, `est_density`, `se_density`, `est_hex`, `se_hex`, `est_ridge`, `se_ridge`, `heat`, `nlp`, with `forest` being the default.
#' @param stats Summary statistic to plot, defaults to `bias`. See [summary.simsum()] for further details on supported summary statistics.
#' @param target Target of summary statistic, e.g. 0 for `bias`. Defaults to `NULL`, in which case target will be inferred.
#' @param fitted Superimpose a fitted regression line, useful when `type` = (`est`, `se`, `est_ba`, `se_ba`, `est_density`, `se_density`, `est_hex`, `se_hex`). Defaults to `TRUE`.
#' @param scales Should scales be fixed (`fixed`, the default), free (`free`), or free in one dimension (`free_x`, `free_y`)?
#' @param top Should the legend for a nested loop plot be on the top side of the plot? Defaults to `TRUE`.
#' @param density.legend Should the legend for density and hexbin plots be included? Defaults to `TRUE`.
#' @param zoom A numeric value between 0 and 1 signalling that a zip plot should _zoom_ on the top x% of the plot (to ease interpretation). Defaults to 1, where the whole zip plot is displayed.
#' @param zip_ci_colours A string with (1) a hex code to use for plotting coverage probability and its Monte Carlo confidence intervals (the default, with value `zip_ci_colours = "yellow"`), (2) a string vector of two hex codes denoting optimal coverage (first element) and over/under coverage (second element) or (3) a vector of three hex codes denoting optimal coverage (first), undercoverage (second), and overcoverage (third).
#' @param ... Not used.
#'
#' @return A `ggplot` object.
#' @export
#'
#' @examples
#' data("frailty", package = "rsimsum")
#' ms <- multisimsum(
#'   data = frailty,
#'   par = "par", true = c(trt = -0.50, fv = 0.75),
#'   estvarname = "b", se = "se", methodvar = "model",
#'   by = "fv_dist", x = TRUE
#' )
#'
#' library(ggplot2)
#' autoplot(ms, par = "trt")
#' autoplot(ms, par = "trt", type = "lolly", stats = "cover")
#' autoplot(ms, par = "trt", type = "zip")
#' autoplot(ms, par = "trt", type = "est_ba")
autoplot.multisimsum <- function(
  object,
  par,
  type = "forest",
  stats = "nsim",
  target = NULL,
  fitted = TRUE,
  scales = "fixed",
  top = TRUE,
  density.legend = TRUE,
  zoom = 1,
  zip_ci_colours = "yellow",
  ...
) {
  ### Manipulate object to make it a simsum-like object
  object[["summ"]] <- object[["summ"]][
    object[["summ"]][[object[["par"]]]] == par,
  ]
  object[["true"]] <- object[["true"]][par]
  if (
    type %in%
      c(
        "zip",
        "est",
        "se",
        "est_ba",
        "se_ba",
        "est_density",
        "se_density",
        "est_hex",
        "se_hex",
        "est_ridge",
        "se_ridge"
      )
  ) {
    object[["x"]] <- object[["x"]][object[["x"]][[object[["par"]]]] == par, ]
  }
  #
  if (inherits(object, "summary.multisimsum")) {
    class(object) <- c("summary.simsum", "list")
  } else {
    class(object) <- c("simsum", "list")
  }

  # Call autoplot.simsum on the subset of results for a given parameter:
  plot <- autoplot(
    object = object,
    type = type,
    stats = stats,
    target = target,
    fitted = fitted,
    scales = scales,
    top = top,
    density.legend = density.legend,
    zoom = zoom,
    zip_ci_colours = zip_ci_colours,
    ...
  )

  # Add title with parameter of interest
  plot <- plot +
    ggplot2::labs(title = paste0("Parameter: ", par))

  ### Return plot
  return(plot)
}

#' @title Identify replications with large point estimates, standard errors
#' @description `dropbig` is useful to identify replications with large point estimates or standard errors. Large values are defined as standardised values above a given threshold, as defined when calling `dropbig`. Regular standardisation using mean and standard deviation is implemented, as well as robust standardisation using median and inter-quartile range. Further to that, the  standardisation process is stratified by data-generating mechanism if `by` factors are defined.
#'
#' @param data A `data.frame` in which variable names are interpreted. It has to be in tidy format, e.g. each variable forms a column and each observation forms a row.
#' @param estvarname The name of the variable containing the point estimates.
#' @param se The name of the variable containing the standard errors of the point estimates.
#' @param methodvar The name of the variable containing the methods to compare. For instance, methods could be the models compared within a simulation study. Can be `NULL`.
#' @param by A vector of variable names to compute performance measures by a list of factors. Factors listed here are the (potentially several) data-generating mechanisms used to simulate data under different scenarios (e.g. sample size, true distribution of a variable, etc.). Can be `NULL`.
#' @param max Specifies the maximum acceptable absolute value of the point estimates, after standardisation. Defaults to 10.
#' @param semax Specifies the maximum acceptable absolute value of the standard error, after standardisation. Defaults to 100.
#' @param robust Specifies whether to use robust standardisation (using median and inter-quartile range) rather than normal standardisation (using mean and standard deviation). Defaults to `TRUE`.
#'
#' @return The same `data.frame` given as input with an additional column named `.dropbig` identifying rows that are classified as large (`.dropbig = TRUE`) according to the specified criterion.
#' @export
#'
#' @examples
#' data("frailty", package = "rsimsum")
#' frailty2 <- subset(frailty, par == "fv")
#'
#' # Using low values of max, semax for illustration purposes:
#' dropbig(
#'   data = frailty2, estvarname = "b", se = "se",
#'   methodvar = "model", by = "fv_dist", max = 2, semax = 2
#' )
#'
#' # Using regular standardisation:
#' dropbig(
#'   data = frailty2, estvarname = "b", se = "se",
#'   methodvar = "model", by = "fv_dist", max = 2, semax = 2, robust = FALSE
#' )
dropbig <- function(
  data,
  estvarname,
  se = NULL,
  methodvar = NULL,
  by = NULL,
  max = 10,
  semax = 100,
  robust = TRUE
) {
  ### Check arguments
  arg_checks <- checkmate::makeAssertCollection()
  # 'data' must be a data.frame
  checkmate::assert_data_frame(x = data, add = arg_checks)
  # 'estvarname', 'se', 'methodvar' must be a single string value
  checkmate::assert_string(x = estvarname, add = arg_checks)
  checkmate::assert_string(x = se, null.ok = TRUE, add = arg_checks)
  checkmate::assert_string(x = methodvar, null.ok = TRUE, add = arg_checks)
  # 'robust' must be single logical value
  checkmate::assert_logical(x = robust, len = 1, add = arg_checks)
  # 'by' must be a vector of strings; can be NULL
  checkmate::assert_character(x = by, null.ok = TRUE, add = arg_checks)
  # 'estvarname', 'se' must be in 'data'; all elements of 'by' must be in 'data'; 'methodvar' must be in 'data'
  checkmate::assert_subset(
    x = estvarname,
    choices = names(data),
    add = arg_checks
  )
  checkmate::assert_subset(x = se, choices = names(data), add = arg_checks)
  checkmate::assert_subset(x = by, choices = names(data), add = arg_checks)
  checkmate::assert_subset(
    x = methodvar,
    choices = names(data),
    add = arg_checks
  )
  # 'max', 'semax' must be a single number
  checkmate::assert_number(x = max, add = arg_checks)
  checkmate::assert_number(x = semax, add = arg_checks)
  # Report
  if (!arg_checks$isEmpty()) {
    checkmate::reportAssertions(arg_checks)
  }

  ### Uses the internal function .dropbig
  return(.dropbig(
    data = data,
    estvarname = estvarname,
    se = se,
    methodvar = methodvar,
    by = by,
    max = max,
    semax = semax,
    robust = robust,
    internal = FALSE
  ))
}

testthat::context("dropbig")

testthat::test_that("dropbig detects values above 'max'", {
  set.seed(238746)
  n <- 1000
  df <- data.frame(
    theta = rnorm(n),
    se = exp(rnorm(n, sd = 0.1))
  )
  df$theta[1] <- rnorm(1, mean = 1000)
  out <- dropbig(
    data = df,
    estvarname = "theta",
    se = "se",
    methodvar = NULL,
    by = NULL,
    robust = FALSE
  )
  testthat::expect_equal(object = out$.dropbig[1], expected = TRUE)
  out <- dropbig(
    data = df,
    estvarname = "theta",
    se = "se",
    methodvar = NULL,
    by = NULL,
    robust = TRUE
  )
  testthat::expect_equal(object = out$.dropbig[1], expected = TRUE)
  # Using robust standardisation is better at detecting outliers:
  df$theta[1] <- rnorm(1, mean = 500)
  out <- dropbig(
    data = df,
    estvarname = "theta",
    se = "se",
    methodvar = NULL,
    by = NULL,
    robust = TRUE
  )
  testthat::expect_equal(object = out$.dropbig[1], expected = TRUE)
  df$theta[1] <- rnorm(1, mean = 100)
  out <- dropbig(
    data = df,
    estvarname = "theta",
    se = "se",
    methodvar = NULL,
    by = NULL,
    robust = TRUE
  )
  testthat::expect_equal(object = out$.dropbig[1], expected = TRUE)
  df$theta[1] <- rnorm(1, mean = 50)
  out <- dropbig(
    data = df,
    estvarname = "theta",
    se = "se",
    methodvar = NULL,
    by = NULL,
    robust = TRUE
  )
  testthat::expect_equal(object = out$.dropbig[1], expected = TRUE)
  df$theta[1] <- rnorm(1, mean = 25)
  out <- dropbig(
    data = df,
    estvarname = "theta",
    se = "se",
    methodvar = NULL,
    by = NULL,
    robust = TRUE
  )
  testthat::expect_equal(object = out$.dropbig[1], expected = TRUE)
  # Without passing 'se':
  df$theta[1] <- rnorm(1, mean = 500)
  out <- dropbig(
    data = df,
    estvarname = "theta",
    methodvar = NULL,
    by = NULL,
    robust = TRUE
  )
  testthat::expect_equal(object = out$.dropbig[1], expected = TRUE)
  df$theta[1] <- rnorm(1, mean = 100)
  out <- dropbig(
    data = df,
    estvarname = "theta",
    methodvar = NULL,
    by = NULL,
    robust = TRUE
  )
  testthat::expect_equal(object = out$.dropbig[1], expected = TRUE)
  df$theta[1] <- rnorm(1, mean = 50)
  out <- dropbig(
    data = df,
    estvarname = "theta",
    methodvar = NULL,
    by = NULL,
    robust = TRUE
  )
  testthat::expect_equal(object = out$.dropbig[1], expected = TRUE)
  df$theta[1] <- rnorm(1, mean = 25)
  out <- dropbig(
    data = df,
    estvarname = "theta",
    methodvar = NULL,
    by = NULL,
    robust = TRUE
  )
  testthat::expect_equal(object = out$.dropbig[1], expected = TRUE)
})

testthat::test_that("dropbig throws errors when appropriate", {
  set.seed(238746)
  n <- 1000
  df <- data.frame(
    theta = rnorm(n),
    se = exp(rnorm(n, sd = 0.1))
  )
  df$theta[1] <- rnorm(1, mean = 1000)
  testthat::expect_error(rsimsum::dropbig(data = df))
  testthat::expect_error(
    object = rsimsum::dropbig(data = df, estvarname = 1, se = "se")
  )
  testthat::expect_error(
    object = rsimsum::dropbig(data = df, estvarname = "theta", se = 1)
  )
  testthat::expect_error(
    object = rsimsum::dropbig(data = df, estvarname = TRUE, se = "se")
  )
  testthat::expect_error(
    object = rsimsum::dropbig(data = df, estvarname = "theta", se = TRUE)
  )
  testthat::expect_error(
    object = rsimsum::dropbig(
      data = df,
      estvarname = "theta",
      se = "se",
      methodvar = 1
    )
  )
  testthat::expect_error(
    object = rsimsum::dropbig(
      data = df,
      estvarname = "theta",
      se = "se",
      methodvar = TRUE
    )
  )
  testthat::expect_error(
    object = rsimsum::dropbig(
      data = df,
      estvarname = "theta",
      se = "se",
      by = 1
    )
  )
  testthat::expect_error(
    object = rsimsum::dropbig(
      data = df,
      estvarname = "theta",
      se = "se",
      by = TRUE
    )
  )
  testthat::expect_error(
    object = rsimsum::dropbig(
      data = df,
      estvarname = "theta",
      se = "se",
      robust = "yeah!"
    )
  )
})

#' @title Analyses of simulation studies including Monte Carlo error
#' @description `simsum()` computes performance measures for simulation studies in which each simulated data set yields point estimates by one or more analysis methods.
#' Bias, relative bias, empirical standard error and precision relative to a reference method can be computed for each method.
#' If, in addition, model-based standard errors are available then `simsum()` can compute the average model-based standard error, the relative error in the model-based standard error, the coverage of nominal confidence intervals, the coverage under the assumption that there is no bias (bias-eliminated coverage), and the power to reject a null hypothesis.
#' Monte Carlo errors are available for all estimated quantities.
#' @param data A `data.frame` in which variable names are interpreted.
#' It has to be in tidy format, e.g. each variable forms a column and each observation forms a row.
#' @param estvarname The name of the variable containing the point estimates.
#' Note that some column names are forbidden: these are listed below in the _Details_ section.
#' @param se The name of the variable containing the standard errors of the point estimates.
#' Note that some column names are forbidden: these are listed below in the _Details_ section.
#' @param true The true value of the parameter; this is used in calculations of bias, relative bias, coverage, and mean squared error and is required whenever these performance measures are requested.
#' `true` can be a numeric value or a string that identifies a column in `data`.
#' In the former setting, `simsum` will assume the same value for all replications; conversely, each replication will use a distinct value for `true` as identified by each row of `data`.
#' See `vignette("E-custom-inputs", package = "rsimsum")` for more details.
#' Note that some column names are forbidden: these are listed below in the _Details_ section.
#' @param methodvar The name of the variable containing the methods to compare.
#' For instance, methods could be the models compared within a simulation study.
#' Can be `NULL`.
#' If a vector of column names is passed to `simsum()`, those columns will be combined into a single column named `:methodvar` using the [base::interaction()] function before computing all performance measures.
#' Note that some column names are forbidden: these are listed below in the _Details_ section.
#' @param ref Specifies the reference method against which relative precision will be calculated.
#' Only useful if `methodvar` is specified.
#' @param by A vector of variable names to compute performance measures by a list of factors. Factors listed here are the (potentially several) data-generating mechanisms used to simulate data under different scenarios (e.g. sample size, true distribution of a variable, etc.).
#' Can be `NULL`.
#' Note that some column names are forbidden: these are listed below in the _Details_ section.
#' @param ci.limits Can be used to specify the limits (lower and upper) of confidence intervals used to calculate coverage and bias-eliminated coverage.
#' Useful for non-Wald type estimators (e.g. bootstrap).
#' Defaults to `NULL`, where Wald-type confidence intervals based on the provided SEs are calculated for coverage; otherwise, it can be a numeric vector (for fixed confidence intervals) or a vector of strings that identify columns in `data` with replication-specific lower and upper limits.
#' See `vignette("E-custom-inputs", package = "rsimsum")` for more details.
#' Note that some column names are forbidden: these are listed below in the _Details_ section.
#' @param df Can be used to specify that a column containing the replication-specific number of degrees of freedom that will be used to calculate confidence intervals for coverage (and bias-eliminated coverage) assuming t-distributed critical values (rather than normal theory intervals).
#' See `vignette("E-custom-inputs", package = "rsimsum")` for more details.
#' Note that some column names are forbidden: these are listed below in the _Details_ section.
#' @param dropbig Specifies that point estimates or standard errors beyond the maximum acceptable values should be dropped. Defaults to `FALSE`.
#' @param x Set to `TRUE` to include the `data` argument used to calculate summary statistics (i.e. after pre-processing the input dataset e.g. removing values deemed too large via the `dropbig` argument) as a slot. Calling `simsum` with `x = TRUE` is required to produce zipper plots. The downside is that the size of the returned object increases considerably, therefore it is set to `FALSE` by default.
#' @param control A list of parameters that control the behaviour of `simsum`.
#' Possible values are:
#' * `mcse`, whether to calculate Monte Carlo standard errors. Defaults to `TRUE`;
#' * `level`, the significance level used for coverage, bias-eliminated coverage, and power. Defaults to 0.95;
#' * `power_df`, whether to use robust critical values from a t distribution with `power_df` degrees of freedom when calculating power. Defaults to `NULL`, in which case a Gaussian distribution is used;
#' * `na.rm`, whether to remove point estimates or standard errors where either (or both) is missing. Defaults to `TRUE`;
#' * `char.sep`, a character utilised when splitting the input dataset `data`. Generally, this should not be changed;
#' * `dropbig.max`, specifies the maximum acceptable absolute value of the point estimates, after standardisation. Defaults to 10;
#' * `dropbig.semax`, specifies the maximum acceptable absolute value of the standard error, after standardisation. Defaults to 100
#' * `dropbig.robust`, specifies whether to use robust standardisation (using median and inter-quartile range) rather than normal standardisation (using mean and standard deviation). Defaults to `TRUE`, in which case robust standardisation will be used for `dropbig`.
#' @return An object of class `simsum`.
#' @references White, I.R. 2010. simsum: Analyses of simulation studies including Monte Carlo error. The Stata Journal 10(3): 369-385. \url{https://www.stata-journal.com/article.html?article=st0200}
#' @references Morris, T.P., White, I.R. and Crowther, M.J. 2019. _Using simulation studies to evaluate statistical methods_. Statistics in Medicine, \doi{10.1002/sim.8086}
#' @references Gasparini, A. 2018. rsimsum: Summarise results from Monte Carlo simulation studies. Journal of Open Source Software 3(26):739, \doi{10.21105/joss.00739}
#' @export
#' @details
#' The following names are not allowed for any column in `data` that is passed to [simsum()]: `stat`, `est`, `mcse`, `lower`, `upper`, `:methodvar`, `:true`.
#'
#' @examples
#' data("MIsim", package = "rsimsum")
#' s <- simsum(data = MIsim, estvarname = "b", true = 0.5, se = "se", methodvar = "method", ref = "CC")
#' # If 'ref' is not specified, the reference method is inferred
#' s <- simsum(data = MIsim, estvarname = "b", true = 0.5, se = "se", methodvar = "method")
simsum <- function(
  data,
  estvarname,
  se = NULL,
  true = NULL,
  methodvar = NULL,
  ref = NULL,
  by = NULL,
  ci.limits = NULL,
  df = NULL,
  dropbig = FALSE,
  x = FALSE,
  control = list()
) {
  # set.seed(135)
  # data = data.frame(
  #   dataset_id = 1:5,
  #   method = 1,
  #   estimate = rnorm(5, mean = 1),
  #   se = rnorm(5, 1 / sqrt(5), 0.05),
  #   true = 1
  # )
  # data[1, "se"] = NA
  # estvarname = "estimate"
  # se = "se"
  # true = "true"
  # methodvar = "method"
  # x = TRUE
  # ref = "1"
  # by = NULL
  # ci.limits = NULL
  # df = NULL
  # dropbig = FALSE
  # x = FALSE
  # control = list()

  ### Check arguments
  arg_checks <- checkmate::makeAssertCollection()
  # 'data' must be a data.frame
  checkmate::assert_data_frame(x = data, add = arg_checks)
  # 'estvarname', 'se', 'methodvar', 'ref' must be a single string value
  checkmate::assert_string(x = estvarname, add = arg_checks)
  checkmate::assert_string(x = se, null.ok = TRUE, add = arg_checks)
  checkmate::assert_string(x = ref, null.ok = TRUE, add = arg_checks)
  checkmate::assert_string(x = df, null.ok = TRUE, add = arg_checks)
  # 'true' must be a single numeric value, or a string that identifies a column in 'data'
  if (!is.null(true)) {
    checkmate::assert_true(
      x = inherits(x = true, what = c("character", "numeric", "integer")),
      add = arg_checks
    )
    if (is.character(true)) {
      checkmate::assert_string(x = true, add = arg_checks)
      checkmate::assert_true(x = all(true %in% names(data)), add = arg_checks)
    } else {
      checkmate::assert_number(x = true, null.ok = TRUE, add = arg_checks)
    }
  }
  # 'dropbig', 'mcse', 'x' must be single logical value
  checkmate::assert_logical(x = dropbig, len = 1, add = arg_checks)
  checkmate::assert_logical(x = x, len = 1, add = arg_checks)
  # 'by' and 'methodvar' must be a vector of strings; can be NULL
  checkmate::assert_character(x = methodvar, null.ok = TRUE, add = arg_checks)
  checkmate::assert_character(x = by, null.ok = TRUE, add = arg_checks)
  # 'estvarname', 'se' must be in 'data'; all elements of 'by' must be in 'data'; 'methodvar' must be in 'data'; 'df' must be in 'data'
  checkmate::assert_subset(
    x = estvarname,
    choices = names(data),
    add = arg_checks
  )
  checkmate::assert_subset(x = se, choices = names(data), add = arg_checks)
  checkmate::assert_subset(x = by, choices = names(data), add = arg_checks)
  checkmate::assert_subset(
    x = methodvar,
    choices = names(data),
    add = arg_checks
  )
  checkmate::assert_subset(x = df, choices = names(data), add = arg_checks)
  # 'estvarname', 'se', 'methodvar', 'by' , 'df' must not be any in ('stat', 'est', 'mcse', 'lower', 'upper', ':methodvar', ':true')
  .private_names <- c(
    "stat",
    "est",
    "mcse",
    "lower",
    "upper",
    ":methodvar",
    ":true"
  )
  .check_private(
    var = estvarname,
    label = "estvarname",
    private_names = .private_names
  )
  .check_private(var = se, label = "se", private_names = .private_names)
  .check_private(
    var = methodvar,
    label = "methodvar",
    private_names = .private_names
  )
  .check_private(var = by, label = "by", private_names = .private_names)
  .check_private(var = df, label = "df", private_names = .private_names)
  # Process vector of 'methodvar' if a vector
  user_methodvar <- NULL
  if (length(methodvar) > 1) {
    reftable <- .compact_method_columns(
      data = data,
      methodvar = methodvar
    )$reftable
    data <- .compact_method_columns(data = data, methodvar = methodvar)$data
    user_methodvar <- methodvar
    methodvar <- ":methodvar"
  }
  # 'ref' must be one of the options in 'methodvar'
  if (!is.null(methodvar)) {
    checkmate::assert_subset(
      x = ref,
      choices = as.character(unique(data[[methodvar]])),
      add = arg_checks
    )
  }
  # 'ci.limits' must be either a numeric vector of length 2 or a string vector with column names in 'data'
  if (!is.null(ci.limits)) {
    checkmate::assert_true(
      x = inherits(x = ci.limits, what = c("character", "numeric", "integer")),
      add = arg_checks
    )
    if (is.character(ci.limits)) {
      checkmate::assert_character(x = ci.limits, len = 2, add = arg_checks)
      checkmate::assert_true(
        x = all(ci.limits %in% names(data)),
        add = arg_checks
      )
      lapply(X = ci.limits, FUN = function(x) {
        .check_private(
          var = x,
          label = "ci.limits",
          private_names = .private_names
        )
      })
    }
    if (is.numeric(ci.limits)) {
      checkmate::assert_numeric(x = ci.limits, len = 2, add = arg_checks)
    }
  }
  # 'control' must be a list, with well defined components
  checkmate::assert_list(x = control, add = arg_checks)
  checkmate::assert_subset(
    x = names(control),
    choices = c(
      "mcse",
      "level",
      "power_df",
      "na.rm",
      "char.sep",
      "dropbig.max",
      "dropbig.semax",
      "dropbig.robust"
    ),
    empty.ok = TRUE,
    add = arg_checks
  )
  checkmate::assert_logical(
    x = control$mcse,
    len = 1,
    null.ok = TRUE,
    add = arg_checks
  )
  checkmate::assert_number(
    x = control$level,
    lower = 0,
    upper = 1,
    null.ok = TRUE,
    add = arg_checks
  )
  checkmate::assert_number(
    x = control$power_df,
    null.ok = TRUE,
    add = arg_checks
  )
  checkmate::assert_logical(
    x = control$na.rm,
    len = 1,
    null.ok = TRUE,
    add = arg_checks
  )
  checkmate::assert_string(
    x = control$char.sep,
    null.ok = TRUE,
    add = arg_checks
  )
  checkmate::assert_number(
    x = control$dropbig.max,
    null.ok = TRUE,
    add = arg_checks
  )
  checkmate::assert_number(
    x = control$dropbig.semax,
    null.ok = TRUE,
    add = arg_checks
  )
  checkmate::assert_logical(
    x = control$dropbig.robust,
    len = 1,
    null.ok = TRUE,
    add = arg_checks
  )
  # Report
  if (!arg_checks$isEmpty()) {
    checkmate::reportAssertions(arg_checks)
  }

  ### Only one of 'ci.limits' and 'df' can be specified
  if (!is.null(ci.limits) & !is.null(df)) {
    stop("Only one of 'ci.limits' and 'df' can be specified.", call. = FALSE)
  }

  ### Set control parameters
  control.default <- list(
    mcse = TRUE,
    level = 0.95,
    power_df = NULL,
    na.rm = TRUE,
    char.sep = "~",
    dropbig.max = 10,
    dropbig.semax = 100,
    dropbig.robust = TRUE
  )
  control.tmp <- unlist(
    list(
      control[names(control) %in% names(control.default)],
      control.default[!(names(control.default) %in% names(control))]
    ),
    recursive = FALSE
  )
  control <- control.tmp

  ### Add hidden column with true values
  if (!is.null(true)) {
    if (is.character(true)) {
      data[[":true"]] <- data[[true]]
    } else {
      data[[":true"]] <- true
    }
  }

  ### Factorise 'methodvar', 'by'
  data <- .factorise(data = data, cols = c(methodvar, by))

  ### Check that levels of factors are ok
  .validate_levels(
    data = data,
    cols = c(methodvar, by),
    char = control$char.sep
  )

  ### Set reference method if `ref` is not specified
  if (!is.null(methodvar)) {
    methods <- levels(data[[methodvar]])
    if (is.null(ref)) {
      ref <- methods[1]
      message(paste(
        "'ref' method was not specified,",
        ref,
        "set as the reference"
      ))
    }
    data[[methodvar]] <- relevel(data[[methodvar]], ref = ref)
  }

  ### Throw a warning if `ref` is specified and `methodvar` is not
  if (is.null(methodvar) & !is.null(ref)) {
    warning(
      "'ref' method is specified while 'methodvar' is not: 'ref' will be ignored"
    )
    ref <- NULL
  }

  ### Identify and drop (if required) point estimates and standard errors that are too big
  if (dropbig) {
    data <- .dropbig(
      data = data,
      estvarname = estvarname,
      se = se,
      methodvar = methodvar,
      by = by,
      max = control$dropbig.max,
      semax = control$dropbig.semax,
      robust = control$dropbig.robust
    )
  }

  ### Drop estimates if SE is missing, and vice versa
  data <- .na_pair(data = data, estvarname = estvarname, se = se)

  ### Compute summary statistics
  # Split by first
  data <- .split_by(data = data, by = by)

  # Then, split methodvar
  data <- lapply(X = seq_along(data), FUN = function(i) {
    .split_by(data = data[[i]], by = methodvar)
  })

  # Remove elements in 'data' where we get empty datasets (#47)
  data <- .drop_empty_splits(data)

  # Then call .performance to compute all performance measures
  summ <- lapply(X = seq_along(data), FUN = function(i) {
    if (!is.null(methodvar)) {
      rho <- vapply(
        X = methods,
        FUN = function(x) {
          stats::cor(
            data[[i]][[ref]][[estvarname]],
            data[[i]][[x]][[estvarname]],
            use = ifelse(control$na.rm, "na.or.complete", "everything")
          )
        },
        FUN.VALUE = numeric(1)
      )
    } else {
      rho <- NULL
    }
    out.out <- lapply(X = seq_along(data[[i]]), FUN = function(j) {
      if (!is.null(methodvar)) {
        empse_ref <- sqrt(stats::var(
          data[[i]][[ref]][[estvarname]],
          na.rm = control$na.rm
        ))
      } else {
        empse_ref <- NULL
      }
      out.in <- .performance(
        data = data[[i]][[j]],
        estvarname = estvarname,
        se = se,
        true = true,
        rho = rho[names(data[[i]][j])],
        empse_ref = empse_ref,
        ci.limits = ci.limits,
        df = df,
        control = control
      )
      if (!is.null(methodvar)) {
        out.in[[methodvar]] <- unique(data[[i]][[j]][[methodvar]])
      }
      for (byval in by) {
        out.in[[byval]] <- unique(data[[i]][[j]][[byval]])
      }
      out.in
    })
    out.out <- .br(out.out)
    out.out
  })
  summ <- .br(summ)

  # If vector of 'methodvar', restore input data
  if (!is.null(user_methodvar)) {
    summ[[".nr"]] <- seq_len(nrow(summ))
    summ <- merge(summ, reftable, by = methodvar)
    summ <- summ[order(summ[[".nr"]]), ]
    summ[, methodvar] <- NULL
    summ[, ".nr"] <- NULL
    methodvar <- user_methodvar
  }

  ### Include stuff into object to return
  obj <- list()
  obj$summ <- summ
  obj$estvarname <- estvarname
  obj$true <- true
  obj$se <- se
  obj$methodvar <- methodvar
  obj$ref <- ref
  obj$dropbig <- dropbig
  obj$ci.limits <- ci.limits
  obj$df <- df
  obj$by <- by
  obj$control <- control
  if (x) {
    obj$x <- .br(lapply(data, .br))
    if (!is.null(true)) {
      obj$x[[":true"]] <- NULL
    }
    rownames(obj$x) <- NULL
  }

  ### Return object of class simsum
  class(obj) <- c("simsum", "list")
  return(obj)
}

test_that("MCSE of relative error is the same as in Stata's simsum", {
  data("MIsim", package = "rsimsum")
  s <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    ref = "CC"
  )
  s <- summary(s)
  expect_equal(
    object = s$summ[s$summ$stat == "relerror", ]$mcse,
    expected = c(2.205481529236, 2.332338094711, 2.269521713257),
    tolerance = 1e-6
  )
})

context("Testing relative bias")

test_that("Relative bias calculations are okay, if unbiased", {
  sims <- vapply(
    X = seq(500),
    FUN = function(i) {
      N <- 3e3
      x <- rnorm(n = N)
      e <- rnorm(n = N)
      y <- 2 * x + e
      fit <- lm(y ~ x)
      coef(fit)["x"]
    },
    FUN.VALUE = numeric(1)
  )
  sims <- data.frame(b = sims)
  s <- simsum(data = sims, estvarname = "b", true = 2)
  expect_equal(
    object = s$summ$est[s$summ$stat == "rbias"],
    expected = 0,
    tolerance = 1e-2
  )
})

test_that("Relative bias calculations are okay, if biased", {
  sims <- vapply(
    X = seq(500),
    FUN = function(i) {
      N <- 3e3
      x <- rnorm(n = N)
      e <- rnorm(n = N)
      y <- 2 * x + e
      fit <- lm(y ~ x)
      coef(fit)["x"]
    },
    FUN.VALUE = numeric(1)
  )
  sims <- data.frame(b = sims)
  s <- simsum(data = sims, estvarname = "b", true = 1)
  expect_equal(
    object = s$summ$est[s$summ$stat == "rbias"],
    expected = 1,
    tolerance = 1e-2
  )
})

testthat::context("summary.simsum")

testthat::test_that("summarising a simsum object works fine and prints ok", {
  data("MIsim")
  x <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  testthat::expect_output(print(summary(x)))
  testthat::expect_output(print(summary(x, ci_level = 0.99)))
  x <- simsum(data = MIsim, estvarname = "b", true = 0.5, se = "se")
  testthat::expect_output(print(summary(x)))
  data("relhaz")
  x <- simsum(
    data = relhaz,
    estvarname = "theta",
    true = -0.5,
    se = "se",
    methodvar = "model",
    by = c("n", "baseline")
  )
  testthat::expect_output(print(summary(x)))
  x <- simsum(
    data = relhaz,
    estvarname = "theta",
    true = -0.5,
    se = "se",
    by = c("n", "baseline")
  )
  testthat::expect_output(print(summary(x)))
  testthat::expect_output(print(summary(x), mcse = NULL))
  testthat::expect_output(print(summary(x), mcse = TRUE))
  testthat::expect_output(print(summary(x), mcse = FALSE))
  testthat::expect_error(print(summary(x), digits = -1))
})

testthat::test_that("summary.simsum returns an object of class summary.simsum", {
  data("MIsim")
  x <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  s <- summary(x)
  testthat::expect_s3_class(object = s, class = "summary.simsum")
})

testthat::test_that("summary.simsum returns confidence intervals when mcse = TRUE", {
  data("MIsim")
  x <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  s <- summary(x)
  testthat::expect_true(object = all(c("lower", "upper") %in% names(s$summ)))
})

testthat::test_that("summary.simsum does not return confidence intervals when mcse = FALSE", {
  data("MIsim")
  x <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    control = list(mcse = FALSE)
  )
  s <- summary(x)
  testthat::expect_true(object = all(!(c("lower", "upper") %in% names(s$summ))))
})

testthat::test_that("summary.simsum with wrong arguments throws an error", {
  data("MIsim")
  x <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    control = list(mcse = FALSE)
  )
  testthat::expect_error(object = summary(x, ci_level = -1))
  testthat::expect_error(object = summary(x, ci_level = 2))
  testthat::expect_error(object = summary(x, ci_level = 1.50))
  testthat::expect_error(object = summary(x, stats = "42"))
  testthat::expect_error(object = summary(x, ci_level = "0.05"))
})

testthat::test_that("summary.simsum with t distribution, results differ with default settings", {
  data("MIsim")
  x <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  testthat::expect_false(
    object = all(
      tidy(summary(x))[["lower"]] == tidy(summary(x, df = 3))[["lower"]]
    )
  )
  testthat::expect_false(
    object = all(
      tidy(summary(x, df = 3))[["lower"]] ==
        tidy(summary(x, df = 10))[["lower"]]
    )
  )
  testthat::expect_false(
    object = all(
      tidy(summary(x))[["upper"]] == tidy(summary(x, df = 3))[["upper"]]
    )
  )
  testthat::expect_false(
    object = all(
      tidy(summary(x, df = 3))[["upper"]] ==
        tidy(summary(x, df = 10))[["upper"]]
    )
  )
})

testthat::test_that("summary.simsum returns selected stats only", {
  data("MIsim")
  x <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  testthat::expect_true(
    object = all(tidy(summary(x, stats = "bias"))[["stat"]] == "bias")
  )
  testthat::expect_true(
    object = all(tidy(summary(x, stats = "rbias"))[["stat"]] == "rbias")
  )
  testthat::expect_true(
    object = all(
      tidy(summary(x, stats = c("bias", "cover")))[["stat"]] %in%
        c("bias", "cover")
    )
  )
})

testthat::test_that("print.summary.simsum complains if users ask for the moon", {
  data("MIsim")
  x <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    control = list(mcse = FALSE)
  )
  testthat::expect_message(object = print(summary(x, mcse = TRUE)))
  testthat::expect_message(object = print(summary(x, mcse = FALSE)))
  testthat::expect_message(object = print(summary(x, mcse = NULL)))
})

### New tests for PR #33
testthat::context("#33")

testthat::test_that("Test that power_df is actually used", {
  data(MIsim, package = "rsimsum")
  s1 <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    control = list(power_df = NULL)
  )
  s2 <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    control = list(power_df = 3)
  )
  testthat::expect_false(
    object = identical(
      x = tidy(s1)$est[tidy(s1)$stat == "power"],
      y = tidy(s2)$est[tidy(s2)$stat == "power"]
    )
  )

  s3 <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method"
  )
  testthat::expect_false(
    object = identical(
      x = tidy(s3)$est[tidy(s3)$stat == "power"],
      y = tidy(s2)$est[tidy(s2)$stat == "power"]
    )
  )
  testthat::expect_identical(
    object = tidy(s1)$est[tidy(s1)$stat == "power"],
    expected = tidy(s3)$est[tidy(s3)$stat == "power"]
  )

  s4 <- simsum(
    data = MIsim,
    estvarname = "b",
    true = 0.5,
    se = "se",
    methodvar = "method",
    control = list(power_df = 1000)
  )
  testthat::expect_identical(
    object = tidy(s1)$est[tidy(s1)$stat == "power"],
    expected = tidy(s4)$est[tidy(s4)$stat == "power"]
  )
})

testthat::context("performance-math")

testthat::test_that("bias", {
  set.seed(20190107)
  n <- 1000

  # No bias
  df <- data.frame(
    i = seq(n),
    b = rnorm(n = n, mean = 0.50, sd = 0.001),
    se = exp(rnorm(n = n, mean = 0.1, sd = 0.1))
  )
  s <- rsimsum::simsum(data = df, estvarname = "b", se = "se", true = 0.50)
  testthat::expect_equal(
    object = s$summ$est[s$summ$stat == "bias"],
    expected = 0,
    tolerance = 1e-4
  )

  # Positive bias
  df <- data.frame(
    i = seq(n),
    b = rnorm(n = n, mean = 0.5001, sd = 0.001),
    se = exp(rnorm(n = n, mean = 0.1, sd = 0.1))
  )
  s <- rsimsum::simsum(data = df, estvarname = "b", se = "se", true = 0.50)
  testthat::expect_true(object = (s$summ$est[s$summ$stat == "bias"] > 0))

  # Negative bias
  df <- data.frame(
    i = seq(n),
    b = rnorm(n = n, mean = 0.4999, sd = 0.001),
    se = exp(rnorm(n = n, mean = 0.1, sd = 0.1))
  )
  s <- rsimsum::simsum(data = df, estvarname = "b", se = "se", true = 0.50)
  testthat::expect_true(object = (s$summ$est[s$summ$stat == "bias"] < 0))
})

### Compare MIsim results with Stata's
testthat::test_that("MIsim, vs Stata", {
  data("MIsim", package = "rsimsum")
  r <- rsimsum::simsum(
    data = MIsim,
    estvarname = "b",
    se = "se",
    true = 0.5,
    methodvar = "method"
  )$summ
  r <- r[
    !(r$stat %in%
      c(
        "thetamean",
        "thetamedian",
        "se2mean",
        "se2median",
        "nsim",
        "rbias",
        "becover"
      )),
  ]
  r$method <- as.character(r$method)
  r <- r[order(r$stat, r$method), ]
  s <- MIsim_res_stata
  s <- s[order(s$stat, s$method), ]
  # Test equivalence of estimates
  testthat::expect_equivalent(
    object = r$est,
    expected = s$est,
    tolerance = 1e-6
  )
  # Test equivalence of MCSEs
  testthat::expect_equivalent(
    object = r$mcse,
    expected = s$mcse,
    tolerance = 1e-3
  )
})

### Compare relhaz results with Stata's
testthat::test_that("relhaz, vs Stata", {
  data("relhaz", package = "rsimsum")
  relhaz$model <- as.numeric(factor(relhaz$model))
  r <- rsimsum::simsum(
    data = relhaz,
    estvarname = "theta",
    se = "se",
    true = -0.5,
    methodvar = "model",
    by = c("n", "baseline")
  )$summ
  r <- r[
    !(r$stat %in%
      c(
        "thetamean",
        "thetamedian",
        "se2mean",
        "se2median",
        "nsim",
        "rbias",
        "becover"
      )),
  ]
  r$model <- as.character(r$model)
  r$n <- as.character(r$n)
  r$baseline <- as.character(r$baseline)
  r <- r[order(r$stat, r$model, r$n, r$baseline), ]
  s <- relhaz_res_stata
  s <- s[order(s$stat, s$model, s$n, s$baseline), ]
  # Test equivalence of estimates
  testthat::expect_equivalent(
    object = r$est,
    expected = s$est,
    tolerance = 1e-6
  )
  # Test equivalence of MCSEs
  testthat::expect_equivalent(
    object = r$mcse,
    expected = s$mcse,
    tolerance = 0.005
  )
})

testthat::context("ci.limits")
data(tt, package = "rsimsum")

testthat::test_that("ci.limits breaks if columns not in data", {
  testthat::expect_error(rsimsum::simsum(
    data = tt,
    estvarname = "diff",
    true = -1,
    se = "se",
    ci.limits = c("lower.missing", "upper.missing"),
    methodvar = "method",
    by = "dgm"
  ))
})

testthat::test_that("ci.limits breaks if logic vector", {
  testthat::expect_error(rsimsum::simsum(
    data = tt,
    estvarname = "diff",
    true = -1,
    se = "se",
    ci.limits = c(TRUE, FALSE),
    methodvar = "method",
    by = "dgm"
  ))
})

testthat::test_that("ci.limits works ok with string or numeric vectors", {
  testthat::expect_s3_class(
    object = rsimsum::simsum(
      data = tt,
      estvarname = "diff",
      true = -1,
      se = "se",
      ci.limits = c("conf.low", "conf.high"),
      methodvar = "method",
      by = "dgm"
    ),
    class = "simsum"
  )
  testthat::expect_s3_class(
    object = rsimsum::simsum(
      data = tt,
      estvarname = "diff",
      true = -1,
      se = "se",
      ci.limits = c(-2, 0),
      methodvar = "method",
      by = "dgm"
    ),
    class = "simsum"
  )
})

testthat::test_that("ci.limits with string vector yields different values of coverage than the default", {
  s <- rsimsum::simsum(
    data = tt,
    estvarname = "diff",
    true = -1,
    se = "se",
    ci.limits = c("conf.low", "conf.high"),
    methodvar = "method",
    by = "dgm"
  )
  s <- tidy(s, stats = "cover")
  sdef <- rsimsum::simsum(
    data = tt,
    estvarname = "diff",
    true = -1,
    se = "se",
    methodvar = "method",
    by = "dgm"
  )
  sdef <- tidy(sdef, stats = "cover")

  testthat::expect_false(object = all(s == sdef))
})

test_that("NLP with non-fully-factorial design", {
  data("nlp", package = "rsimsum")
  nlp.subset <- nlp %>%
    dplyr::filter(!(ss == 100 & esigma == 2))
  #
  s.nlp <- rsimsum::simsum(
    data = nlp,
    estvarname = "b",
    true = 0,
    se = "se",
    methodvar = "model",
    by = c("baseline", "ss", "esigma")
  )
  td.nlp <- tidy(summary(s.nlp))
  #
  s.nlp.subset <- rsimsum::simsum(
    data = nlp.subset,
    estvarname = "b",
    true = 0,
    se = "se",
    methodvar = "model",
    by = c("baseline", "ss", "esigma")
  )
  td.nlp.subset <- tidy(summary(s.nlp.subset))
  #
  expect_true(object = (nrow(td.nlp) > nrow(td.nlp.subset)))
  #
  expect_s3_class(
    object = autoplot(s.nlp, type = "nlp"),
    class = c("gg", "ggplot")
  )
  expect_s3_class(
    object = autoplot(s.nlp.subset, type = "nlp"),
    class = c("gg", "ggplot")
  )
})

testthat::test_that("#60", {
  # See https://github.com/ellessenne/rsimsum/issues/60
  set.seed(135)
  test <- data.frame(
    dataset_id = 1:5,
    method = 1,
    estimate = rnorm(5, mean = 1),
    se = rnorm(5, 1 / sqrt(5), 0.05),
    true = 1
  )
  ss <- simsum(
    data = test,
    estvarname = "estimate",
    se = "se",
    true = "true",
    methodvar = "method",
    x = TRUE,
    ref = "1"
  )
  a <- summary(ss, stat = "rbias") |>
    tidy()
  testthat::expect_equal(
    object = a$est,
    expected = -0.1846501,
    tolerance = 1e-6
  )
  testthat::expect_equal(
    object = a$mcse,
    expected = 0.4455973,
    tolerance = 1e-6
  )
  #
  test2 <- test
  test2[1, "se"] <- NA
  ss2 <- simsum(
    data = test2,
    estvarname = "estimate",
    se = "se",
    true = "true",
    methodvar = "method",
    x = TRUE,
    ref = "1"
  )
  b <- summary(ss2, stat = "rbias") |>
    tidy()
  testthat::expect_equal(
    object = b$est,
    expected = -0.1195449,
    tolerance = 1e-6
  )
  testthat::expect_equal(
    object = b$mcse,
    expected = 0.5690903,
    tolerance = 1e-6
  )
})

### New tests for issue #23
testthat::context("#26")

testthat::test_that("argument checks for new option", {
  data("tt", package = "rsimsum")

  testthat::expect_s3_class(
    object = simsum(
      data = tt,
      estvarname = "diff",
      se = "se",
      df = "df",
      true = -1
    ),
    class = "simsum"
  )
  testthat::expect_error(
    object = simsum(
      data = tt,
      estvarname = "diff",
      se = "se",
      df = 42,
      true = -1
    )
  )
  testthat::expect_error(
    object = simsum(
      data = tt,
      estvarname = "diff",
      se = "se",
      df = NA,
      true = -1
    )
  )
  testthat::expect_error(
    object = simsum(
      data = tt,
      estvarname = "diff",
      se = "se",
      df = "NULL",
      true = -1
    )
  )
})

testthat::test_that("expect equivalent values when passing lower, upper (based on df) and df", {
  data("tt", package = "rsimsum")

  s6 <- simsum(data = tt, estvarname = "diff", se = "se", df = "df", true = -1)
  s7 <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    ci.limits = c("conf.low", "conf.high"),
    true = -1
  )
  testthat::expect_equal(object = tidy(s6), expected = tidy(s7))

  s6 <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    methodvar = "method",
    df = "df",
    true = -1
  )
  s7 <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    methodvar = "method",
    ci.limits = c("conf.low", "conf.high"),
    true = -1
  )
  testthat::expect_equal(object = tidy(s6), expected = tidy(s7))

  s6 <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    methodvar = "method",
    by = "dgm",
    df = "df",
    true = -1
  )
  s7 <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    methodvar = "method",
    by = "dgm",
    ci.limits = c("conf.low", "conf.high"),
    true = -1
  )
  testthat::expect_equal(object = tidy(s6), expected = tidy(s7))
})

testthat::test_that("#55", {
  data("tt", package = "rsimsum")

  s <- simsum(data = tt, estvarname = "diff", se = "se", true = -1, x = TRUE)
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip", zip_ci_colours = "blue"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(s, type = "zip", zip_ci_colours = c("green", "red")),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(
      s,
      type = "zip",
      zip_ci_colours = c("green", "red", "blue")
    ),
    class = c("gg", "ggplot")
  )
  testthat::expect_error(
    object = autoplot(
      s,
      type = "zip",
      zip_ci_colours = c("green", "red", "blue", "yellow")
    )
  )
  testthat::expect_error(object = autoplot(s, type = "zip", zip_ci_colours = 1))
  testthat::expect_error(
    object = autoplot(s, type = "zip", zip_ci_colours = TRUE)
  )
})

### New tests for issue #23
testthat::context("#23")

testthat::test_that("expect error when autoplot 'vs' plots and no methods are defined", {
  data("tt", package = "rsimsum")
  s6 <- simsum(data = tt, estvarname = "diff", se = "se", true = -1, x = TRUE)

  testthat::expect_error(object = autoplot(s6, type = "est"))
  testthat::expect_error(object = autoplot(s6, type = "se"))
  testthat::expect_error(object = autoplot(s6, type = "est_ba"))
  testthat::expect_error(object = autoplot(s6, type = "se_ba"))
  testthat::expect_error(object = autoplot(s6, type = "est_density"))
  testthat::expect_error(object = autoplot(s6, type = "se_density"))
  testthat::expect_error(object = autoplot(s6, type = "est_hex"))
  testthat::expect_error(object = autoplot(s6, type = "se_hex"))
})

testthat::test_that("output from autoplot without methods is ok for other types of plot", {
  data("tt", package = "rsimsum")
  s6 <- simsum(data = tt, estvarname = "diff", se = "se", true = -1, x = TRUE)
  s7 <- simsum(
    data = tt,
    estvarname = "diff",
    se = "se",
    by = "dgm",
    true = -1,
    x = TRUE
  )

  testthat::expect_s3_class(object = autoplot(s6), class = c("gg", "ggplot"))
  testthat::expect_s3_class(
    object = autoplot(s6, type = "lolly"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(summary(s6)),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(summary(s6), type = "lolly"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(s6, type = "heat"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(s6, type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(s6, type = "est_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(s6, type = "se_ridge"),
    class = c("gg", "ggplot")
  )

  testthat::expect_s3_class(
    object = autoplot(s7, type = "heat"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(s7, type = "zip"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(s7, type = "est_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(s7, type = "se_ridge"),
    class = c("gg", "ggplot")
  )
  testthat::expect_s3_class(
    object = autoplot(s7, type = "nlp"),
    class = c("gg", "ggplot")
  )
})

#' @title autoplot method for summary.simsum objects
#'
#' @param object An object of class `summary.simsum`.
#' @inheritParams autoplot.simsum
#'
#' @return A `ggplot` object.
#' @export
#'
#' @examples
#' data("MIsim", package = "rsimsum")
#' s <- rsimsum::simsum(
#'   data = MIsim, estvarname = "b", true = 0.5, se = "se",
#'   methodvar = "method", x = TRUE
#' )
#' ss <- summary(s)
#'
#' library(ggplot2)
#' autoplot(ss)
#' autoplot(ss, type = "lolly")
autoplot.summary.simsum <- function(
  object,
  type = "forest",
  stats = "nsim",
  target = NULL,
  fitted = TRUE,
  scales = "fixed",
  top = TRUE,
  density.legend = TRUE,
  zoom = 1,
  zip_ci_colours = "yellow",
  ...
) {
  autoplot.simsum(
    object = object,
    type = type,
    stats = stats,
    target = target,
    fitted = fitted,
    scales = scales,
    top = top,
    density.legend = density.legend,
    zoom = zoom,
    zip_ci_colours = zip_ci_colours
  )
}

#' @title Example of a simulation study on missing data
#' @description A dataset from a simulation study comparing different ways to handle missing covariates when fitting a Cox model (White and Royston, 2009).
#' One thousand datasets were simulated, each containing normally distributed covariates \eqn{x} and \eqn{z} and time-to-event outcome.
#' Both covariates have 20% of their values deleted independently of all other variables so the data became missing completely at random (Little and Rubin, 2002).
#' Each simulated dataset was analysed in three ways.
#' A Cox model was fit to the complete cases (`CC`).
#' Then two methods of multiple imputation using chained equations (van Buuren, Boshuizen, and Knook, 1999) were used.
#' The `MI_LOGT` method multiply imputes the missing values of \eqn{x} and \eqn{z} with the outcome included as \eqn{\log (t)} and \eqn{d}, where \eqn{t} is the survival time and \eqn{d} is the event indicator.
#' The \code{MI_T} method is the same except that \eqn{\log (t)} is replaced by \eqn{t} in the imputation model.
#' The results are stored in long format.
#' @format A data frame with 3,000 rows and 4 variables:
#' * `dataset` Simulated dataset number.
#' * `method` Method used (`CC`, `MI_LOGT` or `MI_T`).
#' * `b` Point estimate.
#' * `se` Standard error of the point estimate.
#' @rdname MIsim
#' @note `MIsim2` is a version of the same dataset with the `method` column split into two columns, `m1` and `m2`.
#'
#' @examples
#' data("MIsim", package = "rsimsum")
#' data("MIsim2", package = "rsimsum")
#' @references White, I.R., and P. Royston. 2009. Imputing missing covariate values for the Cox model. Statistics in Medicine 28(15):1982-1998 \doi{10.1002/sim.3618}
"MIsim"

#' @rdname MIsim
"MIsim2"

#' @title Example of a simulation study on survival modelling
#' @description A dataset from a simulation study assessing the impact of misspecifying the baseline hazard in survival models on regression coefficients.
#' One thousand datasets were simulated, each containing a binary treatment variable with a log-hazard ratio of -0.50.
#' Survival data was simulated for two different sample sizes, 50 and 250 individuals, and under two different baseline hazard functions, exponential and Weibull.
#' Consequently, a Cox model (Cox, 1972), a fully parametric exponential model, and a Royston-Parmar (Royston and Parmar, 2002) model with two degrees of freedom were fit to each simulated dataset.
#' See `vignette("B-relhaz", package = "rsimsum")` for more information.
#' @format A data frame with 1,200 rows and 6 variables:
#' * `dataset` Simulated dataset number.
#' * `n` Sample size of the simulate dataset.
#' * `baseline` Baseline hazard function of the simulated dataset.
#' * `model` Method used (`Cox`, `Exp`, or `RP(2)`).
#' * `theta` Point estimate for the log-hazard ratio.
#' * `se` Standard error of the point estimate.
#'
#' @examples
#' data("relhaz", package = "rsimsum")
#' @references Cox D.R. 1972. Regression models and life-tables. Journal of the Royal Statistical Society, Series B (Methodological) 34(2):187-220. \doi{10.1007/978-1-4612-4380-9_37}
#' @references Royston, P. and Parmar, M.K. 2002. Flexible parametric proportional-hazards and proportional-odds models for censored survival data, with application to prognostic modelling and estimation of treatment effects. Statistics in Medicine 21(15):2175-2197 \doi{10.1002/sim.1203}
"relhaz"

#' @title Example of a simulation study on frailty survival models
#' @description A dataset from a simulation study comparing frailty flexible parametric models fitted using penalised likelihood to semiparametric frailty models.
#' Both models are fitted assuming a Gamma and a log-Normal frailty.
#' One thousand datasets were simulated, each containing a binary treatment variable with a log-hazard ratio of -0.50. Clustered survival data was simulated assuming 50 clusters of 50 individuals each, with a mixture Weibull baseline hazard function and a frailty following either a Gamma or a Log-Normal distribution.
#' The comparison involves estimates of the log-treatment effect, and estimates of heterogeneity (i.e. the estimated frailty variance).
#' @format A data frame with 16,000 rows and 6 variables:
#' * `i` Simulated dataset number.
#' * `b` Point estimate.
#' * `se` Standard error of the point estimate.
#' * `par` The estimand. `trt` is the log-treatment effect, `fv` is the variance of the frailty.
#' * `fv_dist` The true frailty distribution.
#' * `model` Method used (`Cox, Gamma`, `Cox, Log-Normal`, `RP(P), Gamma`, or `RP(P), Log-Normal`).
#' @rdname frailty
#' @note `frailty2` is a version of the same dataset with the `model` column split into two columns, `m_baseline` and `m_frailty`.
#'
#' @examples
#' data("frailty", package = "rsimsum")
#' data("frailty2", package = "rsimsum")
"frailty"

#' @rdname frailty
"frailty2"

#' @title Example of a simulation study on survival modelling
#' @description A dataset from a simulation study with 150 data-generating mechanisms, useful to illustrate nested loop plots. This simulation study aims to compare the Cox model and flexible parametric models in a variety of scenarios: different baseline hazard functions, sample size, and varying amount of heterogeneity unaccounted for in the model (simulated as white noise with a given variance). A Cox model and a Royston-Parmar model with 5 degrees of freedom are fit to each replication.
#' @format A data frame with 30,000 rows and 10 variables:
#' * `dgm` Data-generating mechanism, 1 to 150.
#' * `i` Simulated dataset number.
#' * `model` Method used, with 1 the Cox model and 2 the RP(5) model.
#' * `b` Point estimate for the log-hazard ratio.
#' * `se` Standard error of the point estimate.
#' * `baseline` Baseline hazard function of the simulated dataset.
#' * `ss` Sample size of the simulated dataset.
#' * `esigma` Standard deviation of the white noise.
#' * `pars` (Ancillary) Parameters of the baseline hazard function.
#'
#' @note Further details on this simulation study can be found in the R script used to generate this dataset, available on GitHub: [https://github.com/ellessenne/rsimsum/blob/master/data-raw/nlp-data.R](https://github.com/ellessenne/rsimsum/blob/master/data-raw/nlp-data.R)
#' @examples
#' data("nlp", package = "rsimsum")
#' @references Cox D.R. 1972. Regression models and life-tables. Journal of the Royal Statistical Society, Series B (Methodological) 34(2):187-220. \doi{10.1007/978-1-4612-4380-9_37}
#' @references Royston, P. and Parmar, M.K. 2002. Flexible parametric proportional-hazards and proportional-odds models for censored survival data, with application to prognostic modelling and estimation of treatment effects. Statistics in Medicine 21(15):2175-2197 \doi{10.1002/sim.1203}
#' @references Rücker, G. and Schwarzer, G. 2014. Presenting simulation results in a nested loop plot. BMC Medical Research Methodology 14:129 \doi{10.1186/1471-2288-14-129}
"nlp"

#' @title Example of a simulation study on the t-test
#' @description A dataset from a simulation study with 4 data-generating mechanisms, useful to illustrate custom input of confidence intervals to calculate coverage probability.
#' This simulation study aims to compare the t-test assuming pooled or unpooled variance in violation (or not) of the t-test assumptions: normality of data, and equality (or not) or variance between groups.
#' The true value of the difference between groups is -1.
#' @format A data frame with 4,000 rows and 8 variables:
#' * `diff` The difference in mean between groups estimated by the t-test;
#' * `se` Standard error of the estimated difference;
#' * `conf.low`, `conf.high` Confidence interval for the difference in mean as reported by the t-test;
#' * `df` The number of degrees of freedom assumed by the t-test;
#' * `repno` Identifies each replication, between 1 and 500;
#' * `dgm` Identifies each data-generating mechanism: 1 corresponds to normal data with equal variance between the groups, 2 is normal data with unequal variance, 3 and 4 are skewed data (simulated from a Gamma distribution) with equal and unequal variance between groups, respectively;
#' * `method` Analysis method: 1 represents the t-test with pooled variance, while 2 represents the t-test with unpooled variance.
#'
#' @note Further details on this simulation study can be found in the R script used to generate this dataset, available on GitHub: [https://github.com/ellessenne/rsimsum/blob/master/data-raw/tt-data.R](https://github.com/ellessenne/rsimsum/blob/master/data-raw/tt-data.R)
#' @examples
#' data("tt", package = "rsimsum")
"tt"

#' @title Summarising simsum objects
#' @description The `summary()` method for objects of class `simsum` returns confidence intervals for performance measures based on Monte Carlo standard errors.
#' @param object An object of class `simsum`.
#' @param ci_level Significance level for confidence intervals based on Monte Carlo standard errors. Ignored if a `simsum` object with control parameter `mcse = FALSE` is passed.
#' @param df Degrees of freedom of a t distribution that will be used to calculate confidence intervals based on Monte Carlo standard errors. If `NULL` (the default), quantiles of a Normal distribution will be used instead. However, using Z-based or t-based confidence intervals is valid only for summary statistics such a bias and coverage. Confidence intervals for other quantities may not be appropriate, therefore their usage is not recommended.
#' @param stats Summary statistics to include; can be a scalar value or a vector (for multiple summary statistics at once). Possible choices are:
#' * `nsim`, the number of replications with non-missing point estimates and standard error.
#' * `thetamean`, average point estimate.
#' * `thetamedian`, median point estimate.
#' * `se2mean`, average variance.
#' * `se2median`, median variance.
#' * `bias`, bias in point estimate.
#' * `rbias`, relative (to the true value) bias in point estimate.
#' * `empse`, empirical standard error.
#' * `mse`, mean squared error.
#' * `relprec`, percentage gain in precision relative to the reference method.
#' * `modelse`, model-based standard error.
#' * `relerror`, relative percentage error in standard error.
#' * `cover`, coverage of a nominal `level`\% confidence interval.
#' * `becover`, bias corrected coverage of a nominal `level`\% confidence interval.
#' * `power`, power of a (1 - `level`)\% level test.
#'
#' Defaults to `NULL`, in which case all possible summary statistics are included.
#' @param ... Ignored.
#' @return An object of class `summary.simsum`.
#' @seealso [simsum()], [print.summary.simsum()]
#' @export
#'
#' @examples
#' data("MIsim")
#' object <- simsum(
#'   data = MIsim, estvarname = "b", true = 0.5, se = "se",
#'   methodvar = "method"
#' )
#' xs <- summary(object)
#' xs
summary.simsum <- function(
  object,
  ci_level = 0.95,
  df = NULL,
  stats = NULL,
  ...
) {
  ### Check arguments
  arg_checks <- checkmate::makeAssertCollection()
  # 'level' must be a numeric value
  checkmate::assert_number(ci_level, lower = 0, upper = 1, add = arg_checks)
  # 'stats' must be one of the allowed values, can be NULL
  checkmate::assert_subset(
    x = stats,
    choices = c(
      "nsim",
      "thetamean",
      "thetamedian",
      "se2mean",
      "se2median",
      "bias",
      "rbias",
      "empse",
      "mse",
      "relprec",
      "modelse",
      "relerror",
      "cover",
      "becover",
      "power"
    ),
    add = arg_checks
  )
  ### Report if there are any errors
  if (!arg_checks$isEmpty()) {
    checkmate::reportAssertions(arg_checks)
  }

  ### Compute confidence intervals if Monte Carlo standard errors are available
  if (object$control$mcse) {
    if (is.null(df)) {
      crit <- stats::qnorm(1 - (1 - ci_level) / 2)
    } else {
      crit <- stats::qt(1 - (1 - ci_level) / 2, df = df)
    }
    object$summ$lower <- object$summ$est - crit * object$summ$mcse
    object$summ$upper <- object$summ$est + crit * object$summ$mcse
  }

  ### Add ci_level slot
  if (object$control$mcse) {
    object$ci_level <- ci_level
  }

  ## If no methodvar is specified, it is not possible to compute relprec
  if (is.null(object$methodvar)) {
    object$summ <- object$summ[object$summ$stat != "relprec", ]
  }

  ## Select only stats that are requested
  if (!is.null(stats)) {
    object$summ <- object$summ[object$summ$stat %in% stats, ]
  }

  ## Remove row.names
  row.names(object$summ) <- NULL

  ### Return object of class summary.simsum
  class(object) <- c("summary.simsum", "list")
  return(object)
}

#' @keywords internal
.performance <- function(
  data,
  estvarname,
  se,
  true,
  empse_ref = NULL,
  rho = NULL,
  ci.limits,
  df,
  control
) {
  ### Adjust 'true' to use internal column if not NULL
  if (!is.null(true)) {
    true <- ":true"
  }

  ### Make object to return
  obj <- list()

  ### Compute performance measures
  # Number of non-missing point estimates, standard errors:
  if (!is.null(se)) {
    nsim <- sum(
      !is.na(data[[estvarname]]) & !is.na(data[[se]]),
      na.rm = control$na.rm
    )
  } else {
    nsim <- sum(!is.na(data[[estvarname]]), na.rm = control$na.rm)
  }
  # Mean, median and variance of betas
  theta_mean <- mean(data[[estvarname]], na.rm = control$na.rm)
  theta_median <- stats::median(data[[estvarname]], na.rm = control$na.rm)
  # theta_var <- stats::var(data[[estvarname]], na.rm = control$na.rm)
  # Mean, median and average of ses
  if (!is.null(se)) {
    se2_mean <- mean(data[[se]]^2, na.rm = control$na.rm)
    se2_median <- stats::median(data[[se]]^2, na.rm = control$na.rm)
    se2_var <- stats::var(data[[se]]^2, na.rm = control$na.rm)
  }
  # Bias
  if (!is.null(true)) {
    bias <- 1 /
      nsim *
      sum(data[[estvarname]] - data[[":true"]], na.rm = control$na.rm)
  }
  # Relative bias
  if (!is.null(true)) {
    rbias <- 1 /
      nsim *
      sum(
        (data[[estvarname]] - data[[":true"]]) / data[[":true"]],
        na.rm = control$na.rm
      )
  }
  # Empirical standard error
  empse <- sqrt(
    1 /
      (nsim - 1) *
      sum(
        (data[[estvarname]] -
          mean(data[[estvarname]], na.rm = control$na.rm))^2,
        na.rm = control$na.rm
      )
  )
  # Mean squared error
  if (!is.null(true)) {
    mse <- 1 /
      nsim *
      sum((data[[estvarname]] - data[[":true"]])^2, na.rm = control$na.rm)
  }
  # Relative change in precision
  if (!is.null(empse_ref) & !is.null(rho)) {
    relprec <- 100 * ((empse_ref / empse)^2 - 1)
  } else {
    relprec <- NA
  }
  names(relprec) <- NULL
  # Model-based standard error
  if (!is.null(se)) {
    modelse <- sqrt(1 / nsim * sum(data[[se]]^2, na.rm = control$na.rm))
  }
  # Relative error in model-based standard error
  if (!is.null(se)) {
    relerror <- 100 * (modelse / empse - 1)
  }
  # Compute critical values (for coverage) from either a normal or a t distribution
  if (!is.null(df)) {
    crit <- stats::qt(1 - (1 - control$level) / 2, df = data[[df]])
  } else {
    crit <- stats::qnorm(1 - (1 - control$level) / 2)
  }
  # Compute critical value (for power) from either a normal or a t distribution
  crit_power <- ifelse(
    is.null(control$power_df),
    stats::qnorm(1 - (1 - control$level) / 2),
    stats::qt(1 - (1 - control$level) / 2, df = control$power_df)
  )
  # Coverage of a nominal (1 - level)% confidence interval
  if (!is.null(true) & !is.null(se)) {
    if (is.null(ci.limits)) {
      cover <- 1 /
        nsim *
        sum(
          data[[":true"]] >= data[[estvarname]] - crit * data[[se]] &
            data[[":true"]] <= data[[estvarname]] + crit * data[[se]],
          na.rm = control$na.rm
        )
    } else {
      if (is.character(ci.limits)) {
        cover <- 1 /
          nsim *
          sum(
            data[[":true"]] >= data[[ci.limits[1]]] &
              data[[":true"]] <= data[[ci.limits[2]]],
            na.rm = control$na.rm
          )
      } else if (is.numeric(ci.limits)) {
        data[["lower"]] <- ci.limits[1]
        data[["upper"]] <- ci.limits[2]
        cover <- 1 /
          nsim *
          sum(
            data[[":true"]] >= data[["lower"]] &
              data[[":true"]] <= data[["upper"]],
            na.rm = control$na.rm
          )
      }
    }
  }
  # Bias-corrected coverage of a nominal (1 - level)% confidence interval
  if (!is.null(se)) {
    if (is.null(ci.limits)) {
      becover <- 1 /
        nsim *
        sum(
          mean(data[[estvarname]], na.rm = control$na.rm) >=
            data[[estvarname]] - crit * data[[se]] &
            mean(data[[estvarname]], na.rm = control$na.rm) <=
              data[[estvarname]] + crit * data[[se]],
          na.rm = control$na.rm
        )
    } else {
      if (is.character(ci.limits)) {
        becover <- 1 /
          nsim *
          sum(
            mean(data[[estvarname]], na.rm = control$na.rm) >=
              data[[ci.limits[1]]] &
              mean(data[[estvarname]], na.rm = control$na.rm) <=
                data[[ci.limits[2]]],
            na.rm = control$na.rm
          )
      } else if (is.numeric(ci.limits)) {
        data[["lower"]] <- ci.limits[1]
        data[["upper"]] <- ci.limits[2]
        becover <- 1 /
          nsim *
          sum(
            mean(data[[estvarname]], na.rm = control$na.rm) >=
              data[[estvarname]] - data[["lower"]] &
              mean(data[[estvarname]], na.rm = control$na.rm) <=
                data[["upper"]],
            na.rm = control$na.rm
          )
      }
    }
  }
  # Power of a significance test at the `level` level
  if (!is.null(se)) {
    power <- 1 /
      nsim *
      sum(
        abs(data[[estvarname]]) >= crit_power * data[[se]],
        na.rm = control$na.rm
      )
  }

  ### Compute Monte Carlo SEs if requested:
  if (control$mcse) {
    if (!is.null(true)) {
      bias_mcse <- sqrt(
        1 /
          (nsim * (nsim - 1)) *
          sum(
            (data[[estvarname]] -
              mean(data[[estvarname]], na.rm = control$na.rm))^2,
            na.rm = control$na.rm
          )
      )
      rbias_i <- (data[[estvarname]] - data[[":true"]]) / data[[":true"]]
      rbias_mcse <- sd(rbias_i, na.rm = control$na.rm) / sqrt(nsim)
    }
    empse_mcse <- empse / sqrt(2 * (nsim - 1))
    if (!is.null(true)) {
      mse_mcse <- sqrt(
        sum(
          ((data[[estvarname]] - data[[":true"]])^2 - mse)^2,
          na.rm = control$na.rm
        ) /
          (nsim * (nsim - 1))
      )
    }
    if (!is.null(empse_ref) & !is.null(rho)) {
      relprec_mcse <- 200 *
        (empse_ref / empse)^2 *
        sqrt((1 - rho^2) / (nsim - 1))
    } else {
      relprec_mcse <- NA
    }
    names(relprec_mcse) <- NULL
    if (!is.null(se)) {
      modelse_mcse <- sqrt(se2_var / (4 * nsim * modelse^2))
    }
    if (!is.null(se)) {
      relerror_mcse <- 100 *
        (modelse / empse) *
        sqrt(se2_var / (4 * nsim * modelse^4) + 1 / (2 * (nsim - 1)))
    }
    if (!is.null(true) & !is.null(se)) {
      cover_mcse <- sqrt(cover * (1 - cover) / nsim)
    }
    if (!is.null(se)) {
      becover_mcse <- sqrt(becover * (1 - becover) / nsim)
    }
    if (!is.null(se)) power_mcse <- sqrt(power * (1 - power) / nsim)
  }

  # Easy thing to do is to add what can't be computed as null
  if (is.null(true)) {
    bias <- rbias <- cover <- mse <- NULL
    bias_mcse <- rbias_mcse <- cover_mcse <- mse_mcse <- NULL
  }
  if (is.null(se)) {
    se2_mean <- se2_median <- se2_var <- modelse <- relerror <- cover <- becover <- power <- NULL
    modelse_mcse <- relerror_mcse <- cover_mcse <- becover_mcse <- power_mcse <- NULL
  }

  ### Assemble object to return
  obj$stat <- c(
    "nsim",
    "thetamean",
    "thetamedian",
    "se2mean",
    "se2median",
    "bias",
    "rbias",
    "empse",
    "mse",
    "relprec",
    "modelse",
    "relerror",
    "cover",
    "becover",
    "power"
  )
  if (is.null(true)) {
    obj$stat <- obj$stat[!(obj$stat %in% c("bias", "rbias", "cover", "mse"))]
  }
  if (is.null(se)) {
    obj$stat <- obj$stat[
      !(obj$stat %in%
        c(
          "se2mean",
          "se2median",
          "modelse",
          "relerror",
          "cover",
          "becover",
          "power"
        ))
    ]
  }
  obj$est <- c(
    nsim,
    theta_mean,
    theta_median,
    se2_mean,
    se2_median,
    bias,
    rbias,
    empse,
    mse,
    relprec,
    modelse,
    relerror,
    cover,
    becover,
    power
  )
  if (control$mcse) {
    obj$mcse <- c(
      rep(NA, ifelse(is.null(se), 3, 5)),
      bias_mcse,
      rbias_mcse,
      empse_mcse,
      mse_mcse,
      relprec_mcse,
      modelse_mcse,
      relerror_mcse,
      cover_mcse,
      becover_mcse,
      power_mcse
    )
  }
  obj <- as.data.frame(obj, stringsAsFactors = FALSE)

  ### Return object
  return(obj)
}

#' @title autoplot method for simsum objects
#' @description `autoplot` can produce a series of plot to summarise results of simulation studies. See `vignette("C-plotting", package = "rsimsum")` for further details.
#' @param object An object of class `simsum`.
#' @param type The type of the plot to be produced. Possible choices are: `forest`, `lolly`, `zip`, `est`, `se`, `est_ba`, `se_ba`, `est_ridge`, `se_ridge`, `est_density`, `se_density`, `est_hex`, `se_hex`, `heat`, `nlp`, with `forest` being the default.
#' @param stats Summary statistic to plot, defaults to `nsim` (the number of replications with non-missing point estimates/SEs).
#' See [summary.simsum()] for further details on supported summary statistics.
#' @param target Target of summary statistic, e.g. 0 for `bias`. Defaults to `NULL`, in which case target will be inferred.
#' @param fitted Superimpose a fitted regression line, useful when `type` = (`est`, `se`, `est_ba`, `se_ba`, `est_density`, `se_density`, `est_hex`, `se_hex`). Defaults to `TRUE`.
#' @param scales Should scales be fixed (`fixed`, the default), free (`free`), or free in one dimension (`free_x`, `free_y`)?
#' @param top Should the legend for a nested loop plot be on the top side of the plot? Defaults to `TRUE`.
#' @param density.legend Should the legend for density and hexbin plots be included? Defaults to `TRUE`.
#' @param zoom A numeric value between 0 and 1 signalling that a zip plot should _zoom_ on the top x% of the plot (to ease interpretation). Defaults to 1, where the whole zip plot is displayed.
#' @param zip_ci_colours A string with (1) a hex code to use for plotting coverage probability and its Monte Carlo confidence intervals (the default, with value `zip_ci_colours = "yellow"`), (2) a string vector of two hex codes denoting optimal coverage (first element) and over/under coverage (second element) or (3) a vector of three hex codes denoting optimal coverage (first), undercoverage (second), and overcoverage (third).
#' @param ... Not used.
#'
#' @return A `ggplot` object.
#' @export
#'
#' @examples
#' data("MIsim", package = "rsimsum")
#' s <- rsimsum::simsum(
#'   data = MIsim, estvarname = "b", true = 0.5, se = "se",
#'   methodvar = "method", x = TRUE
#' )
#'
#' library(ggplot2)
#' autoplot(s)
#' autoplot(s, type = "lolly")
#' autoplot(s, type = "est_hex")
#' autoplot(s, type = "zip", zoom = 0.5)
#'
#' # Nested loop plot:
#' data("nlp", package = "rsimsum")
#' s1 <- rsimsum::simsum(
#'   data = nlp, estvarname = "b", true = 0, se = "se",
#'   methodvar = "model", by = c("baseline", "ss", "esigma")
#' )
#' autoplot(s1, stats = "bias", type = "nlp")
autoplot.simsum <- function(
  object,
  type = "forest",
  stats = "nsim",
  target = NULL,
  fitted = TRUE,
  scales = "fixed",
  top = TRUE,
  density.legend = TRUE,
  zoom = 1,
  zip_ci_colours = "yellow",
  ...
) {
  # object <- s.nlp.subset
  # type <- "nlp"
  # stats <- "bias"
  # target <- NULL
  # fitted <- TRUE
  # scales <- "fixed"
  # top <- TRUE
  # density.legend <- TRUE
  # zoom <- 1

  ### Check arguments
  arg_checks <- checkmate::makeAssertCollection()
  # 'type' must be a single string value, among those allowed
  checkmate::assert_string(x = type, add = arg_checks)
  checkmate::assert_subset(
    x = type,
    choices = c(
      "forest",
      "lolly",
      "zip",
      "est",
      "se",
      "est_ba",
      "se_ba",
      "est_ridge",
      "se_ridge",
      "est_density",
      "se_density",
      "est_hex",
      "se_hex",
      "heat",
      "nlp"
    ),
    empty.ok = FALSE,
    add = arg_checks
  )
  # 'stats' must be a single string value, among those allowed
  checkmate::assert_string(x = stats, add = arg_checks)
  checkmate::assert_subset(
    x = stats,
    choices = unique(object$summ$stat),
    empty.ok = FALSE,
    add = arg_checks
  )
  # 'target' must be single numeric value, can be null
  checkmate::assert_number(
    x = target,
    null.ok = TRUE,
    na.ok = FALSE,
    add = arg_checks
  )
  # 'zoom' needs to be a numeric value between zero and one
  checkmate::assert_number(
    x = zoom,
    lower = 0,
    upper = 1,
    null.ok = FALSE,
    na.ok = FALSE,
    add = arg_checks
  )
  # 'fitted', 'top', 'density.legend' must be a single boolean value
  checkmate::assert_logical(x = fitted, len = 1, add = arg_checks)
  checkmate::assert_logical(x = top, len = 1, add = arg_checks)
  checkmate::assert_logical(x = density.legend, len = 1, add = arg_checks)
  # 'scales' must be a single string value, among those allowed
  checkmate::assert_string(x = scales, add = arg_checks)
  checkmate::assert_subset(
    x = scales,
    choices = c("fixed", "free", "free_x", "free_y"),
    empty.ok = FALSE,
    add = arg_checks
  )
  # If type = 'zip', 'est', 'se', 'est_ba', 'se_ba', 'est_ridge', 'se_ridge' then object$x must be true
  if (
    type %in%
      c(
        "zip",
        "est",
        "se",
        "est_ba",
        "se_ba",
        "est_ridge",
        "se_ridge",
        "est_density",
        "se_density",
        "est_hex",
        "se_hex"
      )
  ) {
    checkmate::assert_true(x = !is.null(object$x), add = arg_checks)
  }
  # zip_ci_colours must be a string
  checkmate::assert_character(x = zip_ci_colours, min.len = 1, max.len = 3)
  # Report
  if (!arg_checks$isEmpty()) {
    checkmate::reportAssertions(arg_checks)
  }

  ### All 'vs' plots not meaningful if there are no 'methodvar' to compare
  if (
    type %in%
      c(
        "est",
        "se",
        "est_ba",
        "se_ba",
        "est_density",
        "se_density",
        "est_hex",
        "se_hex"
      ) &
      is.null(object[["methodvar"]])
  ) {
    stop(
      "This plot is not meaningful when no methods are compared",
      call. = FALSE
    )
  }

  ### All 'se' plots not meaningful if there is no 'se'
  if (
    type %in%
      c("se", "se_ba", "se_ridge", "se_density", "se_hex") &
      is.null(object[["se"]])
  ) {
    stop(
      "Plot of standard errors not available without standard errors",
      call. = FALSE
    )
  }

  ### Zip plot not meaningful if there are no standard errors
  if (
    type == "zip" &
      (is.null(object[["true"]]) |
        is.null(object[["se"]]) |
        !is.numeric(object[["true"]]))
  ) {
    stop(
      "Zip plot not available without a (single numeric) true value or standard errors",
      call. = FALSE
    )
  }

  ### Nested loop plot not meaningful if there are no 'by' factors
  if (type == "nlp" & is.null(object[["by"]])) {
    stop(
      "Nested loop plot not meaningful when no 'by' factors are defined",
      call. = FALSE
    )
  }

  ### Lolly plot with no true values not available for some stats
  if (
    type == "lolly" &
      stats %in% c("thetamean", "thetamedian") &
      !is.numeric(object[["true"]])
  ) {
    stop(
      "Lolly plot not available for average/median point estimates if 'true' is not defined (as a single numeric value)",
      call. = FALSE
    )
  }

  ### Extract data
  df <- tidy(object, stats = stats)

  ### Infer target
  if (is.null(target)) {
    if (stats %in% c("cover", "becover", "power")) {
      target <- 0.95
    } else if (stats %in% c("thetamean", "thetamedian")) {
      target <- object[["true"]]
    } else {
      target <- 0
    }
  }

  ### Add CI if it is a summary object
  ci <- inherits(x = object, what = "summary.simsum")

  ### Process if string of methods columns to be combined
  user_methodvar <- NULL
  if (length(object$methodvar) > 1) {
    reftable <- .compact_method_columns(
      data = df,
      methodvar = object$methodvar
    )$reftable
    df <- .compact_method_columns(data = df, methodvar = object$methodvar)$data
    object$user_methodvar <- object$methodvar
    object$methodvar <- ":methodvar"
  }

  ### Call internal function to build plot
  plot <- switch(
    type,
    "forest" = .forest_plot(
      data = df,
      methodvar = object$methodvar,
      by = object$by,
      stats = stats,
      ci = ci,
      target = target,
      scales = scales
    ),
    "lolly" = .lolly_plot(
      data = df,
      methodvar = object$methodvar,
      by = object$by,
      stats = stats,
      ci = ci,
      target = target,
      scales = scales
    ),
    "zip" = .zip_plot(
      data = object$x,
      estvarname = object$estvarname,
      se = object$se,
      true = object$true,
      methodvar = object$methodvar,
      by = object$by,
      ci.limits = object$ci.limits,
      df = object$df,
      control = object$control,
      summ = object$summ,
      zoom = zoom,
      zip_ci_colours = zip_ci_colours
    ), # zip for coverage
    "est" = .vs_plot(
      data = object$x,
      b = object$estvarname,
      methodvar = object$methodvar,
      by = object$by,
      fitted = fitted,
      scales = scales,
      ba = FALSE
    ),
    "se" = .vs_plot(
      data = object$x,
      b = object$se,
      methodvar = object$methodvar,
      by = object$by,
      fitted = fitted,
      scales = scales,
      ba = FALSE
    ),
    "est_ba" = .vs_plot(
      data = object$x,
      b = object$estvarname,
      methodvar = object$methodvar,
      by = object$by,
      fitted = fitted,
      scales = scales,
      ba = TRUE
    ),
    "se_ba" = .vs_plot(
      data = object$x,
      b = object$se,
      methodvar = object$methodvar,
      by = object$by,
      fitted = fitted,
      scales = scales,
      ba = TRUE
    ),
    "est_ridge" = .ridge_plot(
      data = object$x,
      b = object$estvarname,
      methodvar = object$methodvar,
      by = object$by
    ),
    "se_ridge" = .ridge_plot(
      data = object$x,
      b = object$se,
      methodvar = object$methodvar,
      by = object$by
    ),
    "est_density" = .density_plot(
      data = object$x,
      b = object$estvarname,
      methodvar = object$methodvar,
      by = object$by,
      fitted = fitted,
      scales = scales,
      hex = FALSE,
      density.legend = density.legend
    ),
    "se_density" = .density_plot(
      data = object$x,
      b = object$se,
      methodvar = object$methodvar,
      by = object$by,
      fitted = fitted,
      scales = scales,
      hex = FALSE,
      density.legend = density.legend
    ),
    "est_hex" = .density_plot(
      data = object$x,
      b = object$estvarname,
      methodvar = object$methodvar,
      by = object$by,
      fitted = fitted,
      scales = scales,
      hex = TRUE,
      density.legend = density.legend
    ),
    "se_hex" = .density_plot(
      data = object$x,
      b = object$se,
      methodvar = object$methodvar,
      by = object$by,
      fitted = fitted,
      scales = scales,
      hex = TRUE,
      density.legend = density.legend
    ),
    "heat" = .heat_plot(
      data = df,
      methodvar = object$methodvar,
      by = object$by,
      stats = stats
    ),
    "nlp" = .nlp(
      data = df,
      methodvar = object$methodvar,
      by = object$by,
      stats = stats,
      target = target,
      top = top
    )
  )

  ### Return plot
  return(plot)
}

testthat::context("data")

testthat::test_that("loading MIsim works", {
  data("MIsim", package = "rsimsum")
  testthat::expect_s3_class(object = MIsim, class = "data.frame")
})

testthat::test_that("loading MIsim2 works", {
  data("MIsim2", package = "rsimsum")
  testthat::expect_s3_class(object = MIsim2, class = "data.frame")
})

testthat::test_that("MIsim and MIsim2 are the same (if we forget about the methods stuff)", {
  data("MIsim", package = "rsimsum")
  attr(x = MIsim, which = "label") <- NULL
  data("MIsim2", package = "rsimsum")
  testthat::expect_equal(object = nrow(MIsim), expected = nrow(MIsim2))
  .columns <- c("dataset", "b", "se")
  MIsim_small <- MIsim[, .columns]
  MIsim2_small <- MIsim2[, .columns]
  testthat::expect_equal(object = MIsim_small, expected = MIsim2_small)
})

testthat::test_that("loading relhaz works", {
  data("relhaz", package = "rsimsum")
  testthat::expect_s3_class(object = relhaz, class = "data.frame")
})

testthat::test_that("loading frailty works", {
  data("frailty", package = "rsimsum")
  testthat::expect_s3_class(object = frailty, class = "data.frame")
})

testthat::test_that("loading frailty2 works", {
  data("frailty2", package = "rsimsum")
  testthat::expect_s3_class(object = frailty2, class = "data.frame")
})

testthat::test_that("frailty and frailty2 are the same (if we forget about the model stuff)", {
  data("frailty", package = "rsimsum")
  data("frailty2", package = "rsimsum")
  testthat::expect_equal(object = nrow(frailty), expected = nrow(frailty2))
  .columns <- c("i", "b", "se", "par", "fv_dist")
  frailty_small <- frailty[, .columns]
  frailty2_small <- frailty2[, .columns]
  testthat::expect_equal(object = frailty_small, expected = frailty2_small)
})

testthat::test_that("loading nlp works", {
  data("nlp", package = "rsimsum")
  testthat::expect_s3_class(object = nlp, class = "data.frame")
})

testthat::test_that("loading tt works", {
  data("tt", package = "rsimsum")
  testthat::expect_s3_class(object = tt, class = "data.frame")
  testthat::expect_equal(object = dim(tt), expected = c(4000, 8))
})

---
title: "Introduction to rsimsum"
author: "Alessandro Gasparini"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to rsimsum}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options:
  chunk_output_type: console
---

```{r setup, include = FALSE}
options(width = 150)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.align = "center",
  fig.height = 6,
  fig.width = 6,
  out.width = "80%"
)
```

# rsimsum

`rsimsum` is an R package that can compute summary statistics from simulation studies.
It is inspired by the user-written command `simsum` in Stata (White I.R., 2010).

The aim of `rsimsum` is helping reporting of simulation studies, including understanding the role of chance in results of simulation studies.
Specifically, `rsimsum` can compute Monte Carlo standard errors of summary statistics, defined as the standard deviation of the estimated summary statistic; these are reported by default.

Formula for summary statistics and Monte Carlo standard errors are presented in the next section.
Note that the terms _summary statistic_ and _performance measure_ are used interchangeably.

# Notation

We will use th following notation throughout this vignette:

* $\theta$: an estimand, and its true value
* $n_{\text{sim}}$: number of simulations
* $i = 1, \dots, n_{\text{sim}}$: indexes a given simulation
* $\hat{\theta}_i$: the estimated value of $\theta$ for the $i^{\text{th}}$ replication
* $\widehat{\text{Var}}(\hat{\theta}_i)$: the estimated variance $\text{Var}(\hat{\theta}_i)$ of $\hat{\theta}_i$ for the $i^{\text{th}}$ replication
* $\text{Var}(\hat{\theta})$: the empirical variance of $\hat{\theta}$
* $\alpha$: the nominal significance level

# Performance measures

The first performance measure of interest is **bias**, which quantifies whether the estimator targets the true value $\theta$ on average. Bias is calculated as:

$$
\text{Bias} = \frac{1}{n_{\text{sim}}} \sum_{i = 1} ^ {n_{\text{sim}}} \hat{\theta}_i - \theta
$$

The Monte Carlo standard error of bias is calculated as:

$$
\text{MCSE(Bias)} = \sqrt{\frac{\frac{1}{n_{\text{sim}} - 1} \sum_{i = 1} ^ {n_{\text{sim}}} (\hat{\theta}_i - \bar{\theta}) ^ 2}{n_{\text{sim}}}}
$$

`rsimsum` can also compute **relative bias** (_relative_ to the true value $\theta$), which can be interpreted similarly as with bias, but in relative terms rather than absolute.
This is calculated as:

$$
\text{Relative Bias} = \frac{1}{n_{\text{sim}}} \sum_{i = 1} ^ {n_{\text{sim}}} \frac{\hat{\theta}_i - \theta}{\theta}
$$

Its Monte Carlo standard error is calculated as:

$$
\text{MCSE(Relative Bias)} = \sqrt{\frac{1}{n_{\text{sim}} (n_{\text{sim}} - 1)} \sum_i^{n_{\text{sim}}} \left[ \frac{\hat{\theta}_i - \theta}{\theta} - \widehat{\text{Relative Bias}} \right]^2}
$$

The **empirical standard error** of $\theta$ depends only on $\hat{\theta}$ and does not require any knowledge of $\theta$.
It estimates the standard deviation of $\hat{\theta}$ over the $n_{\text{sim}}$ replications:

$$
\text{Empirical SE} = \sqrt{\frac{1}{n_{\text{sim}} - 1} \sum_{i = 1} ^ {n_{\text{sim}}} (\hat{\theta}_i - \bar{\theta}) ^ 2}
$$

The Monte Carlo standard error is calculated as:

$$
\text{MCSE(Emp. SE)} = \frac{\widehat{\text{Emp. SE}}}{\sqrt{2 (n_{\text{sim}} - 1)}}
$$

When comparing different methods, the **relative precision of a given method B against a reference method A** is computed as:

$$
\text{Relative \% increase in precision} = 100 \left[ \left( \frac{\widehat{\text{Emp. SE}}_A}{\widehat{\text{Emp. SE}}_B} \right) ^ 2 - 1 \right]
$$

Its (approximated) Monte Carlo standard error is:

$$
\text{MCSE(Relative \% increase in precision)} \simeq 200 \left( \frac{\widehat{\text{Emp. SE}}_A}{\widehat{\text{Emp. SE}}_B} \right)^2 \sqrt{\frac{1 - \rho^2_{AB}}{n_{\text{sim}} - 1}}
$$

$\rho^2_{AB}$ is the correlation of $\hat{\theta}_A$ and $\hat{\theta}_B$.


A measure that takes into account both precision and accuracy of a method is the **mean squared error**, which is the sum of the squared bias and variance of $\hat{\theta}$:

$$
\text{MSE} = \frac{1}{n_{\text{sim}}} \sum_{i = 1} ^ {n_{\text{sim}}} (\hat{\theta}_i - \theta) ^ 2
$$

The Monte Carlo standard error is:

$$
\text{MCSE(MSE)} = \sqrt{\frac{\sum_{i = 1} ^ {n_{\text{sim}}} \left[ (\hat{\theta}_i - \theta) ^2 - \text{MSE} \right] ^ 2}{n_{\text{sim}} (n_{\text{sim}} - 1)}}
$$

The **model based standard error** is computed by averaging the estimated standard errors for each replication:

$$
\text{Model SE} = \sqrt{\frac{1}{n_{\text{sim}}} \sum_{i = 1} ^ {n_{\text{sim}}} \widehat{\text{Var}}(\hat{\theta}_i)}
$$

Its (approximated) Monte Carlo standard error is computed as:

$$
\text{MCSE(Model SE)} \simeq \sqrt{\frac{\text{Var}[\widehat{\text{Var}}(\hat{\theta}_i)]}{4 n_{\text{sim}} \widehat{\text{Model SE}}}}
$$

The model standard error targets the empirical standard error. Hence, the **relative error in the model standard error** is an informative performance measure:

$$
\text{Relative \% error in model SE} = 100 \left( \frac{\text{Model SE}}{\text{Empirical SE}} - 1\right)
$$

Its Monte Carlo standard error is computed as:

$$
\text{MCSE(Relative \% error in model SE)} = 100 \left( \frac{\text{Model SE}}{\text{Empirical SE}} \right) \sqrt{\frac{\text{Var}[\widehat{\text{Var}}(\hat{\theta}_i)]}{4 n_{\text{sim}} \widehat{\text{Model SE}} ^ 4} + \frac{1}{2(n_{\text{sim}} - 1)}}
$$

**Coverage** is another key property of an estimator. It is defined as the probability that a confidence interval contains the true value $\theta$, and computed as:

$$
\text{Coverage} = \frac{1}{n_{\text{sim}}} \sum_{i = 1} ^ {n_{\text{sim}}} I(\hat{\theta}_{i, \text{low}} \le \theta \le \hat{\theta}_{i, \text{upp}})
$$

where $I(\cdot)$ is the indicator function. The Monte Carlo standard error is computed as:

$$
\text{MCSE(Coverage)} = \sqrt{\frac{\text{Coverage} \times (1 - \text{Coverage})}{n_{\text{sim}}}}
$$

Under coverage is to be expected if:

1. $\text{Bias} \ne 0$, or
2. $\text{Models SE} < \text{Empirical SE}$, or
3. the distribution of $\hat{\theta}$ is not normal and intervals have been constructed assuming normality, or
4. $\widehat{\text{Var}}(\hat{\theta}_i)$ is too variable

Over coverage occurs as a result of $\text{Models SE} > \text{Empirical SE}$.

As under coverage may be a result of bias, another useful summary statistic is **bias-eliminated coverage**:

$$
\text{Bias-eliminated (BE) coverage} = \frac{1}{n_{\text{sim}}} \sum_{i = 1} ^ {n_{\text{sim}}} I(\hat{\theta}_{i, \text{low}} \le \bar{\theta} \le \hat{\theta}_{i, \text{upp}})
$$

The Monte Carlo standard error is analogously as coverage:

$$
\text{MCSE(BE coverage)} = \sqrt{\frac{\text{BE coverage} \times (1 - \text{BE coverage})}{n_{\text{sim}}}}
$$

Finally, **power** of a significance test at the $\alpha$ level is defined as:

$$
\text{Power} = \frac{1}{n_{\text{sim}}} \sum_{i = 1} ^ {n_{\text{sim}}} I \left[ |\hat{\theta}_i| \ge z_{\alpha/2} \times \sqrt{\widehat{\text{Var}}(\hat{\theta_i})} \right]
$$

The Monte Carlo standard error is analogously as coverage:

$$
\text{MCSE(Power)} = \sqrt{\frac{\text{Power} \times (1 - \text{Power})}{n_{\text{sim}}}}
$$

Further information on summary statistics for simulation studies can be found in White (2010) and Morris, White, and Crowther (2019).

# Example 1: Simulation study on missing data

With this example dataset included in `rsimsum` we aim to summarise a simulation study comparing different ways to handle missing covariates when fitting a Cox model (White and Royston, 2009). One thousand datasets were simulated, each containing normally distributed covariates $x$ and $z$ and time-to-event outcome. Both covariates has $20\%$ of their values deleted independently of all other variables so the data became missing completely at random (Little and Rubin, 2002). Each simulated dataset was analysed in three ways. A Cox model was fit to the complete cases (`CC`). Then two methods of multiple imputation using chained equations (van Buuren, Boshuizen, and Knook, 1999) were used. The `MI_LOGT` method multiply imputes the missing values of $x$ and $z$ with the outcome included as $\log(t)$ and $d$, where $t$ is the survival time and $d$ is the event indicator. The `MI_T` method is the same except that $\log(t)$ is replaced by $t$ in the imputation model.

We load the data in the usual way:

```{r ex1-load-data}
library(rsimsum)
data("MIsim", package = "rsimsum")
```

Let's have a look at the first 10 rows of the dataset:

```{r ex1-inspect-data}
head(MIsim, n = 10)
```

The included variables are:

```{r ex1-included-variables}
str(MIsim)
```

* `dataset`, the number of the simulated dataset;

* `method`, the method used (`CC`, `MI_LOGT` or `MI_T`);

* `b`, the point estimate;

* `se`, the standard error of the point estimate.

We summarise the results of the simulation study by method using the `simsum` function:

```{r ex1-simsum}
s1 <- simsum(data = MIsim, estvarname = "b", true = 0.50, se = "se", methodvar = "method", ref = "CC")
```

We set `true = 0.50` as the true value of the point estimate `b` - under which the data was simulated - is 0.50. We select `CC` as the reference method as we consider the complete cases analysis the reference method to benchmark against; if we do not set a reference method, `simsum` picks one automatically.

Using the default settings, Monte Carlo standard errors are computed and returned.

Summarising a `simsum` object, we obtain the following output:

```{r ex1-summary-simsum}
ss1 <- summary(s1)
ss1
```

The output begins with a brief overview of the setting of the simulation study (e.g. the method variable, unique methods, etc.), and continues with each summary statistic by method (if defined, as in this case).
The values that are reported are point estimates with Monte Carlo standard errors in brackets; however, it is also possible to require confidence intervals based on Monte Carlo standard errors to be reported instead:

```{r ex1-summary-simsum-ci}
print(ss1, mcse = FALSE)
```

Highlighting some points of interest from the summary results above:

1. The `CC` method has small-sample bias away from the null (point estimate 0.0168, with 95% confidence interval: 0.0074 - 0.0261);
2. `CC` is inefficient compared with `MI_LOGT` and `MI_T`: the relative gain in precision for these two methods is 1.3105% and 1.2637% compared to `CC`, respectively;
3. Model-based standard errors are close to empirical standard errors;
4. Coverage of nominal 95% confidence intervals also seems fine, which is not surprising in view of the generally low (or lack of) bias and good model-based standard errors;
5. `CC` has lower power compared with `MI_LOGT` and `MI_T`, which is not surprising
in view of its inefficiency.

## Tabulating summary statistics

It is straightforward to produce a table of summary statistics for use in an R Markdown document:

```{r ex1-table}
library(knitr)
kable(tidy(ss1))
```

Using `tidy()` in combination with R packages such as [xtable](https://cran.r-project.org/package=xtable), [kableExtra](https://cran.r-project.org/package=kableExtra), [tables](https://cran.r-project.org/package=tables) can yield a variety of tables that should suit most purposes.

More information on producing tables directly from R can be found in the [CRAN Task View on Reproducible Research](https://CRAN.R-project.org/view=ReproducibleResearch).

## Plotting summary statistics

In this section, we show how to plot and compare summary statistics using the popular R package [ggplot](https://CRAN.R-project.org/package=ggplot2).

Plotting bias by method with $95\%$ confidence intervals based on Monte Carlo standard errors:

```{r ex1-plot-bias}
library(ggplot2)
ggplot(tidy(ss1, stats = "bias"), aes(x = method, y = est, ymin = lower, ymax = upper)) +
  geom_hline(yintercept = 0, color = "red", lty = "dashed") +
  geom_point() +
  geom_errorbar(width = 1 / 3) +
  theme_bw() +
  labs(x = "Method", y = "Bias")
```

Conversely, say we want to visually compare coverage for the three methods compared with this simulation study:

```{r ex1-plot-cov}
ggplot(tidy(ss1, stats = "cover"), aes(x = method, y = est, ymin = lower, ymax = upper)) +
  geom_hline(yintercept = 0.95, color = "red", lty = "dashed") +
  geom_point() +
  geom_errorbar(width = 1 / 3) +
  coord_cartesian(ylim = c(0, 1)) +
  theme_bw() +
  labs(x = "Method", y = "Coverage")
```

## Dropping large estimates and standard errors

`rsimsum` allows to automatically drop estimates and standard errors that are larger than a predefined value. Specifically, the argument of `simsum` that control this behaviour is `dropbig`, with tuning parameters `dropbig.max` and `dropbig.semax` that can be passed via the `control` argument.

Set `dropbig` to `TRUE` and standardised estimates larger than `max` in absolute value will be dropped; standard errors larger than `semax` times the average standard error will be dropped too.
By default, robust standardisation is used (based on median and inter-quartile range); however, it is also possible to request regular standardisation (based on mean and standard deviation) by setting the control parameter `dropbig.robust = FALSE`.

For instance, say we want to drop standardised estimates larger than $3$ in absolute value and standard errors larger than $1.5$ times the average standard error:

```{r ex1-dropbig-simsum}
s1.2 <- simsum(data = MIsim, estvarname = "b", true = 0.50, se = "se", methodvar = "method", ref = "CC", dropbig = TRUE, control = list(dropbig.max = 4, dropbig.semax = 1.5))
```

Some estimates were dropped, as we can see from the number of non-missing point estimates, standard errors:

```{r ex1-dropbig-nsim}
summary(s1.2, stats = "nsim")
```

Everything else works analogously as before; for instance, to summarise the results:

```{r ex1-dropbig-summary-simsum}
summary(s1.2)
```

# Example 2: Simulation study on survival modelling

```{r ex2-load-data}
data("relhaz", package = "rsimsum")
```

Let's have a look at the first 10 rows of the dataset:

```{r ex2-inspect-data}
head(relhaz, n = 10)
```

The included variables are:

```{r ex2-included-variables}
str(relhaz)
```

* `dataset`, simulated dataset number;

* `n`, sample size of the simulate dataset;

* `baseline`, baseline hazard function of the simulated dataset;

* `model`, method used (Cox model or Royston-Parmar model with 2 degrees of freedom);

* `theta`, point estimate for the log-hazard ratio;

* `se`, standard error of the point estimate.

`rsimsum` can summarise results from simulation studies with several data-generating mechanisms. For instance, with this example we show how to compute summary statistics by baseline hazard function and sample size.

In order to summarise results by data-generating factors, it is sufficient to define the "by" factors in the call to `simsum`:

```{r ex2-simsum}
s2 <- simsum(data = relhaz, estvarname = "theta", true = -0.50, se = "se", methodvar = "model", by = c("baseline", "n"))
s2
```

The difference between `methodvar` and `by` is as follows: `methodvar` represents methods (e.g. the two models, in this example) compared with this simulation study, while `by` represents all possible data-generating factors that varied when simulating data (in this case, sample size and the true baseline hazard function).

Summarising the results will be printed out for each method and combination of data-generating factors:

```{r ex2-summary-simsum}
ss2 <- summary(s2)
ss2
```

## Plotting summary statistics

Tables could get cumbersome when there are many different data-generating mechanisms. Plots are generally easier to interpret, and can be generated as easily as before.

Say we want to compare bias for each method by baseline hazard function and sample size using faceting:

```{r ex2-plot-bias}
ggplot(tidy(ss2, stats = "bias"), aes(x = model, y = est, ymin = lower, ymax = upper)) +
  geom_hline(yintercept = 0, color = "red", lty = "dashed") +
  geom_point() +
  geom_errorbar(width = 1 / 3) +
  facet_grid(baseline ~ n) +
  theme_bw() +
  labs(x = "Method", y = "Bias")
```

# References

* White, I.R. 2010. _simsum: Analyses of simulation studies including Monte Carlo error_. The Stata Journal 10(3): 369-385
* Morris, T.P., White, I.R. and Crowther, M.J. 2019. _Using simulation studies to evaluate statistical methods_. Statistics in Medicine 38:2074-2102
* White, I.R., and P. Royston. 2009. _Imputing missing covariate values for the Cox model_. Statistics in Medicine 28(15):1982-1998
* Little, R.J.A., and D.B. Rubin. 2002. _Statistical analysis with missing data_. 2nd ed. Hoboken, NJ: Wiley
* van Buuren, S., H.C. Boshuizen, and D.L. Knook. 1999. _Multiple imputation of missing blood pressure covariates in survival analysis_. Statistics in Medicine 18(6):681-694 

---
title: "rsimsum and the tidyverse"
author: "Alessandro Gasparini"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{rsimsum and the tidyverse}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
options(width = 150)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.align = "center", fig.height = 6, fig.width = 6,
  out.width = "75%"
)
```

`rsimsum` plays nice with the tidyverse.

```{r packages}
library(rsimsum)
library(dplyr)
library(ggplot2)
library(knitr)
```

For instance, it is possible to chain functions using the piping operator `%>%` to obtain tables and plots with a single call:

```{r table}
data("MIsim", package = "rsimsum")
MIsim %>%
  simsum(estvarname = "b", se = "se", methodvar = "method", true = 0.5) %>%
  summary() %>%
  tidy() %>%
  kable()
```

```{r plot}
MIsim %>%
  simsum(estvarname = "b", se = "se", methodvar = "method", true = 0.5) %>%
  summary() %>%
  tidy(stats = "bias") %>%
  ggplot(aes(x = method, y = est, ymin = lower, ymax = upper)) +
  geom_hline(yintercept = 0, color = "red", lty = "dashed") +
  geom_point() +
  geom_errorbar(width = 1 / 3) +
  theme_bw() +
  labs(x = "Method", y = "Bias")
```

---
title: "Nested loop plots"
author: "Alessandro Gasparini"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Nested loop plots}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include = FALSE}
options(width = 150)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.align = "center", fig.width = 7,
  out.width = "80%"
)
```

As of version `0.6.0`, `rsimsum` supports the fully automated creation of nested loop plots (Rücker and Schwarzer, 2014).

```{r package}
library(rsimsum)
```

A dataset that can be purposefully used to illustrate nested loop plots is bundled and shipped with `rsimsum`:

```{r data}
data("nlp", package = "rsimsum")
```

This data set contains the results of a simulation study on survival modelling with 150 distinct data-generating mechanisms:

```{r display-data}
head(nlp)
```

Further information on the data could be find in the help file (`?nlp`).

We can analyse this simulation study using `rsimsum` as usual:

```{r analyse}
s <- rsimsum::simsum(
  data = nlp, estvarname = "b", true = 0, se = "se",
  methodvar = "model", by = c("baseline", "ss", "esigma")
)
s
```

Finally, a nested loop plot can be automatically produced via the `autoplot` method, e.g. for bias:

```{r nlp, fig.asp = 0.75}
library(ggplot2)
autoplot(s, type = "nlp", stats = "bias")
```

However:

1. Nested loop plots are suited for several DGMs but not for several methods;
1. The decision on how to _nest_ the results is subjective - the top-level of nesting receives most emphasis;
1. It gives an _overall_ impression, without focusing too much on details;
1. It is cumbersome to incorporate Monte Carlo errors in the plot.

# References

* Rücker, G. and Schwarzer, G. 2014 _Presenting simulation results in a nested loop plot_. BMC Medical Research Methodology 14(1) <[doi:10.1186/1471-2288-14-129](https://doi.org/10.1186/1471-2288-14-129)>

---
title: "Simulating a simulation study"
author: "Alessandro Gasparini"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Simulating a simulation study}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---
```{r setup, include = FALSE}
options(width = 150)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.align = "center", fig.height = 6, fig.width = 6,
  out.width = "75%"
)
```

# Introduction

In this vignette, we show how the simulated data included as an example dataset in `simsum` has been generated.

# Motivation

Say we want to run a simulation study in which we want to compare the sensitivity of parametric and semiparametric survival models on relative risk estimates.

# Data generating mechanisms

We simulate an hypothetical trial with a binary treatment. We fix the log-treatment effect to $-0.50$, and we generate a treatment indicator variable for each simulated individual via a $Binom(1, 0.5)$ random variable. We simulate two different sample sizes (50 and 250 individuals) and we assume two different baseline hazard functions: exponential with scale parameter $\lambda = 0.5$, and Weibull with scale parameter $\lambda = 0.5$ and shape parameter $\gamma = 1.5$. Finally, we apply administrative censoring at time $t = 5$.

```{r baseline-hazards}
exp_basehaz <- function(t, lambda = 0.5) lambda * 1 * t^0
exp_weibull <- function(t, lambda = 0.5, gamma = 1.5) lambda * gamma * t^(gamma - 1)
curve(exp_basehaz, from = 0, to = 5, lty = 1, ylim = c(0, 2), ylab = expression(h[0](t)), xlab = "Follow-up time t")
curve(exp_weibull, from = 0, to = 5, lty = 2, add = TRUE)
legend(x = "topleft", lty = 1:2, legend = c("Exponential baseline hazard", "Weibull baseline hazard"), bty = "n")
```

The survival times are estimated using the approach of Bender _et al_. (2005), based on drawing from a $U(0, 1)$ random variable and applying the following transformations:

1. for an exponential baseline hazard, the survival time $t$ is simulated as:
$$t = -\frac{log(U)}{\lambda \exp(\beta ^ T X)}$$

2. for a Weibull baseline hazard, the survival time $t$ is simulated as:
$$t = \left(-\frac{log(U)}{\lambda \exp(\beta ^ T X)}\right) ^ {1 / \gamma}$$

The R function to simulate a dataset for our simulation study is defined as follows:

```{r dgfun}
simulate_data <- function(dataset, n, baseline, params = list(), coveff = -0.50) {
  # Simulate treatment indicator variable
  x <- rbinom(n = n, size = 1, prob = 0.5)
  # Draw from a U(0,1) random variable
  u <- runif(n)
  # Simulate survival times depending on the baseline hazard
  if (baseline == "Exponential") {
    t <- -log(u) / (params$lambda * exp(x * coveff))
  } else {
    t <- (-log(u) / (params$lambda * exp(x * coveff)))^(1 / params$gamma)
  }
  # Winsorising tiny values for t (smaller than one day on a yearly-scale, e.g. 1 / 365.242), and adding a tiny amount of white noise not to have too many concurrent values
  t <- ifelse(t < 1 / 365.242, 1 / 365.242, t)
  t[t == 1 / 365.242] <- t[t == 1 / 365.242] + rnorm(length(t[t == 1 / 365.242]), mean = 0, sd = 1e-4)
  # ...and make sure that the resulting value is positive
  t <- abs(t)

  # Make event indicator variable applying administrative censoring at t = 5
  d <- as.numeric(t < 5)
  t <- pmin(t, 5)
  # Return a data.frame
  data.frame(dataset = dataset, x = x, t = t, d = d, n = n, baseline = baseline, stringsAsFactors = FALSE)
}
```

# Methods

We compare the Cox model (Cox, 1972) with a fully parametric survival model assuming an exponential baseline hazard and a flexible parametric model with 2 degrees of freedom for modelling the baseline hazard (Royston and Parmar, 2002). The Cox model can be fit via the `coxph` function from the `survival` package, the exponential model can be fit via the `phreg` function from the `eha` package, and the Royston-Parmar model can be fixed via the `stpm2` function from the `rstpm2` package.

# Performance measures

Say we are interested in the following performance measures:

* Bias in the estimated log-treatment effect, and corresponding $95\%$ Monte Carlo confidence
intervals
* Coverage of confidence intervals for the log-treatment effect, defined as the proportion of simulated data sets for which the true log-treatment effect of $-0.50$ lies within the $95\%$ confidence intervals obtained from the model

# Sample size

We are primarily interested in bias, and assume that the variance of the estimated log-treatment effect is $0.1$. The Monte Carlo standard error for the bias is:

$$\text{MCSE} = \sqrt{\frac{\text{variance}}{\# \text{simulations}}}$$

Aiming for a Monte Carlo standard error of 0.01 on the estimated bias, we would require $1,000$ replications. 

The Monte Carlo standard error for coverage is:

$$\text{MCSE} = \sqrt{\frac{\text{coverage} \times (1 - \text{coverage})}{\# \text{simulations}}}$$

This Monte Carlo standard error is maximised for a coverage = $0.5$. In that setting, the Monte Carlo standard error with $1,000$ replications would be $0.01581139$, which is deemed to be acceptable.

Therefore, we should run $1,000$ replications of this simulation study.
However, for simplicity, we will run $100$ replications only to speed up the process.

# Running the simulation study

## Generate data

We generate $100$ datasets for each data-generating mechanism.

First, we set a random seed for reproducibility:

```{r set-seed}
set.seed(755353002)
```

Then, we simulate the data:

```{r generate-data}
reps <- 1:100
data <- list()
data[["n = 50, baseline = Exp"]] <- lapply(
  X = reps,
  FUN = simulate_data,
  n = 50,
  baseline = "Exponential",
  params = list(lambda = 0.5)
)
data[["n = 250, baseline = Exp"]] <- lapply(
  X = reps,
  FUN = simulate_data,
  n = 250,
  baseline = "Exponential",
  params = list(lambda = 0.5)
)
data[["n = 50, baseline = Wei"]] <- lapply(
  X = reps,
  FUN = simulate_data,
  n = 50,
  baseline = "Weibull",
  params = list(lambda = 0.5, gamma = 1.5)
)
data[["n = 250, baseline = Wei"]] <- lapply(
  X = reps,
  FUN = simulate_data,
  n = 250,
  baseline = "Weibull",
  params = list(lambda = 0.5, gamma = 1.5)
)
```

## Run models

We define a function to fit the models of interest:

```{r fitting-function}
library(survival)
library(rstpm2)
library(eha)

fit_models <- function(data, model) {
  # Fit model
  if (model == "Cox") {
    fit <- survival::coxph(Surv(t, d) ~ x, data = data)
  } else if (model == "RP(2)") {
    fit <- rstpm2::stpm2(Surv(t, d) ~ x, data = data, df = 2)
  } else {
    fit <- eha::phreg(Surv(t, d) ~ x, data = data, dist = "weibull", shape = 1)
  }
  # Return relevant coefficients
  data.frame(
    dataset = unique(data$dataset),
    n = unique(data$n),
    baseline = unique(data$baseline),
    theta = coef(fit)["x"],
    se = sqrt(ifelse(model == "Exp", fit$var["x", "x"], vcov(fit)["x", "x"])),
    model = model,
    stringsAsFactors = FALSE,
    row.names = NULL
  )
}
```

We now run the models for each simulated dataset:

```{r run-models}
results <- list()
results[["n = 50, baseline = Exp, model = Cox"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 50, baseline = Exp"]],
    FUN = fit_models,
    model = "Cox"
  )
)
results[["n = 250, baseline = Exp, model = Cox"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 250, baseline = Exp"]],
    FUN = fit_models,
    model = "Cox"
  )
)
results[["n = 50, baseline = Wei, model = Cox"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 50, baseline = Wei"]],
    FUN = fit_models,
    model = "Cox"
  )
)
results[["n = 250, baseline = Wei, model = Cox"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 250, baseline = Wei"]],
    FUN = fit_models,
    model = "Cox"
  )
)

results[["n = 50, baseline = Exp, model = Exp"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 50, baseline = Exp"]],
    FUN = fit_models,
    model = "Exp"
  )
)
results[["n = 250, baseline = Exp, model = Exp"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 250, baseline = Exp"]],
    FUN = fit_models,
    model = "Exp"
  )
)
results[["n = 50, baseline = Wei, model = Exp"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 50, baseline = Wei"]],
    FUN = fit_models,
    model = "Exp"
  )
)
results[["n = 250, baseline = Wei, model = Exp"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 250, baseline = Wei"]],
    FUN = fit_models,
    model = "Exp"
  )
)

results[["n = 50, baseline = Exp, model = RP(2)"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 50, baseline = Exp"]],
    FUN = fit_models,
    model = "RP(2)"
  )
)
results[["n = 250, baseline = Exp, model = RP(2)"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 250, baseline = Exp"]],
    FUN = fit_models,
    model = "RP(2)"
  )
)
results[["n = 50, baseline = Wei, model = RP(2)"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 50, baseline = Wei"]],
    FUN = fit_models,
    model = "RP(2)"
  )
)
results[["n = 250, baseline = Wei, model = RP(2)"]] <- do.call(
  rbind.data.frame,
  lapply(
    X = data[["n = 250, baseline = Wei"]],
    FUN = fit_models,
    model = "RP(2)"
  )
)
```

## Aggregating results

```{r aggregate-results}
relhaz <- do.call(
  rbind.data.frame,
  results
)
row.names(relhaz) <- NULL
```

We save the final results, that will be included as an example in the R package `rsimsum`.

```{r saving, eval = FALSE}
library(usethis)
usethis::use_data(relhaz, overwrite = TRUE)
```

## Summarising results

Finally, we obtain summary statistics by calling the `simsum` function:

```{r compute-summaries}
library(rsimsum)
s <- rsimsum::simsum(data = relhaz, estvarname = "theta", se = "se", true = -0.50, methodvar = "model", ref = "Cox", by = c("n", "baseline"))
s
```

```{r print-summaries}
summary(s)
```

# Conclusions

With this vignette we showed how to simulate survival data and run a small, simple simulation study.

# References

* Cox D.R. _Regression models and life-tables_. Journal of the Royal Statistical Society, Series B (Methodological), 1972, 34(2):187-220

* Royston P. and Parmar M.K. _Flexible parametric proportional-hazards and proportional-odds models for censored survival data, with application to prognostic modelling and estimation of treatment effects_. Statistics in Medicine, 2002, 21(15):2175-2197 

* Bender R., Augustin T., and Blettner M. _Generating survival times to simulate Cox proportional hazards models_. Statistics in Medicine, 2005, 24(11):1713-1723

---
title: "Custom input values for confidence intervals and true values"
author: "Alessandro Gasparini"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Custom input values for confidence intervals and true values}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include = FALSE}
options(width = 150)
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.align = "center", fig.height = 6, fig.width = 6,
  out.width = "75%"
)
```

# Single Estimand

`rsimsum` supports custom input values for the true value of the estimand and for confidence intervals limits (used to calculate coverage probability).

To illustrate this feature, we can use the `tt` dataset (bundled with `rsimsum`):

```{r packages}
library(rsimsum)
data("tt", package = "rsimsum")
head(tt)
```

This includes the results of a simulation study assessing robustness of the t-test when estimating the difference between means.
The t-test assumes a t distribution, hence confidence intervals for the estimated mean are generally based on the t distribution.
See for instance the example from the t-test documentation (`?t.test`):

```{r}
t.test(extra ~ group, data = sleep)
```

We can incorporate custom confidence intervals by passing the name of two columns in `data` as the `ci.limits` argument:

```{r}
s1 <- simsum(data = tt, estvarname = "diff", true = -1, se = "se", ci.limits = c("conf.low", "conf.high"), methodvar = "method", by = "dgm")
summary(s1, stats = "cover")
```

By doing so, we can incorporate different types of confidence intervals in the analysis of Monte Carlo simulation studies.
Compare with the default setting:

```{r}
s2 <- simsum(data = tt, estvarname = "diff", true = -1, se = "se", methodvar = "method", by = "dgm")
summary(s2, stats = "cover")
```

The `ci.limits` is also useful when using non-symmetrical confidence intervals, e.g. when using bootstrapped confidence intervals.

A pair of values can also be passed to `rsimsum` as the `ci.limits` argument:

```{r}
s3 <- simsum(data = tt, estvarname = "diff", true = -1, se = "se", ci.limits = c(-1.5, -0.5), methodvar = "method", by = "dgm")
summary(s3, stats = "cover")
```

If you have a better example of the utility of this method please get in touch: I'd love to hear from you!

By default, `simsum` will calculate confidence intervals using normal-theory, Wald-type intervals.
It is possible to use t-based critical values by providing a column for the (replication-specific) degrees of freedom (analogously as passing confidence bounds to `ci.limits`):

```{r}
s4 <- simsum(data = tt, estvarname = "diff", true = -1, se = "se", df = "df", methodvar = "method", by = "dgm")
```

Given that the confidence intervals in (`conf.low`, `conf.high`) are obtained by using critical values from a t distribution, the results of `s4` will be equivalent to the results of `s1`:

```{r}
all.equal(tidy(s1), tidy(s4))
```

We can pass a column of values for `true` as well:

```{r}
tt$true <- -1
s5 <- simsum(data = tt, estvarname = "diff", true = "true", se = "se", ci.limits = c("conf.low", "conf.high"), methodvar = "method", by = "dgm")
summary(s5, stats = "cover")
```

Compare with the default settings:

```{r}
summary(s2, stats = "cover")
```

Finally, we could have multiple columns identifying methods as well.
This uses the `MIsim` and `MIsim2` datasets, which are bundled with {rsimsum}:

```{r}
data("MIsim", package = "rsimsum")
data("MIsim2", package = "rsimsum")
head(MIsim)
head(MIsim2)
```

The syntax when calling `simsum()` is pretty much the same:

```{r}
s6 <- simsum(data = MIsim, estvarname = "b", true = 0.50, se = "se", methodvar = "method")
s7 <- simsum(data = MIsim2, estvarname = "b", true = 0.50, se = "se", methodvar = c("m1", "m2"))
```

See the inferred methods:

```{r}
print(s6)
print(s7)
```

And of course, the estimated performance measures are the same:

```{r}
all.equal(tidy(s6)$est, tidy(s7)$est)
```

# Multiple Estimands at Once

`multisimsum` can be as flexible as `simsum`.
Remember the default behaviour:

```{r}
data("frailty", package = "rsimsum")
ms1 <- multisimsum(
  data = frailty,
  par = "par", true = c(trt = -0.50, fv = 0.75),
  estvarname = "b", se = "se", methodvar = "model",
  by = "fv_dist"
)
summary(ms1, stats = "bias")
```

In this example, we pass the true values of each estimand as the named vector `c(trt = -0.50, fv = 0.75)`.

Say instead we stored the true value of each estimand as a column in our dataset:

```{r}
frailty$true <- ifelse(frailty$par == "trt", -0.50, 0.75)
head(frailty)
```

With this data structure, we can pass a string value to `multisimsum` that will identify the `true` column in our dataset:

```{r}
ms2 <- multisimsum(
  data = frailty,
  par = "par", true = "true",
  estvarname = "b", se = "se", methodvar = "model",
  by = "fv_dist"
)
summary(ms2, stats = "bias")
```

We can confirm that we obtain the same results with the two approaches:

```{r}
identical(tidy(ms1), tidy(ms2))
```

This approach is particularly useful when the true value might vary across replications (e.g. when it depends on the simulated dataset).

Of course, it can be combined with custom confidence interval limits for coverage as well:

```{r}
frailty$conf.low <- frailty$b - qt(1 - 0.05 / 2, df = 10) * frailty$se
frailty$conf.high <- frailty$b + qt(1 - 0.05 / 2, df = 10) * frailty$se

ms3 <- multisimsum(
  data = frailty,
  par = "par", true = "true",
  estvarname = "b", se = "se", methodvar = "model",
  by = "fv_dist",
  ci.limits = c("conf.low", "conf.high")
)
summary(ms3, stats = "cover")
```

This will be completely different than before:

```{r}
summary(ms2, stats = "cover")
```

Multiple columns identifying methods are supported with `multisimsum()` as well; examples are omitted here, but it works analogously as with `simsum()`.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nsim.R
\name{nsim}
\alias{nsim}
\title{Compute number of simulations required}
\usage{
nsim(alpha, sigma, delta, power = 0.5)
}
\arguments{
\item{alpha}{Significance level. Must be a value between 0 and 1.}

\item{sigma}{Variance for the parameter of interest. Must be greater than 0.}

\item{delta}{Specified level of accuracy of the estimate of interest you are willing to accept. Must be greater than 0.}

\item{power}{Power to detect a specific difference from the true value as significant. Must be a value between 0 and 1. Defaults to 0.5, e.g. a power of 50\%.}
}
\value{
A scalar value \eqn{B}{B} representing the number of simulations to perform based on the accuracy required.
}
\description{
The function \code{nsim} computes the number of simulations \eqn{B}{B} to perform based on the accuracy of an estimate of interest, using the following equation:
\deqn{B = \left( \frac{(Z_{1 - \alpha / 2} + Z_{1 - theta}) \sigma}{\delta} \right) ^ 2,}{B = [((Z(1 - \alpha / 2) + Z(1 - \theta)) \sigma) / \delta] ^ 2 }
where \eqn{\delta} is the specified level of accuracy of the estimate of interest you are willing to accept (i.e. the permissible difference from the true value \eqn{\beta}{\beta}), \eqn{Z_{1 - \alpha / 2}}{Z(1 - \alpha / 2)} is the \eqn{(1 - \alpha / 2)}{(1 - \alpha / 2)} quantile of the standard normal distribution, \eqn{Z_{1 - \theta}}{Z(1 - \theta)} is the \eqn{(1 - \theta)}{(1 - \theta)} quantile of the standard normal distribution with \eqn{(1 - \theta)}{1 - \theta} being the power to detect a specific difference from the true value as significant, and \eqn{\sigma ^ 2}{\sigma ^ 2]} is the variance of the parameter of interest.
}
\examples{
# Number of simulations required to produce an estimate to within 5\%
# accuracy of the true coefficient of 0.349 with a 5\% significance level,
# assuming the variance of the estimate is 0.0166 and 50\% power:
nsim(alpha = 0.05, sigma = sqrt(0.0166), delta = 0.349 * 5 / 100, power = 0.5)

# Number of simulations required to produce an estimate to within 1\%
# accuracy of the true coefficient of 0.349 with a 5\% significance level,
# assuming the variance of the estimate is 0.0166 and 50\% power:
nsim(alpha = 0.05, sigma = sqrt(0.0166), delta = 0.349 * 1 / 100, power = 0.5)
}
\references{
Burton, A., Douglas G. Altman, P. Royston. et al. 2006. The design of simulation studies in medical statistics. Statistics in Medicine 25: 4279-4292 \doi{10.1002/sim.2673}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.summary.simsum.R
\name{print.summary.simsum}
\alias{print.summary.simsum}
\title{print.summary.simsum}
\usage{
\method{print}{summary.simsum}(x, digits = 4, mcse = TRUE, ...)
}
\arguments{
\item{x}{An object of class \code{summary.simsum}.}

\item{digits}{Number of significant digits used for printing.
Defaults to 4.}

\item{mcse}{Should Monte Carlo standard errors be reported?
If \code{mcse = FALSE}, confidence intervals based on Monte Carlo standard errors will be reported instead, see \code{\link[=summary.simsum]{summary.simsum()}}.
If a \code{NULL} value is passed, only point estimates are printed regardless of whether Monte Carlo standard errors were computed or not.
Defaults to \code{TRUE}.}

\item{...}{Ignored.}
}
\description{
Print method for \code{summary.simsum} objects
}
\examples{
data("MIsim")
x <- simsum(
  data = MIsim, estvarname = "b", true = 0.5, se = "se",
  methodvar = "method"
)
xs <- summary(x)
xs

# Printing less significant digits:
print(xs, digits = 2)

# Printing confidence intervals:
print(xs, mcse = FALSE)

# Printing values only:
print(xs, mcse = NULL)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoplot.multisimsum.R
\name{autoplot.multisimsum}
\alias{autoplot.multisimsum}
\title{autoplot method for multisimsum objects}
\usage{
\method{autoplot}{multisimsum}(
  object,
  par,
  type = "forest",
  stats = "nsim",
  target = NULL,
  fitted = TRUE,
  scales = "fixed",
  top = TRUE,
  density.legend = TRUE,
  zoom = 1,
  zip_ci_colours = "yellow",
  ...
)
}
\arguments{
\item{object}{An object of class \code{multisimsum}.}

\item{par}{The parameter results to plot.}

\item{type}{The type of the plot to be produced. Possible choices are: \code{forest}, \code{lolly}, \code{zip}, \code{est}, \code{se}, \code{est_ba}, \code{se_ba}, \code{est_density}, \code{se_density}, \code{est_hex}, \code{se_hex}, \code{est_ridge}, \code{se_ridge}, \code{heat}, \code{nlp}, with \code{forest} being the default.}

\item{stats}{Summary statistic to plot, defaults to \code{bias}. See \code{\link[=summary.simsum]{summary.simsum()}} for further details on supported summary statistics.}

\item{target}{Target of summary statistic, e.g. 0 for \code{bias}. Defaults to \code{NULL}, in which case target will be inferred.}

\item{fitted}{Superimpose a fitted regression line, useful when \code{type} = (\code{est}, \code{se}, \code{est_ba}, \code{se_ba}, \code{est_density}, \code{se_density}, \code{est_hex}, \code{se_hex}). Defaults to \code{TRUE}.}

\item{scales}{Should scales be fixed (\code{fixed}, the default), free (\code{free}), or free in one dimension (\code{free_x}, \code{free_y})?}

\item{top}{Should the legend for a nested loop plot be on the top side of the plot? Defaults to \code{TRUE}.}

\item{density.legend}{Should the legend for density and hexbin plots be included? Defaults to \code{TRUE}.}

\item{zoom}{A numeric value between 0 and 1 signalling that a zip plot should \emph{zoom} on the top x\% of the plot (to ease interpretation). Defaults to 1, where the whole zip plot is displayed.}

\item{zip_ci_colours}{A string with (1) a hex code to use for plotting coverage probability and its Monte Carlo confidence intervals (the default, with value \code{zip_ci_colours = "yellow"}), (2) a string vector of two hex codes denoting optimal coverage (first element) and over/under coverage (second element) or (3) a vector of three hex codes denoting optimal coverage (first), undercoverage (second), and overcoverage (third).}

\item{...}{Not used.}
}
\value{
A \code{ggplot} object.
}
\description{
\code{autoplot} can produce a series of plot to summarise results of simulation studies. See \code{vignette("C-plotting", package = "rsimsum")} for further details.
}
\examples{
data("frailty", package = "rsimsum")
ms <- multisimsum(
  data = frailty,
  par = "par", true = c(trt = -0.50, fv = 0.75),
  estvarname = "b", se = "se", methodvar = "model",
  by = "fv_dist", x = TRUE
)

library(ggplot2)
autoplot(ms, par = "trt")
autoplot(ms, par = "trt", type = "lolly", stats = "cover")
autoplot(ms, par = "trt", type = "zip")
autoplot(ms, par = "trt", type = "est_ba")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.multisimsum.R
\name{print.multisimsum}
\alias{print.multisimsum}
\title{print.multisimsum}
\usage{
\method{print}{multisimsum}(x, ...)
}
\arguments{
\item{x}{An object of class \code{multisimsum}.}

\item{...}{Ignored.}
}
\description{
Print method for multisimsum objects
}
\examples{
data(frailty)
ms <- multisimsum(
  data = frailty, par = "par", true = c(
    trt = -0.50,
    fv = 0.75
  ), estvarname = "b", se = "se", methodvar = "model",
  by = "fv_dist"
)
ms

data("frailty", package = "rsimsum")
frailty$true <- ifelse(frailty$par == "trt", -0.50, 0.75)
ms <- multisimsum(data = frailty, par = "par", estvarname = "b", true = "true")
ms
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multisimsum.R
\name{multisimsum}
\alias{multisimsum}
\title{Analyses of simulation studies with multiple estimands at once, including Monte Carlo error}
\usage{
multisimsum(
  data,
  par,
  estvarname,
  se = NULL,
  true = NULL,
  methodvar = NULL,
  ref = NULL,
  by = NULL,
  ci.limits = NULL,
  df = NULL,
  dropbig = FALSE,
  x = FALSE,
  control = list()
)
}
\arguments{
\item{data}{A \code{data.frame} in which variable names are interpreted.
It has to be in tidy format, e.g. each variable forms a column and each observation forms a row.}

\item{par}{The name of the variable containing the methods to compare.
Can be \code{NULL}.}

\item{estvarname}{The name of the variable containing the point estimates.
Note that some column names are forbidden: these are listed below in the \emph{Details} section.}

\item{se}{The name of the variable containing the standard errors of the point estimates.
Note that some column names are forbidden: these are listed below in the \emph{Details} section.}

\item{true}{The true value of the parameter; this is used in calculations of bias, relative bias, coverage, and mean squared error and is required whenever these performance measures are requested.
\code{true} can be a numeric value or a string that identifies a column in \code{data}.
In the former setting, \code{simsum} will assume the same value for all replications; conversely, each replication will use a distinct value for \code{true} as identified by each row of \code{data}.
See \code{vignette("E-custom-inputs", package = "rsimsum")} for more details.
Note that some column names are forbidden: these are listed below in the \emph{Details} section.}

\item{methodvar}{The name of the variable containing the methods to compare.
For instance, methods could be the models compared within a simulation study.
Can be \code{NULL}.
If a vector of column names is passed to \code{simsum()}, those columns will be combined into a single column named \verb{:methodvar} using the \code{\link[base:interaction]{base::interaction()}} function before computing all performance measures.
Note that some column names are forbidden: these are listed below in the \emph{Details} section.}

\item{ref}{Specifies the reference method against which relative precision will be calculated.
Only useful if \code{methodvar} is specified.}

\item{by}{A vector of variable names to compute performance measures by a list of factors. Factors listed here are the (potentially several) data-generating mechanisms used to simulate data under different scenarios (e.g. sample size, true distribution of a variable, etc.).
Can be \code{NULL}.
Note that some column names are forbidden: these are listed below in the \emph{Details} section.}

\item{ci.limits}{Can be used to specify the limits (lower and upper) of confidence intervals used to calculate coverage and bias-eliminated coverage.
Useful for non-Wald type estimators (e.g. bootstrap).
Defaults to \code{NULL}, where Wald-type confidence intervals based on the provided SEs are calculated for coverage; otherwise, it can be a numeric vector (for fixed confidence intervals) or a vector of strings that identify columns in \code{data} with replication-specific lower and upper limits.
See \code{vignette("E-custom-inputs", package = "rsimsum")} for more details.
Note that some column names are forbidden: these are listed below in the \emph{Details} section.}

\item{df}{Can be used to specify that a column containing the replication-specific number of degrees of freedom that will be used to calculate confidence intervals for coverage (and bias-eliminated coverage) assuming t-distributed critical values (rather than normal theory intervals).
See \code{vignette("E-custom-inputs", package = "rsimsum")} for more details.
Note that some column names are forbidden: these are listed below in the \emph{Details} section.}

\item{dropbig}{Specifies that point estimates or standard errors beyond the maximum acceptable values should be dropped. Defaults to \code{FALSE}.}

\item{x}{Set to \code{TRUE} to include the \code{data} argument used to calculate summary statistics (i.e. after pre-processing the input dataset e.g. removing values deemed too large via the \code{dropbig} argument) as a slot. Calling \code{simsum} with \code{x = TRUE} is required to produce zipper plots. The downside is that the size of the returned object increases considerably, therefore it is set to \code{FALSE} by default.}

\item{control}{A list of parameters that control the behaviour of \code{simsum}.
Possible values are:
\itemize{
\item \code{mcse}, whether to calculate Monte Carlo standard errors. Defaults to \code{TRUE};
\item \code{level}, the significance level used for coverage, bias-eliminated coverage, and power. Defaults to 0.95;
\item \code{power_df}, whether to use robust critical values from a t distribution with \code{power_df} degrees of freedom when calculating power. Defaults to \code{NULL}, in which case a Gaussian distribution is used;
\item \code{na.rm}, whether to remove point estimates or standard errors where either (or both) is missing. Defaults to \code{TRUE};
\item \code{char.sep}, a character utilised when splitting the input dataset \code{data}. Generally, this should not be changed;
\item \code{dropbig.max}, specifies the maximum acceptable absolute value of the point estimates, after standardisation. Defaults to 10;
\item \code{dropbig.semax}, specifies the maximum acceptable absolute value of the standard error, after standardisation. Defaults to 100
\item \code{dropbig.robust}, specifies whether to use robust standardisation (using median and inter-quartile range) rather than normal standardisation (using mean and standard deviation). Defaults to \code{TRUE}, in which case robust standardisation will be used for \code{dropbig}.
}}
}
\value{
An object of class \code{multisimsum}.
}
\description{
\code{multisimsum} is an extension of \code{\link[=simsum]{simsum()}} that can handle multiple estimated parameters at once.
\code{multisimsum} calls \code{\link[=simsum]{simsum()}} internally, each estimands at once.
There is only one new argument that must be set when calling \code{multisimsum}: \code{par}, a string representing the column of \code{data} that identifies the different estimands.
Additionally, with \code{multisimsum} the argument \code{true} can be a named vector, where names correspond to each estimand (see examples).
Otherwise, constant values (or values identified by a column in \code{data}) will be utilised.
See \code{vignette("E-custom-inputs", package = "rsimsum")} for more details.
}
\details{
The following names are not allowed for \code{estvarname}, \code{se}, \code{methodvar}, \code{by}, \code{par}: \code{stat}, \code{est}, \code{mcse}, \code{lower}, \code{upper}, \verb{:methodvar}.
}
\examples{
data("frailty", package = "rsimsum")
ms <- multisimsum(
  data = frailty,
  par = "par", true = c(trt = -0.50, fv = 0.75),
  estvarname = "b", se = "se", methodvar = "model",
  by = "fv_dist"
)
ms
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{nlp}
\alias{nlp}
\title{Example of a simulation study on survival modelling}
\format{
A data frame with 30,000 rows and 10 variables:
\itemize{
\item \code{dgm} Data-generating mechanism, 1 to 150.
\item \code{i} Simulated dataset number.
\item \code{model} Method used, with 1 the Cox model and 2 the RP(5) model.
\item \code{b} Point estimate for the log-hazard ratio.
\item \code{se} Standard error of the point estimate.
\item \code{baseline} Baseline hazard function of the simulated dataset.
\item \code{ss} Sample size of the simulated dataset.
\item \code{esigma} Standard deviation of the white noise.
\item \code{pars} (Ancillary) Parameters of the baseline hazard function.
}
}
\usage{
nlp
}
\description{
A dataset from a simulation study with 150 data-generating mechanisms, useful to illustrate nested loop plots. This simulation study aims to compare the Cox model and flexible parametric models in a variety of scenarios: different baseline hazard functions, sample size, and varying amount of heterogeneity unaccounted for in the model (simulated as white noise with a given variance). A Cox model and a Royston-Parmar model with 5 degrees of freedom are fit to each replication.
}
\note{
Further details on this simulation study can be found in the R script used to generate this dataset, available on GitHub: \url{https://github.com/ellessenne/rsimsum/blob/master/data-raw/nlp-data.R}
}
\examples{
data("nlp", package = "rsimsum")
}
\references{
Cox D.R. 1972. Regression models and life-tables. Journal of the Royal Statistical Society, Series B (Methodological) 34(2):187-220. \doi{10.1007/978-1-4612-4380-9_37}

Royston, P. and Parmar, M.K. 2002. Flexible parametric proportional-hazards and proportional-odds models for censored survival data, with application to prognostic modelling and estimation of treatment effects. Statistics in Medicine 21(15):2175-2197 \doi{10.1002/sim.1203}

Rücker, G. and Schwarzer, G. 2014. Presenting simulation results in a nested loop plot. BMC Medical Research Methodology 14:129 \doi{10.1186/1471-2288-14-129}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.simsum.R
\name{summary.simsum}
\alias{summary.simsum}
\title{Summarising simsum objects}
\usage{
\method{summary}{simsum}(object, ci_level = 0.95, df = NULL, stats = NULL, ...)
}
\arguments{
\item{object}{An object of class \code{simsum}.}

\item{ci_level}{Significance level for confidence intervals based on Monte Carlo standard errors. Ignored if a \code{simsum} object with control parameter \code{mcse = FALSE} is passed.}

\item{df}{Degrees of freedom of a t distribution that will be used to calculate confidence intervals based on Monte Carlo standard errors. If \code{NULL} (the default), quantiles of a Normal distribution will be used instead. However, using Z-based or t-based confidence intervals is valid only for summary statistics such a bias and coverage. Confidence intervals for other quantities may not be appropriate, therefore their usage is not recommended.}

\item{stats}{Summary statistics to include; can be a scalar value or a vector (for multiple summary statistics at once). Possible choices are:
\itemize{
\item \code{nsim}, the number of replications with non-missing point estimates and standard error.
\item \code{thetamean}, average point estimate.
\item \code{thetamedian}, median point estimate.
\item \code{se2mean}, average variance.
\item \code{se2median}, median variance.
\item \code{bias}, bias in point estimate.
\item \code{rbias}, relative (to the true value) bias in point estimate.
\item \code{empse}, empirical standard error.
\item \code{mse}, mean squared error.
\item \code{relprec}, percentage gain in precision relative to the reference method.
\item \code{modelse}, model-based standard error.
\item \code{relerror}, relative percentage error in standard error.
\item \code{cover}, coverage of a nominal \code{level}\\% confidence interval.
\item \code{becover}, bias corrected coverage of a nominal \code{level}\\% confidence interval.
\item \code{power}, power of a (1 - \code{level})\\% level test.
}

Defaults to \code{NULL}, in which case all possible summary statistics are included.}

\item{...}{Ignored.}
}
\value{
An object of class \code{summary.simsum}.
}
\description{
The \code{summary()} method for objects of class \code{simsum} returns confidence intervals for performance measures based on Monte Carlo standard errors.
}
\examples{
data("MIsim")
object <- simsum(
  data = MIsim, estvarname = "b", true = 0.5, se = "se",
  methodvar = "method"
)
xs <- summary(object)
xs
}
\seealso{
\code{\link[=simsum]{simsum()}}, \code{\link[=print.summary.simsum]{print.summary.simsum()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.summary.multisimsum.R
\name{print.summary.multisimsum}
\alias{print.summary.multisimsum}
\title{print.summary.multisimsum}
\usage{
\method{print}{summary.multisimsum}(x, digits = 4, mcse = TRUE, ...)
}
\arguments{
\item{x}{An object of class \code{summary.multisimsum}.}

\item{digits}{Number of significant digits used for printing.
Defaults to 4.}

\item{mcse}{Should Monte Carlo standard errors be reported?
If \code{mcse = FALSE}, confidence intervals based on Monte Carlo standard errors will be reported instead, see \code{\link[=summary.multisimsum]{summary.multisimsum()}}.
If a \code{NULL} value is passed, only point estimates are printed regardless of whether Monte Carlo standard errors were computed or not.
Defaults to \code{TRUE}.}

\item{...}{Ignored.}
}
\description{
Print method for \code{summary.multisimsum} objects
}
\examples{
data(frailty)
ms <- multisimsum(
  data = frailty, par = "par", true = c(
    trt = -0.50,
    fv = 0.75
  ), estvarname = "b", se = "se", methodvar = "model",
  by = "fv_dist"
)
sms <- summary(ms, stats = c("bias", "cover", "mse"))
sms

# Printing less significant digits:
print(sms, digits = 3)

# Printing confidence intervals:
print(sms, digits = 3, mcse = FALSE)

# Printing values only:
print(sms, mcse = NULL)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{frailty}
\alias{frailty}
\alias{frailty2}
\title{Example of a simulation study on frailty survival models}
\format{
A data frame with 16,000 rows and 6 variables:
\itemize{
\item \code{i} Simulated dataset number.
\item \code{b} Point estimate.
\item \code{se} Standard error of the point estimate.
\item \code{par} The estimand. \code{trt} is the log-treatment effect, \code{fv} is the variance of the frailty.
\item \code{fv_dist} The true frailty distribution.
\item \code{model} Method used (\verb{Cox, Gamma}, \verb{Cox, Log-Normal}, \verb{RP(P), Gamma}, or \verb{RP(P), Log-Normal}).
}

An object of class \code{data.frame} with 16000 rows and 7 columns.
}
\usage{
frailty

frailty2
}
\description{
A dataset from a simulation study comparing frailty flexible parametric models fitted using penalised likelihood to semiparametric frailty models.
Both models are fitted assuming a Gamma and a log-Normal frailty.
One thousand datasets were simulated, each containing a binary treatment variable with a log-hazard ratio of -0.50. Clustered survival data was simulated assuming 50 clusters of 50 individuals each, with a mixture Weibull baseline hazard function and a frailty following either a Gamma or a Log-Normal distribution.
The comparison involves estimates of the log-treatment effect, and estimates of heterogeneity (i.e. the estimated frailty variance).
}
\note{
\code{frailty2} is a version of the same dataset with the \code{model} column split into two columns, \code{m_baseline} and \code{m_frailty}.
}
\examples{
data("frailty", package = "rsimsum")
data("frailty2", package = "rsimsum")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rsimsum-package.R
\docType{package}
\name{rsimsum-package}
\alias{rsimsum}
\alias{rsimsum-package}
\title{rsimsum: Analysis of Simulation Studies Including Monte Carlo Error}
\description{
Summarise results from simulation studies and compute Monte Carlo standard errors of commonly used summary statistics. This package is modelled on the 'simsum' user-written command in 'Stata' (White I.R., 2010 \url{https://www.stata-journal.com/article.html?article=st0200}), further extending it with additional performance measures and functionality.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://ellessenne.github.io/rsimsum/}
  \item \url{https://github.com/ellessenne/rsimsum}
  \item Report bugs at \url{https://github.com/ellessenne/rsimsum/issues}
}

}
\author{
\strong{Maintainer}: Alessandro Gasparini \email{alessandro@ellessenne.xyz} (\href{https://orcid.org/0000-0002-8319-7624}{ORCID})

Authors:
\itemize{
  \item Ian R. White
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.multisimsum.R
\name{summary.multisimsum}
\alias{summary.multisimsum}
\title{Summarising multisimsum objects}
\usage{
\method{summary}{multisimsum}(object, ci_level = 0.95, df = NULL, stats = NULL, ...)
}
\arguments{
\item{object}{An object of class \code{multisimsum}.}

\item{ci_level}{Significance level for confidence intervals based on Monte Carlo standard errors. Ignored if a \code{multisimsum} object with control parameter \code{mcse = FALSE} is passed.}

\item{df}{Degrees of freedom of a t distribution that will be used to calculate confidence intervals based on Monte Carlo standard errors.
If \code{NULL} (the default), quantiles of a Normal distribution will be used instead.}

\item{stats}{Summary statistics to include; can be a scalar value or a vector (for multiple summary statistics at once).
Possible choices are:
\itemize{
\item \code{nsim}, the number of replications with non-missing point estimates and standard error.
\item \code{thetamean}, average point estimate.
\item \code{thetamedian}, median point estimate.
\item \code{se2mean}, average standard error.
\item \code{se2median}, median standard error.
\item \code{bias}, bias in point estimate.
\item \code{rbias}, relative (to the true value) bias in point estimate.
\item \code{empse}, empirical standard error.
\item \code{mse}, mean squared error.
\item \code{relprec}, percentage gain in precision relative to the reference method.
\item \code{modelse}, model-based standard error.
\item \code{relerror}, relative percentage error in standard error.
\item \code{cover}, coverage of a nominal \code{level}\\% confidence interval.
\item \code{becover}, bias corrected coverage of a nominal \code{level}\\% confidence interval.
\item \code{power}, power of a (1 - \code{level})\\% level test.
Defaults to \code{NULL}, in which case all possible summary statistics are included.
}}

\item{...}{Ignored.}
}
\value{
An object of class \code{summary.multisimsum}.
}
\description{
The \code{summary()} method for objects of class \code{multisimsum} returns confidence intervals for performance measures based on Monte Carlo standard errors.
}
\examples{
data(frailty)
ms <- multisimsum(
  data = frailty, par = "par", true = c(
    trt = -0.50,
    fv = 0.75
  ), estvarname = "b", se = "se", methodvar = "model",
  by = "fv_dist"
)
sms <- summary(ms)
sms
}
\seealso{
\code{\link[=multisimsum]{multisimsum()}}, \code{\link[=print.summary.multisimsum]{print.summary.multisimsum()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{relhaz}
\alias{relhaz}
\title{Example of a simulation study on survival modelling}
\format{
A data frame with 1,200 rows and 6 variables:
\itemize{
\item \code{dataset} Simulated dataset number.
\item \code{n} Sample size of the simulate dataset.
\item \code{baseline} Baseline hazard function of the simulated dataset.
\item \code{model} Method used (\code{Cox}, \code{Exp}, or \code{RP(2)}).
\item \code{theta} Point estimate for the log-hazard ratio.
\item \code{se} Standard error of the point estimate.
}
}
\usage{
relhaz
}
\description{
A dataset from a simulation study assessing the impact of misspecifying the baseline hazard in survival models on regression coefficients.
One thousand datasets were simulated, each containing a binary treatment variable with a log-hazard ratio of -0.50.
Survival data was simulated for two different sample sizes, 50 and 250 individuals, and under two different baseline hazard functions, exponential and Weibull.
Consequently, a Cox model (Cox, 1972), a fully parametric exponential model, and a Royston-Parmar (Royston and Parmar, 2002) model with two degrees of freedom were fit to each simulated dataset.
See \code{vignette("B-relhaz", package = "rsimsum")} for more information.
}
\examples{
data("relhaz", package = "rsimsum")
}
\references{
Cox D.R. 1972. Regression models and life-tables. Journal of the Royal Statistical Society, Series B (Methodological) 34(2):187-220. \doi{10.1007/978-1-4612-4380-9_37}

Royston, P. and Parmar, M.K. 2002. Flexible parametric proportional-hazards and proportional-odds models for censored survival data, with application to prognostic modelling and estimation of treatment effects. Statistics in Medicine 21(15):2175-2197 \doi{10.1002/sim.1203}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.simsum.R
\name{print.simsum}
\alias{print.simsum}
\title{print.simsum}
\usage{
\method{print}{simsum}(x, ...)
}
\arguments{
\item{x}{An object of class \code{simsum}.}

\item{...}{Ignored.}
}
\description{
Print method for simsum objects
}
\examples{
data("MIsim")
x <- simsum(
  data = MIsim, estvarname = "b", true = 0.5, se = "se",
  methodvar = "method"
)
x

MIsim$true <- 0.5
x <- simsum(data = MIsim, estvarname = "b", true = "true", se = "se")
x
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy.R
\name{tidy.simsum}
\alias{tidy.simsum}
\alias{tidy.summary.simsum}
\alias{tidy.multisimsum}
\alias{tidy.summary.multisimsum}
\title{Turn an object into a tidy dataset}
\usage{
\method{tidy}{simsum}(x, stats = NULL, ...)

\method{tidy}{summary.simsum}(x, stats = NULL, ...)

\method{tidy}{multisimsum}(x, stats = NULL, ...)

\method{tidy}{summary.multisimsum}(x, stats = NULL, ...)
}
\arguments{
\item{x}{An object of class \code{simsum}.}

\item{stats}{Summary statistics to include; can be a scalar value or a vector.
Possible choices are:
\itemize{
\item \code{nsim}, the number of replications with non-missing point estimates and standard error.
\item \code{thetamean}, average point estimate.
\item \code{thetamedian}, median point estimate.
\item \code{se2mean}, average standard error.
\item \code{se2median}, median standard error.
\item \code{bias}, bias in point estimate.
\item \code{rbias}, relative (to the true value) bias in point estimate.
\item \code{empse}, empirical standard error.
\item \code{mse}, mean squared error.
\item \code{relprec}, percentage gain in precision relative to the reference method.
\item \code{modelse}, model-based standard error.
\item \code{relerror}, relative percentage error in standard error.
\item \code{cover}, coverage of a nominal \code{level}\\% confidence interval.
\item \code{becover}, bias-eliminated coverage of a nominal \code{level}\\% confidence interval.
\item \code{power}, power of a (1 - \code{level})\\% level test.
Defaults to \code{NULL}, in which case all summary statistics are returned.
}}

\item{...}{Ignored.}
}
\value{
A \code{data.frame} containing summary statistics from a simulation study.
}
\description{
Extract a tidy dataset with results from an object of class \code{simsum}, \code{summary.simsum}, \code{multisimsum}, or \code{summary.multisimsum}.
}
\examples{
data(MIsim)
x <- simsum(
  data = MIsim, estvarname = "b", true = 0.5, se = "se",
  methodvar = "method"
)
tidy(x)

# Extracting only bias and coverage:
tidy(x, stats = c("bias", "cover"))

xs <- summary(x)
tidy(xs)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simsum.R
\name{simsum}
\alias{simsum}
\title{Analyses of simulation studies including Monte Carlo error}
\usage{
simsum(
  data,
  estvarname,
  se = NULL,
  true = NULL,
  methodvar = NULL,
  ref = NULL,
  by = NULL,
  ci.limits = NULL,
  df = NULL,
  dropbig = FALSE,
  x = FALSE,
  control = list()
)
}
\arguments{
\item{data}{A \code{data.frame} in which variable names are interpreted.
It has to be in tidy format, e.g. each variable forms a column and each observation forms a row.}

\item{estvarname}{The name of the variable containing the point estimates.
Note that some column names are forbidden: these are listed below in the \emph{Details} section.}

\item{se}{The name of the variable containing the standard errors of the point estimates.
Note that some column names are forbidden: these are listed below in the \emph{Details} section.}

\item{true}{The true value of the parameter; this is used in calculations of bias, relative bias, coverage, and mean squared error and is required whenever these performance measures are requested.
\code{true} can be a numeric value or a string that identifies a column in \code{data}.
In the former setting, \code{simsum} will assume the same value for all replications; conversely, each replication will use a distinct value for \code{true} as identified by each row of \code{data}.
See \code{vignette("E-custom-inputs", package = "rsimsum")} for more details.
Note that some column names are forbidden: these are listed below in the \emph{Details} section.}

\item{methodvar}{The name of the variable containing the methods to compare.
For instance, methods could be the models compared within a simulation study.
Can be \code{NULL}.
If a vector of column names is passed to \code{simsum()}, those columns will be combined into a single column named \verb{:methodvar} using the \code{\link[base:interaction]{base::interaction()}} function before computing all performance measures.
Note that some column names are forbidden: these are listed below in the \emph{Details} section.}

\item{ref}{Specifies the reference method against which relative precision will be calculated.
Only useful if \code{methodvar} is specified.}

\item{by}{A vector of variable names to compute performance measures by a list of factors. Factors listed here are the (potentially several) data-generating mechanisms used to simulate data under different scenarios (e.g. sample size, true distribution of a variable, etc.).
Can be \code{NULL}.
Note that some column names are forbidden: these are listed below in the \emph{Details} section.}

\item{ci.limits}{Can be used to specify the limits (lower and upper) of confidence intervals used to calculate coverage and bias-eliminated coverage.
Useful for non-Wald type estimators (e.g. bootstrap).
Defaults to \code{NULL}, where Wald-type confidence intervals based on the provided SEs are calculated for coverage; otherwise, it can be a numeric vector (for fixed confidence intervals) or a vector of strings that identify columns in \code{data} with replication-specific lower and upper limits.
See \code{vignette("E-custom-inputs", package = "rsimsum")} for more details.
Note that some column names are forbidden: these are listed below in the \emph{Details} section.}

\item{df}{Can be used to specify that a column containing the replication-specific number of degrees of freedom that will be used to calculate confidence intervals for coverage (and bias-eliminated coverage) assuming t-distributed critical values (rather than normal theory intervals).
See \code{vignette("E-custom-inputs", package = "rsimsum")} for more details.
Note that some column names are forbidden: these are listed below in the \emph{Details} section.}

\item{dropbig}{Specifies that point estimates or standard errors beyond the maximum acceptable values should be dropped. Defaults to \code{FALSE}.}

\item{x}{Set to \code{TRUE} to include the \code{data} argument used to calculate summary statistics (i.e. after pre-processing the input dataset e.g. removing values deemed too large via the \code{dropbig} argument) as a slot. Calling \code{simsum} with \code{x = TRUE} is required to produce zipper plots. The downside is that the size of the returned object increases considerably, therefore it is set to \code{FALSE} by default.}

\item{control}{A list of parameters that control the behaviour of \code{simsum}.
Possible values are:
\itemize{
\item \code{mcse}, whether to calculate Monte Carlo standard errors. Defaults to \code{TRUE};
\item \code{level}, the significance level used for coverage, bias-eliminated coverage, and power. Defaults to 0.95;
\item \code{power_df}, whether to use robust critical values from a t distribution with \code{power_df} degrees of freedom when calculating power. Defaults to \code{NULL}, in which case a Gaussian distribution is used;
\item \code{na.rm}, whether to remove point estimates or standard errors where either (or both) is missing. Defaults to \code{TRUE};
\item \code{char.sep}, a character utilised when splitting the input dataset \code{data}. Generally, this should not be changed;
\item \code{dropbig.max}, specifies the maximum acceptable absolute value of the point estimates, after standardisation. Defaults to 10;
\item \code{dropbig.semax}, specifies the maximum acceptable absolute value of the standard error, after standardisation. Defaults to 100
\item \code{dropbig.robust}, specifies whether to use robust standardisation (using median and inter-quartile range) rather than normal standardisation (using mean and standard deviation). Defaults to \code{TRUE}, in which case robust standardisation will be used for \code{dropbig}.
}}
}
\value{
An object of class \code{simsum}.
}
\description{
\code{simsum()} computes performance measures for simulation studies in which each simulated data set yields point estimates by one or more analysis methods.
Bias, relative bias, empirical standard error and precision relative to a reference method can be computed for each method.
If, in addition, model-based standard errors are available then \code{simsum()} can compute the average model-based standard error, the relative error in the model-based standard error, the coverage of nominal confidence intervals, the coverage under the assumption that there is no bias (bias-eliminated coverage), and the power to reject a null hypothesis.
Monte Carlo errors are available for all estimated quantities.
}
\details{
The following names are not allowed for any column in \code{data} that is passed to \code{\link[=simsum]{simsum()}}: \code{stat}, \code{est}, \code{mcse}, \code{lower}, \code{upper}, \verb{:methodvar}, \verb{:true}.
}
\examples{
data("MIsim", package = "rsimsum")
s <- simsum(data = MIsim, estvarname = "b", true = 0.5, se = "se", methodvar = "method", ref = "CC")
# If 'ref' is not specified, the reference method is inferred
s <- simsum(data = MIsim, estvarname = "b", true = 0.5, se = "se", methodvar = "method")
}
\references{
White, I.R. 2010. simsum: Analyses of simulation studies including Monte Carlo error. The Stata Journal 10(3): 369-385. \url{https://www.stata-journal.com/article.html?article=st0200}

Morris, T.P., White, I.R. and Crowther, M.J. 2019. \emph{Using simulation studies to evaluate statistical methods}. Statistics in Medicine, \doi{10.1002/sim.8086}

Gasparini, A. 2018. rsimsum: Summarise results from Monte Carlo simulation studies. Journal of Open Source Software 3(26):739, \doi{10.21105/joss.00739}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kable.R
\name{kable.simsum}
\alias{kable.simsum}
\alias{kable.summary.simsum}
\alias{kable.multisimsum}
\alias{kable.summary.multisimsum}
\alias{kable}
\title{Create 'kable's}
\usage{
\method{kable}{simsum}(x, stats = NULL, digits = max(3, getOption("digits") - 3), ...)

\method{kable}{summary.simsum}(x, stats = NULL, digits = max(3, getOption("digits") - 3), ...)

\method{kable}{multisimsum}(x, stats = NULL, digits = max(3, getOption("digits") - 3), ...)

\method{kable}{summary.multisimsum}(x, stats = NULL, digits = max(3, getOption("digits") - 3), ...)

kable(x, ...)
}
\arguments{
\item{x}{An object of class \code{simsum}, \code{summary.simsum}, \code{multisimsum}, \code{summary.multisimsum};}

\item{stats}{Summary statistics to include. See \code{\link[=tidy]{tidy()}} for more details;}

\item{digits}{Maximum number of digits for numeric columns;}

\item{...}{Further arguments passed to \code{\link[knitr:kable]{knitr::kable()}}.}
}
\description{
Create tables in LaTeX, HTML, Markdown, or reStructuredText from objects of class \code{simsum}, \code{summary.simsum}, \code{multisimsum}, \code{summary.multisimsum}.
}
\seealso{
\code{\link[knitr:kable]{knitr::kable()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dropbig.R
\name{dropbig}
\alias{dropbig}
\title{Identify replications with large point estimates, standard errors}
\usage{
dropbig(
  data,
  estvarname,
  se = NULL,
  methodvar = NULL,
  by = NULL,
  max = 10,
  semax = 100,
  robust = TRUE
)
}
\arguments{
\item{data}{A \code{data.frame} in which variable names are interpreted. It has to be in tidy format, e.g. each variable forms a column and each observation forms a row.}

\item{estvarname}{The name of the variable containing the point estimates.}

\item{se}{The name of the variable containing the standard errors of the point estimates.}

\item{methodvar}{The name of the variable containing the methods to compare. For instance, methods could be the models compared within a simulation study. Can be \code{NULL}.}

\item{by}{A vector of variable names to compute performance measures by a list of factors. Factors listed here are the (potentially several) data-generating mechanisms used to simulate data under different scenarios (e.g. sample size, true distribution of a variable, etc.). Can be \code{NULL}.}

\item{max}{Specifies the maximum acceptable absolute value of the point estimates, after standardisation. Defaults to 10.}

\item{semax}{Specifies the maximum acceptable absolute value of the standard error, after standardisation. Defaults to 100.}

\item{robust}{Specifies whether to use robust standardisation (using median and inter-quartile range) rather than normal standardisation (using mean and standard deviation). Defaults to \code{TRUE}.}
}
\value{
The same \code{data.frame} given as input with an additional column named \code{.dropbig} identifying rows that are classified as large (\code{.dropbig = TRUE}) according to the specified criterion.
}
\description{
\code{dropbig} is useful to identify replications with large point estimates or standard errors. Large values are defined as standardised values above a given threshold, as defined when calling \code{dropbig}. Regular standardisation using mean and standard deviation is implemented, as well as robust standardisation using median and inter-quartile range. Further to that, the  standardisation process is stratified by data-generating mechanism if \code{by} factors are defined.
}
\examples{
data("frailty", package = "rsimsum")
frailty2 <- subset(frailty, par == "fv")

# Using low values of max, semax for illustration purposes:
dropbig(
  data = frailty2, estvarname = "b", se = "se",
  methodvar = "model", by = "fv_dist", max = 2, semax = 2
)

# Using regular standardisation:
dropbig(
  data = frailty2, estvarname = "b", se = "se",
  methodvar = "model", by = "fv_dist", max = 2, semax = 2, robust = FALSE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{MIsim}
\alias{MIsim}
\alias{MIsim2}
\title{Example of a simulation study on missing data}
\format{
A data frame with 3,000 rows and 4 variables:
\itemize{
\item \code{dataset} Simulated dataset number.
\item \code{method} Method used (\code{CC}, \code{MI_LOGT} or \code{MI_T}).
\item \code{b} Point estimate.
\item \code{se} Standard error of the point estimate.
}

An object of class \code{tbl_df} (inherits from \code{tbl}, \code{data.frame}) with 3000 rows and 5 columns.
}
\usage{
MIsim

MIsim2
}
\description{
A dataset from a simulation study comparing different ways to handle missing covariates when fitting a Cox model (White and Royston, 2009).
One thousand datasets were simulated, each containing normally distributed covariates \eqn{x} and \eqn{z} and time-to-event outcome.
Both covariates have 20\% of their values deleted independently of all other variables so the data became missing completely at random (Little and Rubin, 2002).
Each simulated dataset was analysed in three ways.
A Cox model was fit to the complete cases (\code{CC}).
Then two methods of multiple imputation using chained equations (van Buuren, Boshuizen, and Knook, 1999) were used.
The \code{MI_LOGT} method multiply imputes the missing values of \eqn{x} and \eqn{z} with the outcome included as \eqn{\log (t)} and \eqn{d}, where \eqn{t} is the survival time and \eqn{d} is the event indicator.
The \code{MI_T} method is the same except that \eqn{\log (t)} is replaced by \eqn{t} in the imputation model.
The results are stored in long format.
}
\note{
\code{MIsim2} is a version of the same dataset with the \code{method} column split into two columns, \code{m1} and \code{m2}.
}
\examples{
data("MIsim", package = "rsimsum")
data("MIsim2", package = "rsimsum")
}
\references{
White, I.R., and P. Royston. 2009. Imputing missing covariate values for the Cox model. Statistics in Medicine 28(15):1982-1998 \doi{10.1002/sim.3618}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.R
\name{is.summary.simsum}
\alias{is.summary.simsum}
\title{is.summary.simsum}
\usage{
is.summary.simsum(x)
}
\arguments{
\item{x}{An object to test.}
}
\description{
Reports whether x is a summary.simsum object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tt}
\alias{tt}
\title{Example of a simulation study on the t-test}
\format{
A data frame with 4,000 rows and 8 variables:
\itemize{
\item \code{diff} The difference in mean between groups estimated by the t-test;
\item \code{se} Standard error of the estimated difference;
\item \code{conf.low}, \code{conf.high} Confidence interval for the difference in mean as reported by the t-test;
\item \code{df} The number of degrees of freedom assumed by the t-test;
\item \code{repno} Identifies each replication, between 1 and 500;
\item \code{dgm} Identifies each data-generating mechanism: 1 corresponds to normal data with equal variance between the groups, 2 is normal data with unequal variance, 3 and 4 are skewed data (simulated from a Gamma distribution) with equal and unequal variance between groups, respectively;
\item \code{method} Analysis method: 1 represents the t-test with pooled variance, while 2 represents the t-test with unpooled variance.
}
}
\usage{
tt
}
\description{
A dataset from a simulation study with 4 data-generating mechanisms, useful to illustrate custom input of confidence intervals to calculate coverage probability.
This simulation study aims to compare the t-test assuming pooled or unpooled variance in violation (or not) of the t-test assumptions: normality of data, and equality (or not) or variance between groups.
The true value of the difference between groups is -1.
}
\note{
Further details on this simulation study can be found in the R script used to generate this dataset, available on GitHub: \url{https://github.com/ellessenne/rsimsum/blob/master/data-raw/tt-data.R}
}
\examples{
data("tt", package = "rsimsum")
}
\keyword{datasets}
